// DesktopFrame.java

package org.javalobby.cafebabe;

import java.io.*;
import java.beans.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;

import org.javalobby.classfile.*;
import org.javalobby.mdi.*;
import org.javalobby.util.*;

// Main frame for CafeBabe application

public abstract class DesktopFrame extends JFrame {

  protected JDesktopPane desktopPane;

  protected JInternalFrame currFrame;

  protected MdiMenuBar menuBar;
  protected JToolBar toolBar;

  public DesktopFrame() {
    this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

    menuBar = new MdiMenuBar();
    toolBar = new JToolBar();

    this.setJMenuBar(menuBar);

    desktopPane = new MdiDesktopPane(menuBar);

    desktopPane.putClientProperty("JDesktopPane.dragMode", "outline");

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());

    topPanel.add(toolBar, BorderLayout.NORTH);
    topPanel.add(desktopPane, BorderLayout.CENTER);

    this.setContentPane(topPanel);

    Component glassPane = getGlassPane();

    glassPane.setCursor(new Cursor(Frame.WAIT_CURSOR));

    glassPane.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {}
    });
  }

  public void setCurrentFrame(JInternalFrame currFrame) {
    this.currFrame = currFrame;
  }

  public JInternalFrame getCurrentFrame() {
    return currFrame;
  }

  public JToolBar getJToolBar() {
    return toolBar;
  }

  public MdiMenuBar getMdiMenuBar() {
    return menuBar;
  }

  public boolean canExit() {
    JInternalFrame[] frames = desktopPane.getAllFrames();

    boolean canExit = true;

    for(int i=0; i < frames.length; i++) {
      MdiFrame frame = (MdiFrame)frames[i];
      if(!frame.canExit()) {
        canExit = false;
        break;
      }
    }

    return canExit;
  }

  public abstract void newFrame();

  public abstract void open(String fileName);

  public abstract void openPlugin();

  public abstract void saveAs();

  public abstract void close();

  public abstract void changePlaf();

  public abstract void mail();

  public abstract void about();

  public void exit() {
    WindowEvent evt = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
    processWindowEvent(evt);
  }

  public abstract void load();

  public abstract void save();

  public abstract void invokePlugin(Class pluginClass, File file);

  protected void processWindowEvent(WindowEvent e) {
    if(e.getID() == WindowEvent.WINDOW_CLOSING) {
      if(canExit()) {
        save();
        super.processWindowEvent(e);
        System.exit(0);
      }
    }
    else {
      super.processWindowEvent(e);
    }
  }

}
