// PlainActions.java

package org.javalobby.cafebabe;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import org.javalobby.util.*;

public class PlainActions extends Actions implements Constants {

  private Action collapceAction, expandFirstLevelAction,
                 expandAllLevelsAction, searchAction, tutorialAction;

  protected JToolBar toolBar = new JToolBar();

  private ExtensionFrame parent;

  public PlainActions(ExtensionFrame parent) {
    this.parent = parent;

    init();
  }

  public JToolBar getToolBar() {
    return toolBar;
  }

  protected void init() {
    collapceAction         = createCollapceAction("Collapse tree",
                               IconProducer.getImageIcon(ICON_COLLAPSE));
    expandFirstLevelAction = createExpandFirstLevelAction("Expand first level",
                               IconProducer.getImageIcon(ICON_EXPAND));
    expandAllLevelsAction  = createExpandAllLevelsAction("Expand all levels",
                               IconProducer.getImageIcon(ICON_EXPAND_ALL));
    searchAction           = createSearchAction("Search string",
                               IconProducer.getImageIcon(ICON_SEARCH));
    AbstractButton tutoButton = new JToggleButton();
    tutoButton.setSelectedIcon(IconProducer.getImageIcon(ICON_TUTORIAL_ON));
    tutorialAction         = createTutorialAction("Tutorial mode",
                               IconProducer.getImageIcon(ICON_TUTORIAL_OFF),
                               tutoButton);

    addActionToToolBar(collapceAction, toolBar, false, "Collapse tree", 0);
    addActionToToolBar(expandFirstLevelAction, toolBar, false,
                    "Expand first level", 0);
    addActionToToolBar(expandAllLevelsAction, toolBar, false,
                    "Expand all levels", 0);
    addActionToToolBar(searchAction, toolBar, false, "Search string", 0);

    JPanel emptyArea = new JPanel();
    toolBar.add(emptyArea);

    addActionToToolBar(tutorialAction, toolBar, false, "Tutorial mode", 0);
  }


  public Action createCollapceAction(String text, ImageIcon icon) {
    return new AbstractAction(text, icon) {
      public void actionPerformed(ActionEvent e) {
        parent.collapse();
      }
    };
  }

  public Action createExpandFirstLevelAction(String text, ImageIcon icon) {
    return new AbstractAction(text, icon) {
      public void actionPerformed(ActionEvent e) {
        parent.expandFirstLevel();
      }
    };
  }

  public Action createExpandAllLevelsAction(String text, ImageIcon icon) {
    return new AbstractAction(text, icon) {
      public void actionPerformed(ActionEvent e) {
        parent.expandAllLevels();
      }
    };
  }

  public Action createSearchAction(String text, ImageIcon icon) {
    return new AbstractAction(text, icon) {
      public void actionPerformed(ActionEvent e) {
        parent.search();
      }
    };
  }

  public Action createTutorialAction(String text, ImageIcon icon,
                                     AbstractButton button) {
    return new CustomAction(text, icon, button) {
      boolean isTutorialMode = false;
      public void actionPerformed(ActionEvent e) {
        parent.setTutorialMode(isTutorialMode = !isTutorialMode);
      }
    };
  }

}
