// Plugins.java

package org.javalobby.cafebabe;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.zip.*;

import javax.swing.*;

import org.javalobby.util.*;

public final class Plugins implements Constants {

  public static String MANIFEST_ENTRY_NAME = "META-INF/MANIFEST.MF";
  public static String MAIN_CLASS_TEXT     = "Main-Class";
  public static String CLASS_PATH_TEXT     = "Class-Path";
  public static String EXTENSION_TEXT      = "Extension";
  public static String ACTION_TEXT         = "Action";
  public static String DESCRIPTION_TEXT    = "Description";
  public static String ICON_NAME_TEXT      = "Icon";

  private Vector pluginsList = new Vector();

  private static Plugins instance;

  private Plugins() {}

  public static Plugins getInstance() {
    if(instance == null) {
      instance = new Plugins();
    }

    return instance;
  }

  public void read() {
    File pluginsDir = new File(PLUGINS_DIR_NAME);

    if(!pluginsDir.exists()) {
      return;
    }

    try {
      String[] pluginNames = pluginsDir.list();

      for(int i=0; i < pluginNames.length; i++) {
        String ext = FileUtil.getExtension(pluginNames[i]);

        if(ext != null && ext.equalsIgnoreCase("cba")) {
          Plugin plugin = readPlugin(PLUGINS_DIR_NAME + "/" + pluginNames[i]);

          if(plugin != null) {
            pluginsList.addElement(plugin);
          }
        }
      }
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }

  public Plugin readPlugin(String pluginName) throws IOException {
    ZipFile zipFile = new ZipFile(pluginName);

    Plugin plugin = null;

    Enumeration entries = zipFile.entries();
    while(entries.hasMoreElements()) {
      ZipEntry zipEntry = (ZipEntry)entries.nextElement();
      if(zipEntry.getName().equals(MANIFEST_ENTRY_NAME)) {
        plugin = register(zipFile, zipEntry);

        break;
      }
    }

    zipFile.close();

    return plugin;
  }

  private Plugin register(ZipFile zipFile, ZipEntry manifestEntry) throws IOException {
    InputStream is = zipFile.getInputStream(manifestEntry);
    BufferedReader reader = new BufferedReader(new InputStreamReader(is));

    String mainClassName = null;
    String classPath     = null;
    String extension     = null;
    String action        = null;
    String description   = null;
    String iconName      = null;

    while(true) {
      String line = reader.readLine();
      if(line == null) {
        break;
      }

      if(line.startsWith(MAIN_CLASS_TEXT)) {
        mainClassName = line.substring(line.indexOf(":")+1).trim();
      }
      else if(line.startsWith(CLASS_PATH_TEXT)) {
        classPath = line.substring(line.indexOf(":")+1).trim();
      }
      else if(line.startsWith(EXTENSION_TEXT)) {
        extension = line.substring(line.indexOf(":")+1).trim();
      }
      else if(line.startsWith(ACTION_TEXT)) {
        action = line.substring(line.indexOf(":")+1).trim();
      }
      else if(line.startsWith(DESCRIPTION_TEXT)) {
        description = line.substring(line.indexOf(":")+1).trim();
      }
      else if(line.startsWith(ICON_NAME_TEXT)) {
        iconName = line.substring(line.indexOf(":")+1).trim();
      }
    }

    Plugin plugin = null;

    try {
      if(action != null) { // menu plugin
        Class clazz = getPluginClass(zipFile, mainClassName, classPath);
        if(clazz != null) {
          ImageIcon icon = IconProducer.getImageIcon(iconName, clazz);

          plugin = new MenuPlugin(clazz, classPath, description, action, icon);
        }
      }
      else if(extension != null) { // extension plugin
        Class clazz = getPluginClass(zipFile, mainClassName, classPath);

        if(clazz != null) {
          clazz.newInstance();
          plugin = new ExtensionPlugin(clazz, classPath, description, extension);
        }
      }
    }
    catch(Throwable e) {
      e.printStackTrace();
    }

    return plugin;
  }

  private Class getPluginClass(ZipFile zipFile, String className, String classPath)
                       throws ClassNotFoundException {
    if(className != null) {
      Vector cpEntries = new Vector();

      if(classPath != null) {
        StringTokenizer st = new StringTokenizer(classPath, "\n\t\t ,");
        while(st.hasMoreTokens()) {
          cpEntries.addElement(st.nextToken());
        }
      }

      ZipClassLoader loader = new ZipClassLoader(zipFile.getName(), cpEntries);

      return loader.loadClass(className, true);
    }

    return null;
  }

  public Vector getPluginsList() {
    return pluginsList;
  }

  public Class getExtensionPluginClass(String extension) {
    for(int i=0; i < pluginsList.size(); i++) {
      Plugin plugin = (Plugin)pluginsList.elementAt(i);
      if(plugin instanceof ExtensionPlugin) {
        ExtensionPlugin extensionPlugin = (ExtensionPlugin)plugin;
        if(extensionPlugin.getExtension().equalsIgnoreCase(extension)) {
          return plugin.getMainClass();
        }
      }
    }

    return null;
  }

}
