// ProgramFrame.java

package org.javalobby.cafebabe;

import java.io.*;
import java.beans.*;
import java.util.*;
import java.util.zip.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.plaf.metal.*;
import javax.activation.*;

import org.javalobby.classfile.*;
import org.javalobby.mdi.*;
import org.javalobby.util.*;

import org.javalobby.cafebabe.archooser.*;
import org.javalobby.cafebabe.activation.*;

// Main frame for CafeBabe application

public final class ProgramFrame extends DesktopFrame implements Constants {

  private final Integer PROGRAM_LAYER = JLayeredPane.PALETTE_LAYER;

  static {
    Plugins.getInstance().read();
  }

  private ProgramActions actions;

  private InternalFrameListener internalFrameListener = new InternalFrameAdapter() {

    public void internalFrameActivated(InternalFrameEvent e) {
      setCurrentFrame((MdiFrame)e.getSource());
      actions.enable();
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
      setCurrentFrame((MdiFrame)e.getSource());
      actions.disable();
    }

    public void internalFrameClosing(InternalFrameEvent e) {}

    public void internalFrameOpened(InternalFrameEvent e) {}

    public void internalFrameClosed(InternalFrameEvent e) {
      setCurrentFrame(null);
    }

    public void internalFrameIconified(InternalFrameEvent e) {}

    public void internalFrameDeiconified(InternalFrameEvent e) {}
  };

  private ProgramState programState;

  private ProgramChooser chooser = ProgramChooser.getInstance();

  public ProgramFrame() {
    super();

    setTitle(MAIN_FRAME_TITLE);

    chooser.setProgramFrame(this);

    this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

    ImageIcon icon = IconProducer.getImageIcon(ICON_MAIN_FRAME);
    if(icon != null) {
      this.setIconImage(icon.getImage());
    }

    try {
      MimetypesFileTypeMap fileTypesMap = new MimetypesFileTypeMap(MIMETYPES_FILE);

      FileTypeMap.setDefaultFileTypeMap(fileTypesMap);

      MailcapCommandMap cmdMap = (MailcapCommandMap)CommandMap.getDefaultCommandMap();

      BufferedReader reader = new BufferedReader(new FileReader(MAILCAP_FILE));
      while(true) {
        String line = reader.readLine();
        if(line == null)
          break;

        cmdMap.addMailcap(line + "\n");	
      }
    }
    catch(IOException e) {
      JOptionPane.showMessageDialog((Component)this,
                  e.toString(), "Warning", JOptionPane.WARNING_MESSAGE);
    }

    actions = new ProgramActions(this);
    load();

    this.setVisible(true);
  }

  public void newFrame() {}

  public void open(String fileName) {
    File file = null;

    if(fileName == null) {
      file = chooser.openChooser("Open file");
    }
    else {
      file = new File(fileName);
    }

    if(file != null && !file.isDirectory()) {
      invokePlugin(file);
    }
  }

  public void open(File file) {
    invokePlugin(file);
  }

  public void saveAs() {
    if(currFrame instanceof ExtensionFrame) {
      int returnVal = chooser.showDialog(this, "Save file as");
      if(returnVal != JFileChooser.APPROVE_OPTION)
        return;

      File file = chooser.getSelectedFile();

      if(file != null) {
        ((ExtensionFrame)currFrame).saveAs(file.getPath());
      }
    }
  }

  public void close() {
    try {
      currFrame.setClosed(true);
      currFrame = null;
      actions.disable();
    }
    catch(PropertyVetoException e) {}
  }

  public void mail() {
    new MailFrame(desktopPane, PROGRAM_LAYER);
  }

  public void about() {
    new AboutFrame(desktopPane, PROGRAM_LAYER);
  }

  public void openPlugin() {
    File file = chooser.openChooser("Open file");

    if(file != null && !file.isDirectory()) {
      invokePlugin(file);
    }
  }

  public void invokePlugin(File file) {
    String fileName = file.getPath().toLowerCase();

    if(fileName.endsWith(".zip") || fileName.endsWith(".jar")) {
      JOptionPane.showMessageDialog((Component)this,
            "You should select file (not archive).",
             "Warning", JOptionPane.WARNING_MESSAGE);
      return;
    }

    DataSource dataSource = null;

    if(file instanceof ArchivedFile) {
      ArchivedFile archivedFile = (ArchivedFile)file;
      ZipFile zipFile = archivedFile.getZipFile();

      dataSource = new ArchivedDataSource(zipFile, archivedFile.getEntryName());
    }
    else {
      dataSource = new FileDataSource(file);
    }

    Plugins plugins    = Plugins.getInstance();
    Vector pluginsList = plugins.getPluginsList();

    DataHandler dataHandler = new DataHandler(dataSource);

    if(dataHandler.getContentType().equals("application/cafe-babe-archive")) {
      try {
        Plugin plugin = plugins.readPlugin(fileName);

        if(plugin != null) {
          boolean isExists = false;
          for(int i=0; i < pluginsList.size(); i++) {
            String name = ((Plugin)pluginsList.elementAt(i)).getMainClass().getName();
            if(name.equals(plugin.getMainClass().getName())) {
              isExists = true;
              break;
            }
          }

          if(isExists) {
            JOptionPane.showMessageDialog((Component)this,
                  "Plugin \"" + fileName + "\" is already installed.",
                  "Warning", JOptionPane.WARNING_MESSAGE);
          }
          else {
            pluginsList.addElement(plugin);
  //          invokePlugin(plugin.getMainClass(), null);
            JOptionPane.showMessageDialog((Component)this,
                  "Plugin \"" + fileName + "\" is successfully installed.",
                  "Success", JOptionPane.WARNING_MESSAGE);
          }
        }
      }
      catch(IOException e) {
        e.printStackTrace();
        JOptionPane.showMessageDialog((Component)this,
                    e.toString() + ".", "Warning", JOptionPane.WARNING_MESSAGE);
      }

      return;
    }

    CommandInfo ci = dataHandler.getCommand("view");

    if(ci == null) {
      JOptionPane.showMessageDialog((Component)this,
            "Mime type \"" + dataHandler.getContentType() +
            "\" doesn't have any assosiation.",
            "Warning", JOptionPane.WARNING_MESSAGE);
      return;
    }

    String className = ci.getCommandClass();
    if(className != null) {
      Class clazz = null;

      for(int i=0; i < pluginsList.size(); i++) {
        Plugin plugin = (Plugin)pluginsList.elementAt(i);
        Class mainClass = plugin.getMainClass();

        if(mainClass.getName().equals(className)) {
          clazz = mainClass;
          break;
        }
      }

      if(clazz != null) {
        invokePlugin(clazz, file);
        return;
      }

      try {
        Object o = dataHandler.getBean(ci);

        if(o == null) {
          JOptionPane.showMessageDialog((Component)this,
                "System cannot find a class \"" + className + "\".",
                "Warning", JOptionPane.WARNING_MESSAGE);
        }
        else if(o instanceof Component) {
          DefaultFrame defaultFrame = new DefaultFrame();
          defaultFrame.setTitle(file.getAbsolutePath());
          defaultFrame.setComponent((Component)o);

          open(defaultFrame, file);
        }
        else {
          JOptionPane.showMessageDialog((Component)this,
                "Component \"" + className + "\" should be derived from " +
                "\"Component\" class.",
                "Warning", JOptionPane.WARNING_MESSAGE);
        }
      }
      catch(Exception e) {
        e.printStackTrace();
        JOptionPane.showMessageDialog((Component)this,
                    e.toString() + ".", "Warning", JOptionPane.WARNING_MESSAGE);
      }
    }
  }

  public void invokePlugin(Class pluginClass, File file) {
    try {
      PluginFrame pluginFrame = null;

      if(ExtensionFrame.class.isAssignableFrom(pluginClass)) {
        pluginFrame = (PluginFrame)pluginClass.newInstance();
      }
      else {
        boolean isOpened = false;

        JInternalFrame[] frames = desktopPane.getAllFramesInLayer(PROGRAM_LAYER.intValue());
        for(int i=0; i < frames.length; i++) {
          if(pluginClass.isInstance(frames[i])) {
            pluginFrame = (PluginFrame)frames[i];
          }
        }

        if(pluginFrame == null) {
          pluginFrame = (PluginFrame)pluginClass.newInstance();
        }
      }

      if(pluginFrame != null) {
        open(pluginFrame, file);
      }
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void open(PluginFrame pluginFrame, File file) {
    getGlassPane().setVisible(true);

    if(desktopPane.getIndexOf(pluginFrame) == -1) {
      desktopPane.add(pluginFrame, PROGRAM_LAYER);
      pluginFrame.addInternalFrameListener(internalFrameListener);
    }

    if(currFrame != null) {
      Point point = currFrame.getLocation();
      pluginFrame.setLocation(point.x+25, point.y+25);
    }

    currFrame = pluginFrame;

    pluginFrame.open();

    if(file != null && (pluginFrame instanceof ExtensionFrame)) {
      ((ExtensionFrame)pluginFrame).open(file);
    }

    actions.enable();

    getGlassPane().setVisible(false);
  }

  public void changePlaf() {
    SwingUtilities.updateComponentTreeUI(this);

    if(desktopPane != null) {
      SwingUtilities.updateComponentTreeUI(desktopPane);
    }

    SwingUtilities.updateComponentTreeUI(chooser);
    SwingUtilities.updateComponentTreeUI(menuBar);
    SwingUtilities.updateComponentTreeUI(toolBar);
  }

  public void load() {
    String homeDir = System.getProperty("user.home").replace('\\', '/');
    System.out.println("homeDir " + homeDir);

    String propFileName = PROPERTIES_NAME;

    if(homeDir.length() > 0 && !homeDir.equals("/")) {
      if(homeDir.endsWith("/")) {
        propFileName = homeDir + PROPERTIES_NAME;
      }
      else {
        propFileName = homeDir + "/" + PROPERTIES_NAME;
      }
    }

    programState = new ProgramState(propFileName);

    String className = null;

    programState.load();

    Rectangle bounds = programState.getBoundsProperty("ProgramFrame", "50;50;600;500");
    this.setBounds(bounds);

    actions.load(programState);

    String name2 = (String)programState.get("ProgramChooser");

    if(name2 != null && new File(name2).exists()) {
      chooser.setSelectedFile(new File(name2));
    }

    String name1 = (String)programState.get("DescriptionChooser");

    if(name1 != null && new File(name1).exists()) {
      DescriptionChooser.setCurrentDirectory(new File(name1));
    }

    int index = programState.getIntProperty("chooser.filter.index", 0);

    javax.swing.filechooser.FileFilter[] filters = chooser.getChoosableFileFilters();

    if(index < filters.length) {
      chooser.setFileFilter(filters[index]);
    }
  }

  public void save() {
    programState.load();

    programState.putBoundsProperty("ProgramFrame", this.getBounds());

    actions.save(programState);

    if(chooser.getSelectedFile() != null) {
      programState.put("ProgramChooser", chooser.getSelectedFile().toString());
    }

    if(DescriptionChooser.getCurrentDirectory() != null) {
      programState.put("DescriptionChooser",
                        DescriptionChooser.getCurrentDirectory().toString());
    }

    javax.swing.filechooser.FileFilter currentFilter = chooser.getFileFilter();
    javax.swing.filechooser.FileFilter[] filters = chooser.getChoosableFileFilters();

    int index = 0;
    for(int i=0; i < filters.length; i++) {
      if(filters[i] == currentFilter) {
        index = i;
        break;
      }
    }

    programState.putIntProperty("chooser.filter.index", index);
    programState.save();

    System.out.println(GOODBYE_MESSAGE);
  }

}
