package org.javalobby.cafebabe.activation;

import java.awt.*;
import java.io.*;

import javax.swing.*;
import javax.activation.*;

public class ImageViewer extends JPanel implements CommandObject {
  private ImageViewerCanvas canvas;
  private Image image;
  private DataHandler dataHandler;

  public ImageViewer() {
    canvas = new ImageViewerCanvas();
    add(new JScrollPane(canvas));
  }

  public void setCommandContext(String s, DataHandler datahandler) throws IOException {
    this.dataHandler = datahandler;
    setInputStream(dataHandler.getInputStream());
  }

  private void setInputStream(InputStream inputstream) throws IOException {
    MediaTracker mediatracker = new MediaTracker(this);

    int i = 0;
    byte abyte0[] = new byte[1024];
    ByteArrayOutputStream bytearrayoutputstream = new ByteArrayOutputStream();

    while((i = inputstream.read(abyte0)) > 0) {
      bytearrayoutputstream.write(abyte0, 0, i);
    }

    inputstream.close();

    image = getToolkit().createImage(bytearrayoutputstream.toByteArray());

    mediatracker.addImage(image, 0);
    try {
      mediatracker.waitForID(0);
      mediatracker.waitForAll();
      if(mediatracker.statusID(0, true) != 8) {
        System.out.println("Error occured in image loading = " + mediatracker.getErrorsID(0));
      }
    }
    catch(InterruptedException _ex) {
      throw new IOException("Error reading image data");
    }
    canvas.setImage(image);
  }

  public void addNotify() {
    super.addNotify();

    invalidate();
    validate();
    doLayout();
  }

  public Dimension getPreferredSize() {
    return canvas.getPreferredSize();
  }

}

class ImageViewerCanvas extends JPanel {

  private Image image;

  public void setImage(Image image) {
    this.image = image;

    invalidate();
    repaint();
  }

  public Dimension getPreferredSize() {
    Dimension dimension = null;
    if(image == null) {
      dimension = new Dimension(200, 200);
    }
    else {
      dimension = new Dimension(image.getWidth(this), image.getHeight(this));
    }

    return dimension;
  }

  public void paint(Graphics g) {
    if(image != null) {
      g.drawImage(image, 0, 0, this);
    }
  }

}
