// ArchivedFile.java

package org.javalobby.cafebabe.archooser;

import java.io.*;
import java.util.*;
import java.util.zip.*;

public class ArchivedFile extends File {

  private String entryName;
  private ZipFile zipFile;

  private String zipName;

  public ArchivedFile(String zipName) {
    super(zipName);

    this.zipName = zipName;
    this.entryName = null;
  //  zipFile = new ZipFile(zipName);
  }

  public ArchivedFile(ArchivedFile efile, String entryName) {
    super(efile.getZipFile().getName(), entryName);

    String entryName2 = entryName.replace('\\', '/');

    String path = efile.getPath();
    if(!path.endsWith("/"))
      path += "/";

    String path1 = path.toLowerCase();
    int pos = path1.indexOf(".zip");
    if(pos == -1)
      pos = path1.indexOf(".jar");

    if(pos != -1 && pos < path1.length()-4) {
      int pos1 = (path.length() > pos+4+1) ? pos+4+1 : path.length();

      if(entryName.indexOf("/") == -1)
        entryName2 = path.substring(pos1) + entryName.replace('\\', '/');
    }

    this.zipName   = efile.getZipFile().getName();
    this.entryName = entryName2;
//    zipFile = efile.getZipFile();
  }

  public ArchivedFile(String zipName, String entryName) {
    super(zipName, entryName);

    this.zipName = zipName;
    this.entryName = entryName.replace('\\', '/');
//    zipFile = new ZipFile(zipName);
  }

  public boolean exists() {
    if(entryName == null)
      return true;

    return getZipFile().getEntry(entryName) != null;
  }

  public boolean isDirectory() {
    if(entryName == null)
      return true;

    return new ZipEntry(entryName).isDirectory();
  }

  public boolean isFile() { return !isDirectory(); }

  public ZipFile getZipFile() {
    try {
      if(zipFile == null)
        zipFile = new ZipFile(zipName);
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    return zipFile;
  }

  public String getEntryName() { return entryName; }

  public String getParent() {
    String name = entryName;

    if(name == null)
      return super.getParent();

    if(name.endsWith("/"))
      name = name.substring(0, name.length()-1);

    int pos = name.lastIndexOf('/');
    if(pos == -1)
      return super.getParent();

    return getZipFile().getName() + "/" + entryName.substring(0, pos);
  }


  public String getName() {
    if(entryName == null)
      return super.getName();

    String name = null;
    if(entryName.endsWith("/"))
      name = entryName.substring(0, entryName.length()-1);
    else
      name = entryName;

    int pos = name.lastIndexOf("/");
    if(pos != -1)
      name = name.substring(pos+1);

    return name;
  }

  public String[] list() {
    if(isFile()) return new String[0];

    Vector files = new Vector();

    String template = entryName;
    if(template == null)
      template = "";
    for(Enumeration e = getZipFile().entries(); e.hasMoreElements(); ) {
      ZipEntry entry = (ZipEntry)e.nextElement();
      String entryName = entry.getName();
      if(template.length() > entryName.length()) continue;
      if(entryName.equals(template)) continue;

      if(entryName.startsWith(template)) {
        String subName = entryName.substring(template.length());


        int pos = subName.indexOf('/');
        String name = null;
        if(pos == -1)
          name = entryName;
        else
          name = template + subName.substring(0, pos+1);

        if(!files.contains(name))
          files.addElement(name);
      }
    }

    String[] fileArray = new String[files.size()];
    files.copyInto(fileArray);

    return fileArray;
  }

  public static void print(ArchivedFile ef) throws Exception {
    String[] list = ef.list();
    for(int i=0; i < list.length; i++) {
      System.out.println("! " + list[i]);
      ArchivedFile efi = new ArchivedFile(ef.getZipFile().getName(), list[i]);
      if(efi.isDirectory()) {
        System.out.println(efi + " is directory. Path =" + efi.getPath() + " Name = " + efi.getName() + " " + efi.exists() + " Parent = " + efi.getParent());
      }
      else
        System.out.println(efi + " is file. Path =" + efi.getPath() + " Name = " + efi.getName() + " " + efi.exists() + " Parent = " + efi.getParent());
    }
  }

  public static void main(String args[]) throws Exception {
//    ArchivedFile ef = new ArchivedFile("d:\\work\\archooser\\classes\\juggler.jar"
//                      , "sunw/demo/juggler/");

    ArchivedFile ef = new ArchivedFile("d:\\jdk1.1\\lib\\classes.zip",
                                       "java/applet/");
    if(!ef.exists())
      System.out.println("File " + ef + " does not exist");
    else if(ef.isDirectory()) {
      System.out.println(ef + " is directory");
    }
    else if(ef.isFile())
      System.out.println(ef + " is file");

    System.out.println("ef.getPath() " + ef.getPath());
    System.out.println("ef.getName() " + ef.getName());
    System.out.println("ef.getParent() " + ef.getParent());
    System.out.println("ef.getZipFile() " + ef.getZipFile());
    System.out.println("ef.getEntryName() " + ef.getEntryName());

    ArchivedFile.print(ef);
  }

}

