// ClassFrame.java

package org.javalobby.cafebabe.plugin.classplug;

import java.io.*;
import java.beans.*;
import java.util.zip.*;
import java.awt.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.javalobby.mdi.*;
import org.javalobby.util.*;
import org.javalobby.cafebabe.*;
import org.javalobby.cafebabe.archooser.*;

// internal frame that visualizes tree-table with information about
// sections of class file

public class ClassFrame extends ExtensionFrame implements ClassPlugConstants {

  static {
    ProgramChooser chooser = ProgramChooser.getInstance();
    javax.swing.filechooser.FileFilter currFilter = chooser.getFileFilter();

    chooser.addChoosableFileFilter(new ClassFileFilter());
    chooser.setFileFilter(currFilter);

    ProgramFileView fileView = (ProgramFileView)chooser.getFileView();

    fileView.putIcon("class", IconProducer.getImageIcon(ICON_CLASS_FILE, ClassFrame.class));
  }

  private ClassTreeTable classTreeTable;
  private String className;

  private boolean[] visibleFolders = new boolean[FOLDERS_NAME.length];

  public ClassFrame() {
    for(int i=0; i < FOLDERS_NAME.length; i++) {
      visibleFolders[i] = true;
    }

    setSize(700, 350);
  }

  public void open() {
    PlainActions actions = new PlainActions(this);

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());
    this.getContentPane().add(topPanel);

    topPanel.add(actions.getToolBar(), BorderLayout.NORTH);

    classTreeTable = new ClassTreeTable(getDesktopPane(), new Integer(getLayer()), visibleFolders);

    // Create a new class tree-table
    JScrollPane scrollPane = new JScrollPane(classTreeTable);
    topPanel.add(scrollPane, BorderLayout.CENTER);

    topPanel.add(scrollPane, BorderLayout.CENTER);

    super.open();
  }

  public void load(ProgramState programState) {
    visibleFolders[0]  = programState.getBooleanProperty("MagicNumber", true);
    visibleFolders[1]  = programState.getBooleanProperty("MinorVersion", true);
    visibleFolders[2]  = programState.getBooleanProperty("MajorVersion", true);
    visibleFolders[3]  = programState.getBooleanProperty("ConstantPool", true);
    visibleFolders[4]  = programState.getBooleanProperty("AccessFlags", true);
    visibleFolders[5]  = programState.getBooleanProperty("ThisClass", true);
    visibleFolders[6]  = programState.getBooleanProperty("SuperClass", true);
    visibleFolders[7]  = programState.getBooleanProperty("Interfaces", true);
    visibleFolders[8]  = programState.getBooleanProperty("Fields", true);
    visibleFolders[9]  = programState.getBooleanProperty("Methods", true);
    visibleFolders[10] = programState.getBooleanProperty("ClassAttributes", true);

    className = (String)programState.get("LastClassFileName");

    setBounds(programState.getBoundsProperty("ClassFrame", "20;20;500;400"));
  }

  public void save(ProgramState programState) {
    programState.putBooleanProperty("MagicNumber",     visibleFolders[0]);
    programState.putBooleanProperty("MinorVersion",    visibleFolders[1]);
    programState.putBooleanProperty("MajorVersion",    visibleFolders[2]);
    programState.putBooleanProperty("ConstantPool",    visibleFolders[3]);
    programState.putBooleanProperty("AccessFlags",     visibleFolders[4]);
    programState.putBooleanProperty("ThisClass",       visibleFolders[5]);
    programState.putBooleanProperty("SuperClass",      visibleFolders[6]);
    programState.putBooleanProperty("Interfaces",      visibleFolders[7]);
    programState.putBooleanProperty("Fields",          visibleFolders[8]);
    programState.putBooleanProperty("Methods",         visibleFolders[9]);
    programState.putBooleanProperty("ClassAttributes", visibleFolders[10]);

    if(className != null) {
      programState.put("LastClassFileName", className);
    }

    programState.putBoundsProperty("ClassFrame", getBounds());
  }

/*  public void requestFocus() {
    if(classTreeTable != null) {
      classTreeTable.requestFocus();
    }
  }
*/
  public boolean canExit() {
    if(classTreeTable.isAnyChange()) {
      int value = JOptionPane.showConfirmDialog(this,
                       Constants.SAVE_FILE_QUESTION + " " + className + "?",
                       Constants.CONFIRMATION_STRING,
                       JOptionPane.YES_NO_CANCEL_OPTION);

      if(value == JOptionPane.CANCEL_OPTION) return false;

      if(value == JOptionPane.YES_OPTION)
        if(!saveAs(className))
          return false;
    }

    return true;
  }

  public void close() {
    classTreeTable.closeClassFile();
    className = null;
  }

  public void setTutorialMode(boolean mode) {
    classTreeTable.setTutorialMode(mode);
  }

  public void collapse() {
    classTreeTable.collapse();
  }

  public void expandFirstLevel() {
    classTreeTable.expandFirstLevel();
  }

  public void expandAllLevels() {
    classTreeTable.expandAllLevels();
  }

  public void search() {
    Component current = this;
    Component parent  = null;
    while(true) {
      parent = current.getParent();
      if(parent == null) break;
      if(parent instanceof JFrame) break;
      current = parent;
    }

    if(parent != null)
      classTreeTable.search((JFrame)parent);
  }

  public void open(File f) {
    InputStream is = null;
    try {
      if(f instanceof ArchivedFile) {
        ArchivedFile ef = (ArchivedFile)f;
        ZipFile zipFile = ef.getZipFile();
        ZipEntry zipEntry = zipFile.getEntry(ef.getEntryName());

        is = zipFile.getInputStream(zipEntry);
        className = zipFile.getName() + "#" + ef.getEntryName();
      }
      else {
        is = new FileInputStream(f);
        className = f.getAbsolutePath();
      }
      classTreeTable.openClassFile(is);

      String title = "";
      if(f instanceof ArchivedFile)
       title += f.getParent() + "#" + ((ArchivedFile)f).getEntryName();
      else
       title += f.getAbsolutePath();

      fireUpdatePerformed(this, getTitle(), false);
      setTitle(title);
      fireUpdatePerformed(this, getTitle(), true);
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public boolean saveAs(String className) {
    OutputStream os = null;

    try {
      File f = new File(className);

      if(f instanceof ArchivedFile) {
        JOptionPane.showMessageDialog(this,
             "You cannot save file " + className + " to archive",
             "Message", JOptionPane.WARNING_MESSAGE);
        return false;
      }

      classTreeTable.saveAsClassFile(f);

      fireUpdatePerformed(this, getTitle(), false);
      setTitle(className);
      fireUpdatePerformed(this, getTitle(), true);

      return true;
    }
    catch(Exception e) {
      e.printStackTrace();
    }

    return false;
  }

  public void setMethodPosition(String selectedMethod) {
    if(classTreeTable.search(selectedMethod, SEARCH_METHOD, false, true)) {
      classTreeTable.expandMethodFolder();
    }
  }

  public void setFieldPosition(String selectedField) {
    if(classTreeTable.search(selectedField, SEARCH_FIELD, false, true)) {
      classTreeTable.expandFieldFolder();
    }
  }

  public void setShowSections(boolean[] visibleFolders) {
    classTreeTable.showSections(visibleFolders);
  }

}
