// ClassTreeTable.java

package org.javalobby.cafebabe.plugin.classplug;

import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.border.*;

import treetable.*;
import org.javalobby.classfile.*;
import org.javalobby.task.migrator.*;
import org.javalobby.task.stripper.*;
import org.javalobby.task.obfuscator.*;
import org.javalobby.mdi.*;

import org.javalobby.cafebabe.*;
import org.javalobby.cafebabe.plugin.classplug.bodyeditor.*;

public class ClassTreeTable extends JTreeTable implements ClassPlugConstants {

  private JTree tree;
  private RootNode root;
  private ClassTreeDataModel dataModel;
  private ClassFile classFile;
  private MethodNode methodNode;

  protected JPopupMenu methodNodeMenu = new MethodNodePopupMenu();

  private boolean isTutorialMode = false;

  private JDesktopPane desktopPane;
  private Integer layer;

  public ClassTreeTable(JDesktopPane desktopPane, Integer layer,
                    boolean[] visibleFolders) {

    super(new ClassTreeDataModel(visibleFolders));

    this.desktopPane = desktopPane;
    this.layer       = layer;

    tree = this.getTree();

    dataModel = (ClassTreeDataModel)tree.getModel();

    root = (RootNode)dataModel.getRoot();

    tree.setEditable(false);
    tree.setRootVisible(false);
    tree.setSelectionPath(new TreePath(root));
    tree.setShowsRootHandles(true);

    this.addMouseListener(new ModificationListener());
    addMouseListener(new CustomMouseAdapter());

/*    DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer)tree.getCellRenderer();
    renderer.setLeafIcon(new ImageIcon("Icons/cafebabe2.gif"));
    renderer.setOpenIcon(new ImageIcon("Icons/cafebabe2.gif"));
    renderer.setClosedIcon(new ImageIcon("Icons/cafebabe2.gif"));
    tree.setCellRenderer(renderer);
*/
    TableColumn column1 = this.getColumnModel().getColumn(0);
    TableColumn column2 = this.getColumnModel().getColumn(1);
    column1.setPreferredWidth(250);
    column2.setPreferredWidth(150);

    this.setBorder( new BevelBorder(BevelBorder.RAISED));

    this.setRowSelectionInterval(0, 0);

    ToolTipManager.sharedInstance().registerComponent(this);
    ToolTipManager.sharedInstance().setInitialDelay(250);
  }

  public boolean isAnyChange() {
    if(classFile != null && classFile.isChanged()) {
      return true;
    }

    for(int i=0; i < root.getChildCount(); i++) {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)root.getChildAt(i);
      if(((String)node.getUserObject()).equals(METHODS_TEXT)) {
        for(int j=0; j < node.getChildCount(); j++) {
          Object next = node.getChildAt(j);
          if(next instanceof MethodNode) {
            MethodNode methodNode = (MethodNode)next;
            if(methodNode.isChanged()) {
              classFile.setChanged(true);

              return true;
            }
          }
        }
        break;
      }
    }

    return false;
  }

  public void openClassFile(InputStream is) throws Exception {
    JFrame mainFrame = (JFrame)this.getTopLevelAncestor();
    mainFrame.getGlassPane().setVisible(true);

    classFile = new ClassFile();
    classFile.read(is);

    dataModel.setClassFile(classFile);
    dataModel.updateTree();

    mainFrame.getGlassPane().setVisible(false);
  }

  public void closeClassFile() {
    JFrame mainFrame = (JFrame)this.getTopLevelAncestor();
    if(mainFrame != null)
      mainFrame.getGlassPane().setVisible(true);

    classFile = null;
    dataModel.setClassFile(classFile);
    dataModel.deleteTree();
    dataModel.createTree();

    if(mainFrame != null)
      mainFrame.getGlassPane().setVisible(false);
  }

  public void saveAsClassFile(File file) throws Exception {
    JFrame mainFrame = (JFrame)this.getTopLevelAncestor();

    mainFrame.getGlassPane().setVisible(true);

    classFile.write(new FileOutputStream(file));

    mainFrame.getGlassPane().setVisible(false);
  }

  public void setTutorialMode(boolean mode) {
    isTutorialMode = mode;
  }

  public String getToolTipText(MouseEvent e) {
    if(!isTutorialMode) return null;

    Point p = new Point(e.getX(), e.getY());
    int row    = this.rowAtPoint(p);
    int column = this.columnAtPoint(p);

    TreePath path = tree.getPathForRow(row);
    TreeNode node = (TreeNode)path.getLastPathComponent();

    String toolTip = null;

    if(column == 0)
     toolTip = node.toString();
    else if(column == 1)
     toolTip = dataModel.getToolTipText(node);

    if(toolTip != null && toolTip.length() == 0)
      toolTip = null;

    return toolTip;
  }

  public TreePath getSelectionPath() {
    if(tree == null) return null;
    return tree.getSelectionPath();
  }

  public void setSelectionPath(TreePath path) {
    if(path == null) return;

    int row = tree.getRowForPath(path);

    // find JScrollPane object - parent for this component
    Container curr = getParent();
    while(curr != null &&
          !curr.getClass().getName().equals(JScrollPane.class.getName())) {
      curr = curr.getParent();
    }

    if(curr != null) {
      JScrollPane sp = (JScrollPane)curr;
      this.setRowSelectionInterval(row, row);

      JViewport vp = sp.getViewport();
      vp.setViewPosition(new Point(0, row*this.getRowHeight()));
    }
  }

  public void collapse() {
    TreePath path = getSelectionPath();

    Enumeration e = root.children();
    while(e.hasMoreElements()) {
      collapse((TreeNode)e.nextElement());
    }

    if(path == null) return;
    Object[] o = path.getPath();
    if(o.length > 1) {
      tree.setSelectionPath(new TreePath(new Object[] {o[0], o[1]}));
    }
  }

  public void collapse(TreeNode node) {
    Enumeration e = node.children();
    while(e.hasMoreElements()) {
      TreeNode iNode = (TreeNode)e.nextElement();
      collapse(iNode);
      DefaultMutableTreeNode parent = (DefaultMutableTreeNode)iNode.getParent();
      TreePath path = new TreePath(parent.getPath());
      tree.collapsePath(path);
    }
  }

  public void expandFirstLevel() {
    collapse();

    Enumeration e = root.children();
    while(e.hasMoreElements()) {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
      TreePath path = new TreePath(node.getPath());
      tree.expandPath(path);
    }
  }

  public void expandFieldFolder() {
//    tree.requestFocus();

    TreePath path = tree.getSelectionPath();
    if(path == null) return;

    DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
    while(true) {
      FolderNode parent = (FolderNode)node.getParent();
      path = new TreePath(parent.getPath());

      if(node instanceof FolderNode) {
        FolderNode folderNode = (FolderNode)node;
        node = (DefaultMutableTreeNode)folderNode.getChildAt(0);
      }
      else {
        tree.expandPath(path);
        setSelectionPath(new TreePath((node).getPath()));
        break;
      }
    }
  }

  public void expandMethodFolder() {
//    tree.requestFocus();

    TreePath path = tree.getSelectionPath();
    if(path == null) return;

    DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();

    while(true) {
      DefaultMutableTreeNode parent = (DefaultMutableTreeNode)node.getParent();
      path = new TreePath(parent.getPath());

      if(node instanceof FolderNode) {
        FolderNode folderNode = (FolderNode)node;
        int codePos = -1;
        Enumeration e = folderNode.children();
        for(int i=0; i < folderNode.getChildCount(); i++) {
          TreeTableNode iNode = (TreeTableNode)folderNode.getChildAt(i);

          if(iNode.getText().equals(AttributeEntry.CODE)) {
            codePos = i;
            break;
          }
        }

        if(codePos == -1)
          node = (DefaultMutableTreeNode)folderNode.getChildAt(0);
        else
          node = (DefaultMutableTreeNode)folderNode.getChildAt(codePos);
      }
      else {
        tree.expandPath(path);
        setSelectionPath(new TreePath((node).getPath()));
        break;
      }
    }
  }

  public void expandAllLevels() {
    Enumeration e = root.children();
    while(e.hasMoreElements()) {
      expandNode((TreeNode)e.nextElement());
    }
  }

  private void expandNode(TreeNode node) {
    if(node instanceof FolderNode) {
      Enumeration e = node.children();
      while(e.hasMoreElements()) {
        expandNode((TreeNode)e.nextElement());
      }
    }
    else if(node instanceof LeafNode) {
      LeafNode leaf = (LeafNode)node;
      DefaultMutableTreeNode parent = (DefaultMutableTreeNode)leaf.getParent();
      TreePath path = new TreePath(parent.getPath());
      tree.expandPath(path);
    }
  }

  public void changePlaf() {
    SwingUtilities.updateComponentTreeUI(tree);
    tree.setRowHeight(this.getRowHeight());

    SwingUtilities.updateComponentTreeUI(methodNodeMenu);
  }

class ModificationListener extends MouseAdapter {
  public void mouseClicked(MouseEvent event) {
    if(tree == null) return;
    TreePath path = tree.getSelectionPath();
    if(path == null) return;

    TreeNode node = (TreeNode)path.getLastPathComponent();

    if((node instanceof LeafNode) && (event.getClickCount() >= 2)) {
      LeafNode leaf = (LeafNode)node;

      FolderNode parent = (FolderNode)leaf.getParent();
      String parentText = parent.getText();
      if(parentText.equals(CONSTANT_POOL_TEXT)) {
        modifyLeaf(leaf);
      }
      else if(leaf.getText().equals("Body")) {
        openMethodBodyFrame((MethodNode)parent.getParent().getParent());
      }
    }
  }

  void modifyLeaf(LeafNode leaf) {
    String line = leaf.getUserObject().toString();
    StringTokenizer st = new StringTokenizer(line);
    short index = 0;
    try {
      index = (short)Integer.parseInt(st.nextToken(), 16);
      ConstPool constPool = classFile.getConstPool();
      PoolEntry entry = constPool.getEntry(index);
      if(entry instanceof Utf8Const) {
        Utf8Const c = (Utf8Const)entry;
        String oldValue = c.getValue();
        JFrame mainFrame = (JFrame)ClassTreeTable.this.getTopLevelAncestor();

        EditUtfDialog dialog = new EditUtfDialog(mainFrame, oldValue);
        dialog.show();

        String newValue = dialog.getUtfString();
        if(newValue != null && !newValue.equals(oldValue)) {
          mainFrame.setCursor(new Cursor(Frame.WAIT_CURSOR));
          constPool.modifyUTF(index, newValue);
          classFile.setChanged(true);
//          dataModel.updateTree();
          dataModel.updateNode(leaf, entry, newValue);

          mainFrame.setCursor(new Cursor(Frame.DEFAULT_CURSOR));
        }
      }
    }
    catch(NumberFormatException e) {  e.printStackTrace(); }
  }

}

  public void openMethodBodyFrame(MethodNode methodNode) {
    new BodyEditor(desktopPane, layer, classFile, methodNode, 0);
  }

  public void openMethodBodyFrame(MethodNode methodNode, int pos) {
    new BodyEditor(desktopPane, layer, classFile, methodNode, pos);
  }

  public void showSections(boolean[] visibleFolders) {
    dataModel.showFolders(visibleFolders);
  }

  public void search(JFrame frame) {
    new SearchDialog(frame, ClassTreeTable.this, classFile.getConstPool()).show();
  }

  public boolean search(String line, int searchMode, boolean isNext, boolean exact) {
    if(searchMode == SEARCH_PLAIN) {
      return search(line, root, isNext, exact);
    }

    TreeNode cpNode = null;

    if(searchMode == SEARCH_CONST_POOL) {
      cpNode = getChild(CONSTANT_POOL_TEXT);
    }
    else if(searchMode == SEARCH_FIELD) {
      cpNode = getChild(FIELDS_TEXT);
    }
    else if(searchMode == SEARCH_METHOD) {
      cpNode = getChild(METHODS_TEXT);
    }
    else if(searchMode == SEARCH_CLASS_ATTRIBUTE) {
      cpNode = getChild(CLASS_ATTRIBUTES_TEXT);
    }

    if(cpNode != null) {
      return search(line, cpNode, isNext, exact);
    }

    JOptionPane.showMessageDialog(ClassTreeTable.this,
                "String \"" + line + "\" does not found!", "Search dialog",
                 JOptionPane.INFORMATION_MESSAGE);
    return false;
  }

  private TreeNode getChild(String childText) {
    Enumeration e = root.children();
    while(e.hasMoreElements()) {
      TreeNode iNode = (TreeNode)e.nextElement();
      if(iNode.toString().equals(childText)) {
        return iNode;
      }
    }

    return null;
  }

  private boolean search(String line, TreeNode node, boolean isNext, boolean exact) {
    Enumeration e = node.children();
    while(e.hasMoreElements()) {
      TreeNode iNode = (TreeNode)e.nextElement();
      String text = iNode.toString();

      if(exact) {
        if(text.equals(line)) {
          changePosition(iNode);
          return true;
        }
      }
      else {
        StringTokenizer st = new StringTokenizer(text);
        while(st.hasMoreTokens()) {
          String token = st.nextToken();
          if(token.toLowerCase().startsWith(line.toLowerCase())) {
            changePosition(iNode);
            return true;
          }
        }
      }

      if(search(line, iNode, isNext, exact)) {
        return true;
      }
    }

    return false;
  }

  public void changePosition(TreeNode iNode) {
    DefaultMutableTreeNode current = (DefaultMutableTreeNode)iNode;
    DefaultMutableTreeNode parent  = (DefaultMutableTreeNode)iNode.getParent();

    TreePath path1 = new TreePath(current.getPath());
    TreePath path2 = new TreePath(parent.getPath());

    tree.expandPath(path2);
    this.setSelectionPath(path1);
    tree.repaint();
    repaint();
  }

  public void showPopupMenu(TreeNode node, MouseEvent event) {
    if(node instanceof MethodNode) {
      methodNode = (MethodNode)node;
      methodNodeMenu.show(event.getComponent(), event.getX(), event.getY());
    }
    else {
      methodNode = null;
    }
  }

public class MethodNodePopupMenu extends JPopupMenu {
  final String GOTO_METHOD_BODY_STRING = "Go to method body";

  private JMenuItem gotoMethodBodyItem = new JMenuItem(GOTO_METHOD_BODY_STRING);

  public MethodNodePopupMenu() {
    super();

    gotoMethodBodyItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        MethodEntry methodEntry  = methodNode.getMethodEntry();
        AccessFlags accessFlags = methodEntry.getAccessFlags();
        if(accessFlags.isAbstract()) {
          JOptionPane.showMessageDialog(ClassTreeTable.this,
               "This method is abstract. It has no body.",
               "Message", JOptionPane.WARNING_MESSAGE);
        }
        else {
          openMethodBodyFrame(methodNode);
        }
      }
    });

    this.add(gotoMethodBodyItem);
  }

}

class CustomMouseAdapter extends MouseAdapter {

  public void mouseReleased(MouseEvent event) {
    if(event.isPopupTrigger()) {
      TreePath path = tree.getPathForLocation(event.getX(), event.getY());
      if(path == null) return;

      showPopupMenu((MutableTreeNode)path.getLastPathComponent(), event);
    }
  }

}


  public static void main(String[] args) /*throws Exception*/ {
    JFrame frame = new JFrame("ClassTreeTable Test");

    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) {
        System.exit(0);
      }
    });

    ClassTreeTable treeTable = new ClassTreeTable(
      (MdiDesktopPane)null, new Integer(1),
      new boolean[] { true, true, true, true, true, true,
                      true, true, true, true, true }
    );

    JScrollPane sp = new JScrollPane(treeTable);

    try {
      treeTable.openClassFile(new FileInputStream("AboutFrame.class"));
    }
    catch(Exception e) {
      e.printStackTrace();
    }

    treeTable.expandAllLevels();

    int row = 45;

    treeTable.setRowSelectionInterval(row, row);
    System.out.println("getSelectedRow() = " + treeTable.getSelectedRow());

    JViewport vp = sp.getViewport();
    vp.setViewPosition(new Point(0, row*treeTable.getRowHeight()));

    frame.getContentPane().add(sp);
    frame.getContentPane().setBackground(treeTable.getBackground());
    frame.pack();
    frame.setVisible(true);
  }

}
