// SearchDialog.java

package org.javalobby.cafebabe.plugin.classplug;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.cafebabe.*;

// dialog for search preparation process through class file entities

public class SearchDialog extends JDialog implements ClassPlugConstants {

  private final String PLAIN_ACTION     = "plain";
  private final String CP_ACTION        = "cp";
  private final String FIELD_ACTION     = "field";
  private final String METHOD_ACTION    = "method";
  private final String ATTRIBUTE_ACTION = "attribute";

  static private int searchMode         = SEARCH_PLAIN;
  static private String searchString    = null;

  static private JTextField inputField;

  private boolean isFirst = true;

  private ClassTreeTable classTreeTable;
  private ConstPool constPool;

  public SearchDialog(JFrame parentFrame, ClassTreeTable classTreeTable, ConstPool constPool) {
    // Make sure we call the parent
    super(parentFrame, false);

    this.classTreeTable = classTreeTable;
    this.constPool      = constPool;

    enableEvents(AWTEvent.FOCUS_EVENT_MASK);

    this.getContentPane().setLayout(new BorderLayout());

    // Set the characteristics for this dialog instance
    this.setTitle("Search a string...");
    this.setResizable(false);
    Rectangle r = parentFrame.getBounds();
    this.setBounds(r.x + r.width/2 - 550/2, r.y + r.height/2 - 210/2, 550, 210);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    ButtonGroup group = new ButtonGroup();

    JRadioButton radio1 = new JRadioButton("Plain");
    JRadioButton radio2 = new JRadioButton("Constant pool");
    JRadioButton radio3 = new JRadioButton("Fields");
    JRadioButton radio4 = new JRadioButton("Methods");
    JRadioButton radio5 = new JRadioButton("Class attributes");

    radio1.setActionCommand(PLAIN_ACTION);
    radio2.setActionCommand(CP_ACTION);
    radio3.setActionCommand(FIELD_ACTION);
    radio4.setActionCommand(METHOD_ACTION);
    radio5.setActionCommand(ATTRIBUTE_ACTION);

    radio1.setSelected((searchMode == SEARCH_PLAIN) ? true : false);
    radio2.setSelected((searchMode == SEARCH_CONST_POOL) ? true : false);
    radio3.setSelected((searchMode == SEARCH_FIELD) ? true : false);
    radio4.setSelected((searchMode == SEARCH_METHOD) ? true : false);
    radio5.setSelected((searchMode == SEARCH_CLASS_ATTRIBUTE) ? true : false);

    group.add(radio1);
    group.add(radio2);
    group.add(radio3);
    group.add(radio4);
    group.add(radio5);

    inputField = new JTextField(searchString, 45);
    JButton searchButton = new JButton("Next");
    JButton closeButton   = new JButton("Close");

    ModeListener modeListener     = new ModeListener();
    SearchListener searchListener = new SearchListener();

    radio1.addActionListener(modeListener);
    radio2.addActionListener(modeListener);
    radio3.addActionListener(modeListener);
    radio4.addActionListener(modeListener);
    radio5.addActionListener(modeListener);
    inputField.addActionListener(searchListener);
    searchButton.addActionListener(searchListener);
    closeButton.addActionListener(new CloseListener());

    JPanel panel11 = new JPanel();
    panel11.setBorder(new TitledBorder(new EtchedBorder(), "Search mode"));

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(panel11);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));

    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel11.add(radio1);
    panel11.add(Box.createRigidArea(new Dimension(10, 0)));
    panel11.add(radio2);
    panel11.add(Box.createRigidArea(new Dimension(10, 0)));
    panel11.add(radio3);
    panel11.add(Box.createRigidArea(new Dimension(10, 0)));
    panel11.add(radio4);
    panel11.add(Box.createRigidArea(new Dimension(10, 0)));
    panel11.add(radio5);
    panel11.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel31 = new JPanel();
    panel31.setBorder(new TitledBorder(new EtchedBorder(), "Input a string:"));

    JPanel panel3 = new JPanel();
    panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(panel31);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel31.add(inputField);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(searchButton);
    panel5.add(Box.createRigidArea(new Dimension(20, 0)));
    panel5.add(closeButton);

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
    this.setContentPane(topPanel);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(panel3);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(panel1);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(panel5);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    inputField.requestFocus();
  }

// inner classes-adapters

class CloseListener implements ActionListener {
  public void actionPerformed(ActionEvent e) {
    dispose();
  }
}

class ModeListener implements ActionListener {
  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();

    if(cmd.equals(PLAIN_ACTION)) {
      searchMode = SEARCH_PLAIN;
    }
    else if(cmd.equals(CP_ACTION)) {
      searchMode = SEARCH_CONST_POOL;
    }
    else if(cmd.equals(FIELD_ACTION)) {
      searchMode = SEARCH_FIELD;
    }
    else if(cmd.equals(METHOD_ACTION)) {
      searchMode = SEARCH_METHOD;
    }
    else if(cmd.equals(ATTRIBUTE_ACTION)) {
      searchMode = SEARCH_CLASS_ATTRIBUTE;
    }
  }
}

class SearchListener implements ActionListener {
  private SearchThread searchThread = null;
  private boolean isFirstClick     = true;

  public void actionPerformed(ActionEvent e) {
    if(isFirstClick) {
      isFirstClick = false;

      searchString = inputField.getText();

      if(searchString != null && searchString.length() > 0) {
        searchThread = new SearchThread(searchString, searchMode);
        searchThread.start();
      }
    }
    else {
      if(searchThread != null) {
        synchronized(searchThread) {
          searchThread.notify();
        }
      }
      else {
        isFirstClick = true;
      }
    }
  }

class SearchThread extends Thread {
  JTree tree = classTreeTable.getTree();
  ClassTreeDataModel dataModel = (ClassTreeDataModel)tree.getModel();
  RootNode root = (RootNode)dataModel.getRoot();

  boolean isFound = false;

  private String line;
  private int searchMode;

  SearchThread(String line, int searchMode) {
    super("SearchThread");

    setPriority(Thread.MIN_PRIORITY+2);

    this.line       = line ;
    this.searchMode = searchMode;
  }

  public void run() {
    TreeNode treeNode = null;

    if(searchMode == SEARCH_PLAIN) {
      treeNode = root;
    }
    else if(searchMode == SEARCH_CONST_POOL) {
      treeNode = getChild(CONSTANT_POOL_TEXT);
    }
    else if(searchMode == SEARCH_FIELD) {
      treeNode = getChild(FIELDS_TEXT);
    }
    else if(searchMode == SEARCH_METHOD) {
      treeNode = getChild(METHODS_TEXT);
    }
    else if(searchMode == SEARCH_CLASS_ATTRIBUTE) {
      treeNode = getChild(CLASS_ATTRIBUTES_TEXT);
    }

    if(treeNode != null) {
      search(line, treeNode);
    }

    if(isFound) {
      JOptionPane.showMessageDialog(SearchDialog.this,
                  "No more string \"" + line + "\" in this class file.", "Search dialog",
                   JOptionPane.INFORMATION_MESSAGE);
    }
    else {
      JOptionPane.showMessageDialog(SearchDialog.this,
                  "String \"" + line + "\" does not found!", "Search dialog",
                   JOptionPane.INFORMATION_MESSAGE);
    }

    isFirstClick = true;
  }

  private void search(String line, TreeNode node) {
    Enumeration e = node.children();
    while(e.hasMoreElements()) {
      TreeNode iNode = (TreeNode)e.nextElement();

      if(iNode instanceof MethodNode) {
        if(searchMode == SEARCH_PLAIN || searchMode == SEARCH_METHOD) {
          MethodNode methodNode = (MethodNode)iNode;
          MethodEntry methodEntry = methodNode.getMethodEntry();
          AttributeEntry attribute =
                         methodEntry.getAttribute(AttributeEntry.CODE, constPool);

          if(attribute != null) {
            CodeAttribute codeAttribute = new CodeAttribute(attribute);
            MethodBody methodBody       = codeAttribute.getMethodBody();

            Instruction[] instructions = methodBody.getInstructions();

            for(int i=0; i < instructions.length; i++) {
              Instruction instruction = instructions[i];
              String[] str = instruction.resolve(constPool);

              StringTokenizer st = new StringTokenizer(Converter.merge(str, " "));
              while(st.hasMoreTokens()) {
                String token = st.nextToken();
                if(token.toLowerCase().startsWith(line.toLowerCase())) {
                  classTreeTable.openMethodBodyFrame(methodNode, i);
                  isFound = true;
                  try {
                    synchronized(this) {
                      wait();
                    }
                  }
                  catch(InterruptedException ex) {}
                }
              }
            }
          }
        }
      }

      String text = iNode.toString();
      StringTokenizer st = new StringTokenizer(text);
      while(st.hasMoreTokens()) {
        String token = st.nextToken();
        if(token.toLowerCase().startsWith(line.toLowerCase())) {
          classTreeTable.changePosition(iNode);
          isFound = true;
          try {
            synchronized(this) {
              wait();
            }
          }
          catch(InterruptedException ex) {}
        }
      }

      search(line, iNode);
    }
  }

  private TreeNode getChild(String childText) {
    Enumeration e = root.children();
    while(e.hasMoreElements()) {
      TreeNode iNode = (TreeNode)e.nextElement();
      if(iNode.toString().equals(childText)) {
        return iNode;
      }
    }

    return null;
  }

}

}

}
