// BodyEditor.java

package org.javalobby.cafebabe.plugin.classplug.bodyeditor;

import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.mdi.*;

import org.javalobby.cafebabe.*;
import org.javalobby.cafebabe.plugin.classplug.*;
import org.javalobby.cafebabe.plugin.classplug.help.*;

public class BodyEditor extends MdiFrame {

  private BodyTable bodyTable;

  private BodyEditorActions actions = new BodyEditorActions(this);

  private JDesktopPane desktopPane;
  private Integer layer;
  private ClassFile classFile;
  private MethodNode methodNode;

  public BodyEditor(final JDesktopPane desktopPane, Integer layer,
                    final ClassFile classFile, final MethodNode methodNode, int position) {
    // resizable, closable, maximizable, iconifiable
    super("Method's body: " + methodNode, true, true, true, true);
    this.desktopPane = desktopPane;
    this.layer       = layer;
    this.classFile   = classFile;
    this.methodNode  = methodNode;

    enableEvents(AWTEvent.FOCUS_EVENT_MASK);

    // Set the characteristics for this dialog instance
    Rectangle r = desktopPane.getBounds();
    this.setBounds(r.x + r.width/2 - 650/2, r.y + r.height/2 - 400/2, 650, 300);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    bodyTable = new BodyTable(desktopPane, methodNode, classFile) {
      public void showTopic() {
        help();
      }
    };

//    if(position < bodyTable.getColumnCount()) {
      bodyTable.setRowSelectionInterval(position, position);
//    }

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());
    this.getContentPane().add(topPanel);

    topPanel.add(actions.getToolBar(), BorderLayout.NORTH);


    JScrollPane scrollPane = new JScrollPane();
    scrollPane.getViewport().setView(bodyTable);

    topPanel.add(scrollPane, BorderLayout.CENTER);

    desktopPane.add(this, layer);
    open();
  }

  private void fire(boolean visible) {
    fireUpdatePerformed(this, getTitle(), visible);
  }

  public void editInstruction() {
    bodyTable.editInstruction();
  }

  public void addInstruction() {
    bodyTable.addInstruction(null);
  }

  public void removeInstructions() {
    bodyTable.removeInstructions();
  }

  public void help() {
    String mnemonic = bodyTable.getCurrentMnemonic();

    if(mnemonic == null) {
      JOptionPane.showMessageDialog(this.getParent(),
                  "Please select an instruction to be removed.\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
    }
    else {
      HelpFrame helpFrame = new HelpFrame();

      desktopPane.add(helpFrame, layer);

      helpFrame.open();

      String fileName = ClassPlugConstants.INSTRUCTIONS_ARCHIVE_NAME;

      int index = mnemonic.indexOf('_');
      if(index != -1) {
        mnemonic = mnemonic.substring(0, index+1);
      }
      else if(mnemonic.startsWith("dcmp")) {
        mnemonic = "dcmp";
      }
      else if(mnemonic.startsWith("fcmp")) {
        mnemonic = "fcmp";
      }

      if(new File(fileName).exists()) {
        helpFrame.showTopic(ClassPlugConstants.INSTRUCTIONS_ARCHIVE_NAME,
                            mnemonic + ".html");
      }
      else {
        helpFrame.showTopic(new File("Help/" + mnemonic + ".html").getAbsolutePath());
      }
    }
  }

}
