// BodyTableModel.java

package org.javalobby.cafebabe.plugin.classplug.bodyeditor;

import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;

import org.javalobby.cafebabe.plugin.classplug.*;

public class BodyTableModel extends AbstractTableModel {

  final static String[] names = {"Offset", "Instruction", "Parameter"};

  private Vector offsets = new Vector();

  private CodeAttribute codeAttribute;
  private MethodBody methodBody;

  private MethodNode methodNode;
  private ConstPool constPool;

  public BodyTableModel(MethodNode methodNode, ConstPool constPool) {
    this.methodNode = methodNode;
    this.constPool  = constPool;

    MethodEntry methodEntry  = methodNode.getMethodEntry();
    AttributeEntry attribute = methodEntry.getAttribute(AttributeEntry.CODE, constPool);

    if(attribute != null) {
      codeAttribute = new CodeAttribute(attribute);

      methodBody    = codeAttribute.getMethodBody();
      calculateOffset();
    }
  }

  private void calculateOffset() {
    offsets.removeAllElements();

    Instruction[] instructions = methodBody.getInstructions();

    for(int i=0, pos=0; i < instructions.length; i++) {
      Instruction instruction = instructions[i];

      offsets.addElement(new Short((short)pos));

      pos += instruction.length();
    }

    fireTableDataChanged();
  }

  public void editInstruction(Instruction instruction, int pos) {
    Instruction oldInstruction = methodBody.getInstruction(pos);

    if(!instruction.equals(oldInstruction)) {
      methodBody.setInstruction(instruction, pos);
      calculateOffset();
    }
  }

  public void addInstruction(Instruction instruction, int pos) {
    methodBody.insertInstruction(instruction, pos);
    calculateOffset();
  }

  public void removeInstructions(Instruction[] instructions) {
    for(int i=0; i < instructions.length; i++) {
      methodBody.removeInstruction(instructions[i]);
    }

    calculateOffset();
  }

  public short getOffset(int pos) {
    return ((Short)offsets.elementAt(pos)).shortValue();
  }

  public boolean isBodyChanged() {
    return methodBody.isBodyChanged();
  }

  public CodeAttribute getCodeAttribute() {
    return codeAttribute;
  }

  public Instruction getInstruction(int pos) {
    return methodBody.getInstruction(pos);
  }

  public int getColumnCount() {
    return names.length;
  }

  public int getRowCount() {
    return methodBody.size();
  }

  public Object getValueAt(int row, int column) {
    if(row < 0 || row > methodBody.size()-1)
      return "";

    Instruction instruction = methodBody.getInstruction(row);

    if(column == 0) {
      Short wrapper = (Short)offsets.elementAt(row);
      return Converter.toHexString(wrapper.shortValue());
    }
    else if(column == 1) {
      return instruction.getMnemonic();
    }
    else if(column == 2) {
      String[] r = instruction.resolve(constPool);

      if(r.length > 1)
        return r[1];
    }

    return "";
  }

  public String getTooltipAt(int row, int column) {
    if(row < 0 || row > methodBody.size()-1)
      return "";

    Instruction instruction = methodBody.getInstruction(row);

    if(column == 0) {
      Short wrapper = (Short)offsets.elementAt(row);
      return Converter.toHexString(wrapper.shortValue());
    }
    else if(column == 1) {
      return Converter.toHexString(instruction.getTag());
    }
    else if(column == 2) {
      return Converter.toHexString(instruction.getParameter(), " ");
    }

    return "";
  }

  public String getColumnName(int column) {
    return names[column];
  }

  public Class getColumnClass(int c) {
    return getValueAt(0, c).getClass();
  }

  public boolean isCellEditable(int row, int column) {
    return false;
  }

}
