// CompactFrame.java

package org.javalobby.cafebabe.plugin.compactplug;

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

import org.javalobby.classfile.*;
import org.javalobby.task.*;
import org.javalobby.task.compactor.*;
import org.javalobby.task.event.*;
import org.javalobby.mdi.*;
import org.javalobby.util.*;

import org.javalobby.cafebabe.*;

// internal frame which help to process compact task

public class CompactFrame extends PluginFrame {

  final JTextField fromField     = new JTextField();
  final JTextField toField       = new JTextField();
  final JTextField actionField   = new JTextField();
  final JTextField currFileField = new JTextField();

  final JTextField mainClassField       = new JTextField();
  final JTextField classPathField       = new JTextField();
  final JTextField classpathBoundsField = new JTextField();
  final JTextField packagesBoundsField  = new JTextField();
  final JTextField archiveField          = new JTextField();

  final JCheckBox createArchiveCheck =
                 new JCheckBox("Create archive", true);

  final JButton startButton        = new JButton("Start");
  final JButton browseButton       = new JButton("Browse");
  final JButton closeButton        = new JButton("Close");

  JTabbedPane tabbedPane = new JTabbedPane();

  File selectedFile;

  private ProgramChooser chooser = ProgramChooser.getInstance();

  public CompactFrame() {
    setTitle("Compact Frame");
    setResizable(false);
    setMaximizable(false);

    setSelectedFile();
    setClassPath();

    JPanel taskPanel       = fillTaskPanel();
    JPanel optionsPanel    = fillOptionsPanel();

    tabbedPane = new JTabbedPane();
    this.setContentPane(tabbedPane);

    tabbedPane.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent evt) {
        if(tabbedPane.getSelectedIndex() == 1) {
          setClassPath();
        }
      }
    });

    tabbedPane.addTab("Task", null, taskPanel, "Perform compact task");
    tabbedPane.addTab("Configuration", null, optionsPanel,
                      "Allows to configure compact process");

    tabbedPane.setSelectedIndex(0);

    startButton.requestFocus();
  }

  public void open() {
    // Set the characteristics for this dialog instance
    Rectangle r = getDesktopPane().getBounds();
    this.setBounds(r.x + r.width/2 - 650/2, r.y + r.height/2 - 490/2, 650, 360);

    super.open();
  }

  public void load(ProgramState programState) {
    setBounds(programState.getBoundsProperty("CompactFrame", "20;20;500;400"));
  }

  public void save(ProgramState programState) {
    programState.putBoundsProperty("CompactFrame", getBounds());
  }

  private void setSelectedFile() {
    File currentFile = chooser.getSelectedFile();

    if(currentFile == null) {
      try {
         currentFile = new File(new File(new File(".").getCanonicalPath()).getParent());
      }
      catch(Exception e) { e.printStackTrace(); }
    }
    else if(!currentFile.exists())
      currentFile = new File(new File(currentFile.getAbsolutePath()).getParent());

    String fileName = currentFile.toString();
    if(currentFile.isDirectory() ||
       fileName.endsWith("zip")  || fileName.endsWith("jar"))
      fromField.setText(fileName);
    else
      fromField.setText(new File(currentFile.getAbsolutePath()).getParent());
  }

  private void setClassPath() {
    String classPath = System.getProperty("java.class.path") +
                       File.pathSeparator + fromField.getText();
    classPathField.setText(classPath);
  }

  private JPanel fillTaskPanel() {
    JLabel label1  = new JLabel("From dir :");
    JLabel label2  = new JLabel("To dir   :");
    JLabel label4  = new JLabel("File     :");
    JLabel label3  = new JLabel("Action   :");

    Font font = new Font("Monospaced", Font.BOLD, 14);
    Font font2 = new Font("Monospaced", Font.BOLD, 11);

    label1.setFont(font);
    label2.setFont(font);
    label3.setFont(font);
    label4.setFont(font);

    fromField.setFont(font);
    toField.setFont(font);
    classPathField.setFont(font2);
    actionField.setFont(font2);
    currFileField.setFont(font2);

    actionField.setEnabled(false);
    currFileField.setEnabled(false);

    String parentDir = new File(fromField.getText()).getParent();
    if(parentDir == null) {
      toField.setText("Result");
    }
    else if(parentDir.endsWith(File.separator)) {
      toField.setText(parentDir + "Result");
    }
    else {
      toField.setText(parentDir + File.separatorChar + "Result");
    }

    File f = new File(fromField.getText());

    packagesBoundsField.setText(PackagesBounds.DEFAULT_BOUNDS);
    classpathBoundsField.setText(ClasspathBounds.DEFAULT_BOUNDS);

    startButton.addActionListener(new StartListener());
    browseButton.addActionListener(new BrowseListener());

    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(label1);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(fromField);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(browseButton);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel4 = new JPanel();
    panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));
    panel4.add(label2);
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));
    panel4.add(toField);
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(label3);
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(actionField);
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel3 = new JPanel();
    panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(label4);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(currFileField);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(20, 0)));
    panel5.add(startButton);
    panel5.add(Box.createRigidArea(new Dimension(20, 0)));
    panel5.add(closeButton);
    panel5.add(Box.createRigidArea(new Dimension(20, 0)));

    JPanel taskPanel = new JPanel();
    taskPanel.setLayout(new BoxLayout(taskPanel, BoxLayout.Y_AXIS));
    taskPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    taskPanel.add(panel1);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    taskPanel.add(panel4);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    taskPanel.add(panel3);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    taskPanel.add(panel2);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 50)));
    taskPanel.add(panel5);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 40)));

    return taskPanel;
  }

  private JPanel fillOptionsPanel() {
    JLabel label0 = new JLabel("Main Class   :");
    JLabel label1 = new JLabel("CLASSPATH    :");
    JLabel label2a = new JLabel("CLASSPATH bounds :");
    JLabel label2b = new JLabel("Packages bounds  :");
    JLabel label2c = new JLabel("Archive name :");

    Font font = new Font("Monospaced", Font.BOLD, 14);

    label0.setFont(font);
    label1.setFont(font);
    label2a.setFont(font); label2b.setFont(font); label2c.setFont(font);

    JButton loadButton = new JButton("Load");
    JButton saveButton = new JButton("Save");

    loadButton.addActionListener(new LoadListener());
    saveButton.addActionListener(new SaveListener());

    createArchiveCheck.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if(e.getStateChange() == ItemEvent.SELECTED) {
          archiveField.setEnabled(true);
        }
        else if(e.getStateChange() == ItemEvent.DESELECTED) {
          archiveField.setEnabled(false);
        }
      }
    });

    JPanel panel0 = new JPanel();
    panel0.setLayout(new BoxLayout(panel0, BoxLayout.X_AXIS));
    panel0.add(Box.createRigidArea(new Dimension(10, 0)));
    panel0.add(label0);
    panel0.add(Box.createRigidArea(new Dimension(10, 0)));
    panel0.add(mainClassField);
    panel0.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(label1);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(classPathField);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(createArchiveCheck);
    panel2.add(Box.createGlue());
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2c = new JPanel();
    panel2c.setLayout(new BoxLayout(panel2c, BoxLayout.X_AXIS));
    panel2c.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2c.add(label2c);
    panel2c.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2c.add(archiveField);
    panel2c.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2a = new JPanel();
    panel2a.setLayout(new BoxLayout(panel2a, BoxLayout.X_AXIS));
    panel2a.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2a.add(label2a);
    panel2a.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2a.add(classpathBoundsField);
    panel2a.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2b = new JPanel();
    panel2b.setLayout(new BoxLayout(panel2b, BoxLayout.X_AXIS));
    panel2b.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2b.add(label2b);
    panel2b.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2b.add(packagesBoundsField);
    panel2b.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel4 = new JPanel();
    panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));
    panel4.add(loadButton);
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));
    panel4.add(saveButton);
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));

    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.Y_AXIS));
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    optionsPanel.add(panel0);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel1);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel2);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel2c);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel2b);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel2a);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 60)));
    optionsPanel.add(panel4);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 40)));

    return optionsPanel;
  }

  LongOperation launcher1;

static CompactTaskDescription td = new CompactTaskDescription();

class LoadListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    JFileChooser chooser =
      DescriptionChooser.getInstance(".cmp", "Compact files *.cmp");

    int returnVal = chooser.showOpenDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      File f = chooser.getSelectedFile();

      try {
        td.load(f.getPath());
      }
      catch(Exception e) {
        JOptionPane.showMessageDialog(CompactFrame.this,
             "Error reading file " + f.getPath() + ".",
             "Error", JOptionPane.WARNING_MESSAGE);
        return;
      }

      fromField.setText(td.getFrom());
      toField.setText(td.getTo());
      classPathField.setText(td.getClassPath());
      mainClassField.setText(td.getMainClass());
      String name = td.getArchiveName();
      archiveField.setText(name);
      if(name == null)
        createArchiveCheck.setSelected(false);
      else
        createArchiveCheck.setSelected(true);

      classpathBoundsField.setText(td.getClasspathBounds());
      packagesBoundsField.setText(td.getPackagesBounds());
    }
  }
}


class SaveListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    JFileChooser chooser =
                 DescriptionChooser.getInstance(".cmp", "Compact files *.cmp");
    int returnVal = chooser.showSaveDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      File f = chooser.getSelectedFile();

      td.setFrom(fromField.getText());
      td.setTo(toField.getText());
      td.setClassPath(classPathField.getText());
      td.setMainClass(mainClassField.getText());
      td.setArchiveName(archiveField.getText());
      td.setClasspathBounds(classpathBoundsField.getText());
      td.setPackagesBounds(packagesBoundsField.getText());

      String fileName = f.getPath();
      if(!fileName.endsWith(".cmp")) fileName = fileName + ".cmp";

      try {
        td.save(fileName);
      }
      catch(Exception e) {
        JOptionPane.showMessageDialog(CompactFrame.this,
             "Error writing file " + fileName + ".",
             "Error", JOptionPane.WARNING_MESSAGE);
        return;
      }
    }
  }
}


class StartListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    String text = startButton.getText();
    if(text.equals("Start")) {

      String classPath = classPathField.getText();
      String mainClass = mainClassField.getText();

      if(classPath == null || classPath.length() == 0) {
        JOptionPane.showMessageDialog(CompactFrame.this,
              "Please define CLASSPATH variable in Configuration panel",
               "Warning", JOptionPane.WARNING_MESSAGE);
        return;
      }

      if(mainClass == null || mainClass.length() == 0) {
        JOptionPane.showMessageDialog(CompactFrame.this,
              "Please define main class in Configuration panel",
               "Warning", JOptionPane.WARNING_MESSAGE);
        return;
      }

      Runnable runnable = new Runnable() {
        public void run() {
          launcher1 = new CompactLongOperation();

          setCursor(new Cursor(Frame.WAIT_CURSOR));
          setClosable(false);
          startButton.setText("Cancel");
          browseButton.setEnabled(false);
          fromField.setEnabled(false);
          toField.setEnabled(false);
          closeButton.setEnabled(false);
          tabbedPane.setEnabledAt(1, false);
        }
      };
      SwingUtilities.invokeLater(runnable);
    }
    else if(text.equals("Cancel")) {
      if(launcher1 != null) {
        launcher1.stop();
        launcher1 = null;
      }
    }
  }
}

Vector classesList, dllList;

class CompactLongOperation extends LongOperation {

  CompactLongOperation() {
    super(1000);

    super.startLoading();
  }

  protected void execute() throws Exception {
    String mainClass = mainClassField.getText();
    String from      = fromField.getText();
    String to        = toField.getText();
    String classPath = classPathField.getText() + File.pathSeparator + from;
    String archiveName     = archiveField.getText();
    String classpathBounds = classpathBoundsField.getText();
    String packagesBounds  = packagesBoundsField.getText();

    // Create helping classes

    Archiver archiver = new Archiver();
    Copier copier     = new Copier();

    DependenciesFinder finder =
                new CompleteFinder(classPath, from, to, archiver, copier);
//  finder = new MethodFinder(classPath, toDir, archiver, copier);

    finder.setClasspathBounds(classpathBoundsField.getText());
    finder.setPackagesBounds(packagesBoundsField.getText());

    archiver.addNotificationListener( new NotificationListener() {
      public void notification(NotificationEvent event) {
        currFileField.setText(event.getNotification());
        actionField.setText(event.getDescription());
      }
    });

    copier.addNotificationListener(new NotificationListener() {
      public void notification(NotificationEvent event) {
        currFileField.setText(event.getNotification());
        actionField.setText(event.getDescription());
      }
    });

    finder.addNotificationListener(new NotificationListener() {
      public void notification(NotificationEvent event) {
        currFileField.setText(event.getNotification());
        actionField.setText(event.getDescription());
      }
    });

    finder.setClasspathBounds(classpathBounds);
    finder.setPackagesBounds(packagesBounds);

    finder.analyze(mainClass);

    classesList = finder.getFoundedClasses();
    dllList     = finder.getDllList();

    // zip founded classes
    if(archiveName != null && archiveName.length() > 0) {
      actionField.setText("Zipping...");
      archiver.zip(classesList, to, to + "/" + archiveName);
    }
  }

  public synchronized void stopLoading() {
    super.stopLoading();

    int type = getResultType();
    if(type == LongOperation.OK_OPERATION) {
      StringBuffer sb = new StringBuffer();
      for(int i=0; i < dllList.size(); i++) {
        if(i > 0) sb.append(" ");

        sb.append(dllList.elementAt(i));
      }
      JOptionPane.showMessageDialog(CompactFrame.this,
            "Compact process was succesfully finished." +
             "\n" + classesList.size() + " files founded." +
             "\n" + dllList.size() + " dynamic libraries discovered" +
             ((dllList.size() == 0) ? "." : ":\n" + "(" + sb + ")."),
             "Success", JOptionPane.WARNING_MESSAGE);
    }
    else if(type == LongOperation.ERROR_OPERATION) {
      JOptionPane.showMessageDialog(CompactFrame.this,
                  "Error during file processing \n" + getResultMessage(),
                  "Failure", JOptionPane.ERROR_MESSAGE);
    }

    setCursor(new Cursor(Frame.DEFAULT_CURSOR));
    setClosable(true);
    startButton.setText("Start");
    browseButton.setEnabled(true);
    fromField.setEnabled(true);
    toField.setEnabled(true);
    tabbedPane.setEnabledAt(1, true);
    closeButton.setEnabled(true);
    actionField.setText("");
    currFileField.setText("");
  }

}


class BrowseListener implements ActionListener {
  public void actionPerformed(ActionEvent e) {
    int returnVal = chooser.showOpenDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      setSelectedFile();
      setClassPath();
    }
  }
}

}



