// ClassHoundFrame.java

package org.javalobby.cafebabe.plugin.houndplug;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import org.javalobby.mdi.*;
import org.javalobby.util.*;
import org.javalobby.cafebabe.*;
import org.javalobby.cafebabe.archooser.*;
import org.javalobby.cafebabe.plugin.houndplug.treecombo.*;

// internal frame which supports class-hound service

public class ClassHoundFrame extends PluginFrame implements HoundPlugConstants {

  private ImageIcon classIcon     = IconProducer.getImageIcon(ICON_HIERARCHY_CLASS, ClassHoundFrame.class);
  private ImageIcon interfaceIcon = IconProducer.getImageIcon(ICON_HIERARCHY_INTERFACE, ClassHoundFrame.class);
  private ImageIcon unknownIcon   = IconProducer.getImageIcon(ICON_HIERARCHY_UNKNOWN, ClassHoundFrame.class);

  private final JComboBox entriesCombo = new JComboBox();
  private final JList packagesList     = new JList();
  private final JList classesList      = new JList();
  private final HierarchyTreeCombo hierarchyCombo = new HierarchyTreeCombo();
  private final JList fieldsList       = new JList();
  private final JList methodsList      = new JList();

  private final JButton upButton       = new JButton("Up");
  private final JButton downButton     = new JButton("Down");
  private final JButton addButton      = new JButton("Add");
  private final JButton removeButton   = new JButton("Remove");
  private final JButton defaultsButton = new JButton("Set defaults");

  private ItemListener combosListener = new CombosListener();

  private final JButton selectButton = new JButton("Select");
  private final JButton closeButton  = new JButton("Close");

  private ClassHoundSingleton singleton;

  private String selectedEntry, selectedPackage, selectedClass,
          selectedMethod, selectedField;

  private ProgramChooser chooser = ProgramChooser.getInstance();

  public ClassHoundFrame() {
    setTitle("Class-Hound Service");

//    setFrameIcon(IconProducer.getImageIcon(Constants.ICON_HOUND));

    hierarchyCombo.setClassIcon(classIcon);
    hierarchyCombo.setInterfaceIcon(interfaceIcon);
    hierarchyCombo.setUnknownIcon(unknownIcon);

    entriesCombo.addItemListener(combosListener);
    hierarchyCombo.addItemListener(combosListener);

    ListSelectionListener listsListener = new ListsListener();

    packagesList.addListSelectionListener(listsListener);
    classesList.addListSelectionListener(listsListener);
    fieldsList.addListSelectionListener(listsListener);
    methodsList.addListSelectionListener(listsListener);

    fieldsList.setCellRenderer(new FieldCellRenderer());
    methodsList.setCellRenderer(new MethodCellRenderer());

    upButton.setActionCommand("up");
    downButton.setActionCommand("down");
    addButton.setActionCommand("add");
    removeButton.setActionCommand("remove");
    defaultsButton.setActionCommand("defaults");

    EntriesListener entriesListener = new EntriesListener();

    upButton.addActionListener(entriesListener);
    downButton.addActionListener(entriesListener);
    addButton.addActionListener(entriesListener);
    removeButton.addActionListener(entriesListener);
    defaultsButton.addActionListener(entriesListener);

    MouseListener dcListener = new DoubleClickListener();

    entriesCombo.addMouseListener(dcListener);
    packagesList.addMouseListener(dcListener);
    classesList.addMouseListener(dcListener);
    fieldsList.addMouseListener(dcListener);
    methodsList.addMouseListener(dcListener);

    selectButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        select(selectedEntry, selectedPackage, selectedClass, selectedMethod, selectedField);

      }
    });

    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });

    Dimension dim = new Dimension(350, 125);

    JPanel panel11 = new JPanel();
    panel11.setLayout(new BoxLayout(panel11, BoxLayout.Y_AXIS));
    JScrollPane sp111 = new JScrollPane(packagesList);
    sp111.setBorder(new TitledBorder(new EtchedBorder(), "Packages"));
    sp111.setMaximumSize(dim);
    sp111.setMinimumSize(dim);
//    sp111.setPreferredSize(dim);
    JScrollPane sp112 = new JScrollPane(classesList);
    sp112.setBorder(new TitledBorder(new EtchedBorder(), "Classes"));
    sp112.setMaximumSize(dim);
    sp112.setMinimumSize(dim);
//    sp112.setPreferredSize(dim);
    panel11.add(sp111);
    panel11.add(sp112);

    JPanel panel12 = new JPanel();
    panel12.setLayout(new BoxLayout(panel12, BoxLayout.Y_AXIS));
    JScrollPane sp121 = new JScrollPane(fieldsList);
    sp121.setBorder(new TitledBorder(new EtchedBorder(), "Fields"));
    sp121.setMaximumSize(dim);
    sp121.setMinimumSize(dim);
//    sp121.setPreferredSize(dim);
    JScrollPane sp122 = new JScrollPane(methodsList);
    sp122.setBorder(new TitledBorder(new EtchedBorder(), "Methods"));
    sp122.setMaximumSize(dim);
    sp122.setMinimumSize(dim);
//    sp122.setPreferredSize(dim);

//    hierarchyCombo.setMaximumSize(new Dimension(500, 20));
//    hierarchyCombo.setMinimumSize(new Dimension(500, 20));

//    panel12.add(hierarchyCombo);
    panel12.add(sp121);
    panel12.add(sp122);

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.setBorder(new TitledBorder(new EtchedBorder(), "Entries"));

    panel2.add(entriesCombo);
    panel2.add(Box.createRigidArea(new Dimension(3, 0)));
    panel2.add(upButton);
    panel2.add(Box.createRigidArea(new Dimension(3, 0)));
    panel2.add(downButton);
    panel2.add(Box.createRigidArea(new Dimension(3, 0)));
    panel2.add(addButton);
    panel2.add(Box.createRigidArea(new Dimension(3, 0)));
    panel2.add(removeButton);
    panel2.add(Box.createRigidArea(new Dimension(3, 0)));
    panel2.add(defaultsButton);

    JPanel panel3 = new JPanel();
    panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
    panel3.add(Box.createRigidArea(new Dimension(20, 0)));
    panel3.add(selectButton);
    panel3.add(Box.createRigidArea(new Dimension(20, 0)));
    panel3.add(closeButton);

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(3, 0)));
    panel5.add(hierarchyCombo);
    panel5.add(Box.createRigidArea(new Dimension(3, 0)));

    JPanel panel1 = new JPanel();
    JSplitPane splitPane =
               new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel11, panel12);
    splitPane.setDividerLocation(320);

    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(3, 0)));
    panel1.add(splitPane);
    panel1.add(Box.createRigidArea(new Dimension(3, 0)));

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
    this.setContentPane(topPanel);
    topPanel.add(panel2);
    topPanel.add(panel5);
    topPanel.add(panel1);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(panel3);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    selectButton.requestFocus();

    init();
  }

  public void open() {
    // Set the characteristics for this dialog instance
    Rectangle r = getDesktopPane().getBounds();
    this.setBounds(r.x + r.width/2 - 750/2, r.y + r.height/2 - 520/2, 750, 430);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    super.open();
  }

  public void load(ProgramState programState) {
    setBounds(programState.getBoundsProperty("ClassHoundFrame", "20;20;500;400"));
  }

  public void save(ProgramState programState) {
    programState.putBoundsProperty("ClassHoundFrame", getBounds());
  }

  private void init() {
    Thread t = new Thread() {
      public void run() {
        singleton = ClassHoundSingleton.getInstance();

        entriesCombo.setModel(new DefaultComboBoxModel(singleton.getEntries()));
        entriesCombo.setSelectedIndex(0);

        combosListener.itemStateChanged(
          new ItemEvent(entriesCombo, ItemEvent.ITEM_STATE_CHANGED,
                        entriesCombo.getSelectedItem(), ItemEvent.SELECTED)
        );
      }
    };
    t.setPriority(Thread.NORM_PRIORITY-1);
    t.start();
  }

  private void fillPackagesList(String cpEntry) {
    selectedEntry   = cpEntry;
    selectedPackage = null;
    selectedClass   = null;
    selectedMethod  = null;
    selectedField   = null;

    singleton.fillPackagesList(selectedEntry);

    packagesList.setListData(singleton.getPackages());
    classesList.setListData(singleton.getClasses());
    hierarchyCombo.setModel(new DefaultComboBoxModel(new Object[0]));
    fieldsList.setListData(singleton.getFields());
    methodsList.setListData(singleton.getMethods());
  }

  private void fillClassesList(String packageName) {
    selectedPackage = packageName;
    selectedClass = selectedMethod = selectedField = null;

    singleton.fillClassesList(selectedEntry, selectedPackage);

    classesList.setListData(singleton.getClasses());
    hierarchyCombo.setModel(new DefaultComboBoxModel(new Object[0]));
    fieldsList.setListData(singleton.getFields());
    methodsList.setListData(singleton.getMethods());
  }

  private void fillMembersLists(String className) {
    selectedClass = className;
    selectedMethod = selectedField = null;

    singleton.fillMembersLists(selectedEntry, selectedPackage, selectedClass);

    boolean isAnonymous = (selectedPackage == ClassHoundSingleton.ANONYMOUS_PACKAGE);

    String qualifiedClassName = null;
    if(isAnonymous)
      qualifiedClassName = selectedClass;
    else
      qualifiedClassName = selectedPackage + "." + selectedClass;

    hierarchyCombo.setClass(selectedEntry, qualifiedClassName);
    hierarchyCombo.setSelectedItem(qualifiedClassName);

    fieldsList.setListData(singleton.getFields());
    methodsList.setListData(singleton.getMethods());
  }

  // supports class-hound search process
  public void select(String selectedEntry, String selectedPackage,
         String selectedClass, String selectedMethod, String selectedField) {

    if(selectedEntry == null) return;

    if(selectedPackage == null) return;

    if(selectedClass == null) {
      selectClass(selectedEntry, selectedPackage);
    }
    else {
      selectMember(selectedEntry, selectedPackage, selectedClass,
                   selectedField, selectedMethod);
    }
  }

  private void selectClass(String selectedEntry, String selectedPackage) {
    boolean isAnonymous = (selectedPackage == ClassHoundSingleton.ANONYMOUS_PACKAGE);
    boolean isArchive   = !(new File(selectedEntry)).isDirectory();

    String selectedDir = "";
    if(!isAnonymous)
      selectedDir = selectedPackage.replace('.', '/') + "/";

    try {
      File f = null;
      if(isArchive) {
        f = new ArchivedFile(selectedEntry, selectedDir);
        if(isAnonymous) {
          f = new File(f.getParent());
          chooser.setSelectedFile(f);
          chooser.setCurrentDirectory(new File(f.getParent()));
        }
        else {
          chooser.setSelectedFile(new File(""));
          chooser.setCurrentDirectory(f);
        }
      }
      else {
        f = new File(new File(selectedEntry + File.separator + selectedDir).getAbsolutePath());
        chooser.setSelectedFile(new File(""));
        chooser.setCurrentDirectory(f);
      }

      chooser.openFile();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void selectMember(String selectedEntry, String selectedPackage,
                      String selectedClass,
                      String selectedField, String selectedMethod) {

    boolean isAnonymous = (selectedPackage == ClassHoundSingleton.ANONYMOUS_PACKAGE);
    boolean isArchive   = !(new File(selectedEntry)).isDirectory();

    String selectedDir = "";
    if(!isAnonymous) {
      selectedDir = selectedPackage.replace('.', '/') + "/";
    }

    String name = selectedClass + ".class";
    if(!isAnonymous) {
      name = selectedDir + name;
    }

    File f = null;
    try {
      if(isArchive) {
        f = new ArchivedFile(selectedEntry, name);
      }
      else {
        f = new File(new File(selectedEntry + File.separator + name).getAbsolutePath());
      }
    }
    catch(Exception e) {
      e.printStackTrace();
      return;
    }

    chooser.setSelectedFile(f);
    chooser.setCurrentDirectory(f);

    chooser.openFile(f);

    JFrame frame = (JFrame)this.getTopLevelAncestor();

    if(frame instanceof DesktopFrame) {
      DesktopFrame desktopFrame = (DesktopFrame)frame;
      JInternalFrame internalFrame = (JInternalFrame)desktopFrame.getCurrentFrame();

      ZipClassLoader loader = ((ZipClassLoader)internalFrame.getClass().getClassLoader());

      try {
        Class clazz = loader.loadClass("org.javalobby.cafebabe.plugin.classplug.ClassFrame", true);

        if(selectedField != null) {
          if(clazz.isInstance(internalFrame)) {
            Method method = clazz.getMethod("setFieldPosition", new Class[] {String.class});
            method.invoke(internalFrame, new Object[] {selectedField});
          }
        }
        else if(selectedMethod != null) {
          if(clazz.isInstance(internalFrame)) {
            Method method = clazz.getMethod("setMethodPosition", new Class[] {String.class});
            method.invoke(internalFrame, new Object[] {selectedMethod});
          }
        }
      }
      catch(Exception e) {
        e.printStackTrace();
      }
    }
  }


class DoubleClickListener extends MouseAdapter {
  public void mouseClicked(MouseEvent event) {
    Object source = event.getSource();

    if(source instanceof JList) {
      JList list = (JList)source;
      int index = list.getSelectedIndex();
      if(index != -1)
        if(event.getClickCount() == 2) {
          select(selectedEntry, selectedPackage, selectedClass, selectedMethod, selectedField);
        }
    }
  }
}

class EntriesListener implements ActionListener {
  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();
    if(cmd.equals("up")) {
      int index = entriesCombo.getSelectedIndex();
      if(index > 0)
        entriesCombo.setSelectedIndex(--index);
    }
    else if(cmd.equals("down")) {
      int index = entriesCombo.getSelectedIndex();
      if(index >= 0 && index < entriesCombo.getItemCount()-1)
        entriesCombo.setSelectedIndex(++index);
    }
    else if(cmd.equals("add")) {
      File file = chooser.openChooser("Select directory or archive");

      if(file != null) {
        String name = file.getName().toLowerCase();
        boolean isArchive = (name.endsWith(".jar") || name.endsWith(".zip"));

        if(file != null && (file.isDirectory() || isArchive)) {
          String newEntryName = file.getPath();
          Hashtable entryToFilesMap = singleton.getEntryToFilesMap();
          Vector files = (Vector)entryToFilesMap.get(newEntryName);
          if(files == null) {
            entriesCombo.addItem(newEntryName);
            singleton.addEntry(newEntryName);
            entriesCombo.setSelectedIndex(entriesCombo.getItemCount()-1);
            combosListener.itemStateChanged(
               new ItemEvent(entriesCombo, ItemEvent.ITEM_STATE_CHANGED,
                             entriesCombo.getSelectedItem(),ItemEvent.SELECTED)
            );
          }
        }
      }
    }
    else if(cmd.equals("remove")) {
      String item = (String)entriesCombo.getSelectedItem();
      int index = entriesCombo.getSelectedIndex();
      if(index >= 0) {
        entriesCombo.removeItemAt(index);
        singleton.removeEntry(item);
      }
    }
    else if(cmd.equals("defaults")) {
      init();
    }
  }

}

class ListsListener implements ListSelectionListener {

  public void valueChanged(ListSelectionEvent event) {
    Object source = event.getSource();

    if(source == packagesList) {
      // fill classes list
      fillClassesList((String)packagesList.getSelectedValue());
    }
    else if(source == classesList) {
      // fill methods list
      fillMembersLists((String)classesList.getSelectedValue());
    }
    else if(source == fieldsList) {
      selectedMethod = null;
      methodsList.clearSelection();
      selectedField  = (String)fieldsList.getSelectedValue();
    }
    else if(source == methodsList) {
      selectedField  = null;
      fieldsList.clearSelection();
      selectedMethod = (String)methodsList.getSelectedValue();
    }
  }
}


class CombosListener implements ItemListener {
  public void itemStateChanged(ItemEvent event) {
    if(event.getID() == ItemEvent.DESELECTED) return;

    Object source = event.getSource();
    if(source == entriesCombo) {
      // fill packages list
      if(entriesCombo.getSelectedIndex() >= 0)
        fillPackagesList((String)entriesCombo.getSelectedItem());
      else {
        selectedEntry   = null;
        selectedPackage = null;
        selectedClass   = null;
        selectedMethod  = null;
        selectedField   = null;

        singleton.clearPackages();
        singleton.clearClasses();
        singleton.clearMembers();

        packagesList.setListData(singleton.getPackages());
        classesList.setListData(singleton.getClasses());
        hierarchyCombo.setModel(new DefaultComboBoxModel(new Object[0]));
        fieldsList.setListData(singleton.getFields());
        methodsList.setListData(singleton.getMethods());
      }
    }
    else if(source == hierarchyCombo) {
      String item = hierarchyCombo.getSelectedItem().toString();

      String packagePart = null,
             classPart   = null;
      int pos = item.lastIndexOf(".");
      if(pos == -1) {
        packagePart = ClassHoundSingleton.ANONYMOUS_PACKAGE;
        classPart    = item;
      }
      else {
        packagePart = item.substring(0, pos);
        classPart    = item.substring(pos+1);
      }

      Vector packages = singleton.getPackages();
      if(packages.contains(packagePart)) {
        packagesList.setSelectedValue(packagePart, true);
        classesList.setSelectedValue(classPart, true);
      }
      else {
        String searchingFileName = item.replace('.', '/') + ".class";
        Vector entries = singleton.getEntries();
        for(int i=0; i < entries.size(); i++) {
          String iEntry = (String)entries.elementAt(i);
          boolean isArchive = !(new File(iEntry)).isDirectory();

          if(!iEntry.equals(selectedEntry)) {

            Hashtable entryToFilesMap = singleton.getEntryToFilesMap();
            Vector files = (Vector)entryToFilesMap.get(iEntry);
            for(int j=0; j < files.size(); j++) {
              String iFileName = ((String)files.elementAt(j));
              iFileName = iFileName.replace(File.separatorChar, '/');
              if(searchingFileName.equals(iFileName)) {
                entriesCombo.setSelectedItem(iEntry);
                packagesList.setSelectedValue(packagePart, true);
                classesList.setSelectedValue(classPart, true);
                return;
              }
            }
          }
        }
      }
    }
  }
}


  public static void main(String args[]) {
    ProgramFrame frame = new ProgramFrame();
    MdiDesktopPane desktopPane = new MdiDesktopPane(new MdiMenuBar());

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());

    topPanel.add(desktopPane, BorderLayout.CENTER);

    frame.setContentPane(topPanel);

    frame.setBounds(10, 10, 700, 500);

    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { System.exit(0); }
    });

    ClassHoundFrame hound = new ClassHoundFrame();
    desktopPane.add(hound, JLayeredPane.PALETTE_LAYER);
    hound.setBounds(5, 5, 500, 400);

    frame.setVisible(true);
  }

}

/**
* Run the given class's main() method with the arguments given
* after loading it from the class loader.
*
* @param loader The class loader to use.
* @param className The fully qualified name of the class to run.
* @param args The arguments that will be passed to main() in the class.
* @exception java.lang.Throwable Anything that gets thrown by this will
*            just be passed along.  This catches nothing.
*/
/*public static void runClass(java.lang.ClassLoader loader,String className,String[] args) throws java.lang.Throwable
{
// The argument type array we need to find the right method
// from the class we will load.
Class[] argTypes={(new String[0]).getClass()};

// The argument array we need for the reflection work...
Object[] arg={args};

// Using the loader provided (we don't check it)
// load the class name given and find the main(String[])
// method and invoke it with the arguments given.
loader.loadClass(className).getMethod("main",argTypes).invoke(null,arg);
}
*/