// MethodCellRenderer.java

package org.javalobby.cafebabe.plugin.houndplug;

import java.awt.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

class MethodCellRenderer extends JLabel implements ListCellRenderer {

  protected Border noFocusBorder = new EmptyBorder(1, 1, 1, 1);

  public Component getListCellRendererComponent(JList list, Object value,
                     int index, boolean isSelected, boolean cellHasFocus) {
    String s = value.toString();
    this.setText(s);

    if (isSelected) {
      this.setBackground(list.getSelectionBackground());
      this.setForeground(list.getSelectionForeground());
    }
    else {
      this.setBackground(list.getBackground());
      this.setForeground(list.getForeground());
    }
    this.setFont(list.getFont());

    if(cellHasFocus)
      this.setBorder(UIManager.getBorder("List.focusCellHighlightBorder"));
    else
      this.setBorder(noFocusBorder);

    return this;
  }

  public void paint(Graphics g) {
    FontMetrics m_fm = g.getFontMetrics();

    g.setColor(this.getBackground());
    g.fillRect(0, 0, getWidth(), getHeight());
    getBorder().paintBorder(this, g, 0, 0, getWidth(), getHeight());

    g.setColor(this.getForeground());
    g.setFont(getFont());
    Insets m_insets = getInsets();
    int x = m_insets.left;
    int y = m_insets.top + m_fm.getAscent();

    StringTokenizer st = new StringTokenizer(getText());
    int cnt = st.countTokens();

    while(st.hasMoreTokens()) {
      String token = st.nextToken();

      int pos = token.indexOf("(");
      if(pos != -1) {
        String methodName = token.substring(0, pos);
        Color oldColor = g.getColor();
        g.setColor(Color.blue);
        g.drawString(methodName, x, y);
        g.setColor(oldColor);
        g.drawString(token.substring(pos), x + m_fm.stringWidth(methodName), y);
      }
      else
        g.drawString(token, x, y);

      x += m_fm.stringWidth(token) + 5;

      if (!st.hasMoreTokens())
        break;
    }
  }

}
