// HierarchyTreeCombo.java

package org.javalobby.cafebabe.plugin.houndplug.treecombo;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.zip.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.border.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;

public class HierarchyTreeCombo extends JComboBox {

  static final int OFFSET = 16;

  private TreeModel treeModel;
//  private int levels;
  private ImageIcon classIcon, interfaceIcon, unknownIcon;

  public HierarchyTreeCombo() {
    super();
  }

  public void setModel(TreeModel treeModel) {
    this.treeModel = treeModel;

    setModel(new TreeToListModel(treeModel));
    setRenderer(new ListEntryRenderer());

//    levels = getLevelsNumber();
  }

  public void setClassIcon(ImageIcon classIcon) {
    this.classIcon = classIcon;
  }

  public void setInterfaceIcon(ImageIcon interfaceIcon) {
    this.interfaceIcon = interfaceIcon;
  }

  public void setUnknownIcon(ImageIcon unknownIcon) {
    this.unknownIcon = unknownIcon;
  }

  public void setClass(String selectedEntry, String className) {
    DefaultMutableTreeNode root = null;

    Class clazz = null;
    try {
      clazz = Class.forName(className);
      if(clazz.isInterface())
        root = new InterfaceNode(className);
      else
        root = new ClassNode(className);

      getSuperclasses(clazz, root);
      getInterfaces(clazz, root);
    }
    catch(Exception e) {
      root = setClass0(selectedEntry, className);
    }

    setModel(new DefaultTreeModel(root));
  }

  private void getSuperclasses(Class clazz, DefaultMutableTreeNode currNode) {
    for(Class c=clazz.getSuperclass(); c != null; c = c.getSuperclass()) {
      ClassNode iNode = new ClassNode(c.getName());
      currNode.add(iNode);
      currNode = iNode;
    }
  }

  private void getInterfaces(Class clazz, DefaultMutableTreeNode currNode) {
    Class[] interfaces = clazz.getInterfaces();

    int len = interfaces.length;
    for (int i=0; i < len; i++) {
      Class interfaze =interfaces[i];
      InterfaceNode iNode = new InterfaceNode(interfaze.getName());
      currNode.add(iNode);
      getInterfaces(interfaze, iNode);
    }
  }

  private DefaultMutableTreeNode setClass0(String selectedEntry,
                                           String className) {
    DefaultMutableTreeNode root = null;

    try {
      ClassFile classFile = readFromClassFile(selectedEntry, className);

      if(classFile.getAccessFlags().isInterface())
        root = new InterfaceNode(className);
      else
        root = new ClassNode(className);

      getSuperclasses0(classFile, selectedEntry, className, root);
      getInterfaces0(classFile, selectedEntry, className, root);
    }
    catch(Exception e2) {
      root = new UnknownNode(className);
    }

    return root;
  }

  private void getSuperclasses0(ClassFile classFile, String selectedEntry,
               String className, DefaultMutableTreeNode currNode) {

    short superClass = classFile.getSuperClass();

    while(superClass != 0) {
      ConstPool constPool = classFile.getConstPool();
      ClassConst entry    = (ClassConst)constPool.getEntry(superClass);
      Utf8Const utf       = (Utf8Const)constPool.getEntry(entry.getNameIndex());
      className           = utf.getValue().replace('/', '.');

      if(classFile.getAccessFlags().isInterface() &&
         className.equals("java.lang.Object"))
        break;

      try {
        classFile = readFromClassFile(selectedEntry, className);
      }
      catch(Exception e) {
        try {
          ClassNode iNode = new ClassNode(className);
          currNode.add(iNode);

          Class clazz = Class.forName(className);
          getSuperclasses(clazz, iNode);
        }
        catch(Exception e2) {}
        break;
      }

      ClassNode iNode = new ClassNode(className);
      currNode.add(iNode);
      currNode = iNode;

      superClass = classFile.getSuperClass();
    }
  }

  private void getInterfaces0(ClassFile classFile, String selectedEntry,
               String className, DefaultMutableTreeNode currNode) {
    EntryCollection interfaces = classFile.interfaces();
    ConstPool constPool = classFile.getConstPool();

    short len = (short)interfaces.size();
    for (short i=0; i < len; i++) {
      InterfaceEntry entry = (InterfaceEntry)interfaces.elementAt(i);
      ClassConst entry2    = (ClassConst)constPool.getEntry(entry.getNameIndex());
      Utf8Const utf        = (Utf8Const)constPool.getEntry(entry2.getNameIndex());
      className            = utf.getValue().replace('/', '.');

      try {
        classFile = readFromClassFile(selectedEntry, className);
      }
      catch(Exception e) {
        try {
          InterfaceNode iNode = new InterfaceNode(className);
          currNode.add(iNode);

          Class clazz = Class.forName(className);
          getInterfaces(clazz, iNode);
        }
        catch(Exception e2) {}
        continue;
      }

      InterfaceNode iNode = new InterfaceNode(className);
      currNode.add(iNode);

      getInterfaces0(classFile, selectedEntry, className, iNode);
    }
  }

  private ClassFile readFromClassFile(String selectedEntry, String className)
               throws Exception {
    ClassFile classFile = new ClassFile();
    boolean isArchive = !(new File(selectedEntry)).isDirectory();

    if(isArchive) {
      ZipFile zipFile   = new ZipFile(selectedEntry);
      String entryName  = className.replace('.', '/') + ".class";
      ZipEntry zipEntry = zipFile.getEntry(entryName);

      classFile.read(zipFile.getInputStream(zipEntry));
    }
    else {
      String fileName = className.replace('.', File.separatorChar) + ".class";
      classFile.read(selectedEntry + File.separator + fileName);
    }

    return classFile;
  }


/*
  int maxLevel;

  public int getLevelsNumber() {
    maxLevel = 1;
    if(treeModel != null)
      calculateLevels(treeModel.getRoot(), 1);

    return maxLevel;
  }

  private void calculateLevels(Object node, int currLevel) {

    int childrenCnt = treeModel.getChildCount(node);

    if(childrenCnt == 0) {
//      ++currLevel;
      if(maxLevel < currLevel)
        maxLevel = currLevel;
      return;
    }

    for(int i=0; i < childrenCnt; i++) {
      calculateLevels(treeModel.getChild(node, i), currLevel+1);
    }
  }
*/

// inner class - data model
class TreeToListModel extends AbstractListModel
                      implements ComboBoxModel, TreeModelListener {
  TreeModel source;
  boolean invalid = true;
  Object currentValue;
  Vector cache = new Vector();

  public TreeToListModel(TreeModel aTreeModel) {
    source = aTreeModel;
    aTreeModel.addTreeModelListener(this);
    setRenderer(new ListEntryRenderer());
  }

  public void setSelectedItem(Object anObject) {
    currentValue = anObject;
    fireContentsChanged(this, -1, -1);
  }

  public Object getSelectedItem() {
    return currentValue;
  }

  public int getSize() {
    validate();
    return cache.size();
  }

  public Object getElementAt(int index) {
    return cache.elementAt(index);
  }

  public void treeNodesChanged(TreeModelEvent e)     { invalid = true; }
  public void treeNodesInserted(TreeModelEvent e)    { invalid = true; }
  public void treeNodesRemoved(TreeModelEvent e)     { invalid = true; }
  public void treeStructureChanged(TreeModelEvent e) { invalid = true; }

  void validate() {
    if(invalid) {
      cache = new Vector();
      cacheTree(source.getRoot(), 0);
      if(cache.size() > 0)
        currentValue = cache.elementAt(0);
      invalid = false;
      fireContentsChanged(this, 0, 0);
    }
  }

  void cacheTree(Object anObject, int level) {
    if(source.isLeaf(anObject))
      addListEntry(anObject, level);
    else {
      int c = source.getChildCount(anObject);
      int i;
      Object child;

      addListEntry(anObject, level);
      level++;

      for(i=0;i<c;i++) {
        child = source.getChild(anObject, i);
        cacheTree(child, level);
      }

      level--;
    }
  }

  void addListEntry(Object anObject,int level) {
    cache.addElement(new ListEntry(anObject, level));
  }

}

class ListEntry {
  Object object;
  int level;

  public ListEntry(Object object, int level) {
    this.object = object;
    this.level  = level;
  }

  public Object getObject()  { return object; }
  public int getLevel()      { return level; }
  public String toString()   { return object.toString(); }
}


class ListEntryRenderer extends JLabel implements ListCellRenderer  {
  final Border emptyBorder = new EmptyBorder(0,0,0,0);

  public ListEntryRenderer() {
    this.setOpaque(true);
  }

  public Component getListCellRendererComponent(JList listbox, Object value,
                      int index, boolean isSelected, boolean cellHasFocus) {
    ListEntry listEntry = (ListEntry)value;
    if(listEntry != null) {
      Border border;
      this.setText(listEntry.getObject().toString());

      Object node = listEntry.getObject();
      if(node instanceof ClassNode && classIcon != null)
        this.setIcon(classIcon);
      else if(node instanceof InterfaceNode && interfaceIcon != null)
        this.setIcon(interfaceIcon);
      else if(node instanceof UnknownNode && unknownIcon != null)
        this.setIcon(unknownIcon);

      if(index != -1)
        border = new EmptyBorder(0, OFFSET*listEntry.getLevel(), 0, 0);
      else
        border = emptyBorder;

      if(UIManager.getLookAndFeel().getName().equals("CDE/Motif")) {
        if(index == -1 )
          this.setOpaque(false);
        else
          this.setOpaque(true);
      }
      else
        this.setOpaque(true);

      this.setBorder(border);

      if (isSelected) {
        this.setBackground(UIManager.getColor("ComboBox.selectionBackground"));
        this.setForeground(UIManager.getColor("ComboBox.selectionForeground"));
      }
      else {
        this.setBackground(UIManager.getColor("ComboBox.background"));
        this.setForeground(UIManager.getColor("ComboBox.foreground"));
      }
    }
    else {
      this.setText("");
      this.setIcon(null);
    }

    return this;
  }
}

  public static void main(String args[]) {
    JFrame frame = new JFrame();

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));

    frame.setContentPane(topPanel);

    frame.setBounds(10, 10, 700, 500);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { System.exit(0); }
    });

    HierarchyTreeCombo hierarchyCombo = new HierarchyTreeCombo();

    hierarchyCombo.setClass("D:\\swing\\swingall.jar", "javax.swing.JFrame");

//    System.out.println("l " + hierarchyCombo.getLevelsNumber());

    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(hierarchyCombo);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    frame.setVisible(true);
  }

}
