// MigrateFrame.java

package org.javalobby.cafebabe.plugin.migrateplug;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

import org.javalobby.classfile.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.event.*;
import org.javalobby.task.migrator.*;
import org.javalobby.mdi.*;
import org.javalobby.cafebabe.*;
import org.javalobby.util.*;

// internal frame which help to process migration task

public class MigrateFrame extends PluginFrame {

  final JButton startButton      = new JButton("Start");
  final JButton browseButton     = new JButton("Browse");
  final JButton closeButton      = new JButton("Close");

  final JTextField fromField        = new JTextField();
  final JCheckBox  recurseCheck     = new JCheckBox("Recursively", true);
  final JTextField toField          = new JTextField();
  final JTextField currFileField    = new JTextField();
  final JTextField actionDescrField = new JTextField();
  final JTextField actionField      = new JTextField();

  JTabbedPane tabbedPane = new JTabbedPane();

  private File selectedFile;
  private GroupTask groupTask;

  private final MigrationRulesModel rulesModel = new MigrationRulesModel();
  private final JTable rulesTable              = new JTable(rulesModel);
  private final Vector data                    = rulesModel.getData();

  private ProgramChooser chooser = ProgramChooser.getInstance();

  public MigrateFrame() {
    setTitle("Migrate Frame");
    setResizable(false);
    setMaximizable(false);

    setSelectedFile();

    JPanel taskPanel = fillTaskPanel();
    JPanel optionsPanel = fillOptionsPanel();

    tabbedPane = new JTabbedPane();
    this.setContentPane(tabbedPane);

    tabbedPane.addTab("Task", null, taskPanel, "Perform migration task");
    tabbedPane.addTab("Migration Table", null, optionsPanel,
                      "Allows to setup migration table");

    tabbedPane.setSelectedIndex(0);

    startButton.requestFocus();
  }

  public void open() {
    // Set the characteristics for this dialog instance
    Rectangle r = getDesktopPane().getBounds();
    this.setBounds(r.x + r.width/2 - 650/2, r.y + r.height/2 - 490/2, 650, 360);

    super.open();
  }

  public void load(ProgramState programState) {
    setBounds(programState.getBoundsProperty("MigrateFrame", "20;20;500;400"));
  }

  public void save(ProgramState programState) {
    programState.putBoundsProperty("MigrateFrame", getBounds());
  }

  private void setSelectedFile() {
    File currentFile = chooser.getSelectedFile();

    if(currentFile == null) {
      try {
         currentFile = new File(new File(new File(".").getCanonicalPath()).getParent());
      }
      catch(Exception e) { e.printStackTrace(); }
    }
    else if(!currentFile.exists())
      currentFile = new File(new File(currentFile.getAbsolutePath()).getParent());

    String fileName = currentFile.toString();
    if(currentFile.isDirectory() ||
       fileName.endsWith("zip")  || fileName.endsWith("jar"))
      fromField.setText(fileName);
    else
      fromField.setText(new File(currentFile.getAbsolutePath()).getParent());
  }

  private JPanel fillTaskPanel() {
    JLabel label1 = new JLabel("From        :");
    JLabel label2 = new JLabel("To          :");
    JLabel label3 = new JLabel("File        :");
    JLabel label4 = new JLabel("Description :");
    JLabel label5 = new JLabel("Action      :");

    Font font  = new Font("Monospaced", Font.BOLD, 14);
    Font font2 = new Font("Monospaced", Font.BOLD, 12);

    label1.setFont(font);
    label2.setFont(font);
    label3.setFont(font);
    label4.setFont(font);
    label5.setFont(font);

    fromField.setFont(font);
    toField.setFont(font);
    currFileField.setFont(font2);
    actionDescrField.setFont(font2);
    actionField.setFont(font2);

    currFileField.setEnabled(false);
    actionDescrField.setEnabled(false);
    actionField.setEnabled(false);

    String parentDir = new File(fromField.getText()).getParent();
    if(parentDir == null) {
      toField.setText("Result");
    }
    else if(parentDir.endsWith(File.separator)) {
      toField.setText(parentDir + "Result");
    }
    else {
      toField.setText(parentDir + File.separatorChar + "Result");
    }

    actionField.setForeground(Color.green);

    startButton.addActionListener(new StartListener());

    browseButton.addActionListener(new BrowseListener());

    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(label1);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(fromField);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(browseButton);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel11 = new JPanel();
    panel11.setLayout(new BoxLayout(panel11, BoxLayout.X_AXIS));
    panel11.add(Box.createRigidArea(new Dimension(10, 0)));
    panel11.add(recurseCheck);
    panel11.add(Box.createGlue());

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(label2);
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(toField);
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel3 = new JPanel();
    panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(label3);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(currFileField);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel4 = new JPanel();
    panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));
    panel4.add(label4);
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));
    panel4.add(actionDescrField);
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(10, 0)));
    panel5.add(label5);
    panel5.add(Box.createRigidArea(new Dimension(10, 0)));
    panel5.add(actionField);
    panel5.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel6 = new JPanel();
    panel6.setLayout(new BoxLayout(panel6, BoxLayout.X_AXIS));
    panel6.add(Box.createRigidArea(new Dimension(20, 0)));
    panel6.add(startButton);
    panel6.add(Box.createRigidArea(new Dimension(20, 0)));
    panel6.add(closeButton);

    JPanel taskPanel = new JPanel();
    taskPanel.setLayout(new BoxLayout(taskPanel, BoxLayout.Y_AXIS));
    taskPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    taskPanel.add(panel1);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel11);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel2);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel3);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel4);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel5);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 30)));
    taskPanel.add(panel6);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    return taskPanel;
  }

  private JPanel fillOptionsPanel() {
    JButton loadButton    = new JButton("Load");
    JButton saveButton    = new JButton("Save");
    JButton addButton     = new JButton("Add");
    JButton removeButton  = new JButton("Remove");
    JButton defaultButton = new JButton("Default");

    loadButton.addActionListener(new LoadListener());

    saveButton.addActionListener(new SaveListener());

    addButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.addElement(":");
        rulesModel.fireTableDataChanged();
        int row = rulesTable.getRowCount()-1;
        rulesTable.setRowSelectionInterval(row, row);
      }
    });

    removeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.removeElementAt(rulesTable.getSelectedRow());
        rulesModel.fireTableDataChanged();
      }
    });

    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.removeAllElements();
        data.addElement("com/sun/java/swing:javax/swing");
        data.addElement("com/sun/java/swing/plaf/windows:com/sun/java/swing/plaf/windows");
        data.addElement("com/sun/java/swing/plaf/motif:com/sun/java/swing/plaf/motif");
        data.addElement("com/sun/java/swing/plaf/mac:com/sun/java/swing/plaf/mac");
        data.addElement("com/sun/java/accessibility:javax/accessibility");
        data.addElement("com/sun/java/accessibility/util:com/sun/java/accessibility/util");
        rulesModel.fireTableDataChanged();
      }
    });

    if(rulesTable.getRowCount() > 0)
      rulesTable.setRowSelectionInterval(0, 0);

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(new JScrollPane(rulesTable));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(defaultButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(loadButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(saveButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(addButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(removeButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));

    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.Y_AXIS));

    optionsPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    optionsPanel.add(panel2);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel5);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    return optionsPanel;
  }


class StartListener implements ActionListener {
  LongOperation launcher;

  public void actionPerformed(ActionEvent event) {

    if(startButton.getText().equals("Start")) {
      if(data.size() == 0) {
        JOptionPane.showMessageDialog(MigrateFrame.this,
              "You should define migrations rules.",
               "Warning", JOptionPane.WARNING_MESSAGE);
        return;
      }

      String from   = fromField.getText();
      MigrateTask task = new MigrateTask(toField.getText(), data);
      groupTask = new GroupTask(task, from, recurseCheck.isSelected());

      task.addNotificationListener(new NotificationListener() {
        public void notification(NotificationEvent evt) {
          actionDescrField.setText(evt.getDescription());
          actionField.setText(evt.getNotification());
        }
      });

      groupTask.addNotificationListener(new NotificationListener() {
        public void notification(NotificationEvent evt) {
          currFileField.setText(evt.getNotification());
        }
      });

      Runnable runnable = new Runnable() {
        public void run() {
          launcher = new MigrateLongOperation(groupTask);

          setCursor(new Cursor(Frame.WAIT_CURSOR));
          setClosable(false);
          startButton.setText("Cancel");
          browseButton.setEnabled(false);
          closeButton.setEnabled(false);
          recurseCheck.setEnabled(false);
          fromField.setEnabled(false);
          toField.setEnabled(false);
          tabbedPane.setEnabledAt(1, false);
        }
      };
      SwingUtilities.invokeLater(runnable);
    }
    else if(startButton.getText().equals("Cancel")) {
      if(launcher != null) {
        launcher.stop();
        launcher = null;
      }
    }
  }
}

class MigrateLongOperation extends LongOperation {
  private GroupTask groupTask;

  MigrateLongOperation(GroupTask groupTask) {
    super(100);
    this.groupTask = groupTask;

    super.startLoading();
  }

  protected void execute() throws Exception {
    groupTask.processing();
  }

  public synchronized void stopLoading() {
    super.stopLoading();

    int type = getResultType();
    if(type == LongOperation.OK_OPERATION) {
      JOptionPane.showMessageDialog(MigrateFrame.this,
            "Migration process was succesfully finished." +
             "\n" + this.groupTask.getProcessedFiles() + " files processed." +
             "\n" + this.groupTask.getMovements() + " movements occured." +
             "\n" + this.groupTask.getReplacements() + " replacements occured.",
             "Success", JOptionPane.WARNING_MESSAGE);
    }
    else if(type == LongOperation.ERROR_OPERATION) {
      JOptionPane.showMessageDialog(MigrateFrame.this,
                  "Error during file processing \n" + getResultMessage(),
                  "Failure", JOptionPane.ERROR_MESSAGE);
    }

    setCursor(new Cursor(Frame.DEFAULT_CURSOR));
    setClosable(true);
    startButton.setText("Start");
    browseButton.setEnabled(true);
    recurseCheck.setEnabled(true);
    fromField.setEnabled(true);
    toField.setEnabled(true);
    tabbedPane.setEnabledAt(1, true);
    closeButton.setEnabled(true);
    currFileField.setText("");
    actionDescrField.setText("");
    actionField.setText("");
  }

}

class BrowseListener implements ActionListener {
  public void actionPerformed(ActionEvent e) {
    int returnVal = chooser.showOpenDialog(MigrateFrame.this);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      setSelectedFile();
    }
  }
}

class LoadListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    JFileChooser chooser =
      DescriptionChooser.getInstance(".mig", "Migration  files *.mig");
    int returnVal = chooser.showOpenDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      File f = chooser.getSelectedFile();
      Vector migrateTable = null;
      try {
        BufferedReader br = new BufferedReader(new FileReader(f.getPath()));
        migrateTable = new Vector();

        String line = null;
        while((line = br.readLine()) != null) {
          line = line.trim().replace('.', '/');
          if(line.length() > 0)
            migrateTable.addElement(line);
        }
        br.close();
      }
      catch(Exception e) {
        JOptionPane.showMessageDialog(MigrateFrame.this,
             "Error reading file " + f.getPath() + ".",
             "Error", JOptionPane.WARNING_MESSAGE);
        return;
      }

      if(migrateTable != null) {
        data.removeAllElements();
        for(int i=0; i < migrateTable.size(); i++) {
          data.addElement(migrateTable.elementAt(i));
        }
        rulesModel.fireTableDataChanged();
      }
    }
  }
}


class SaveListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    JFileChooser chooser =
      DescriptionChooser.getInstance(".mig", "Migration  files *.mig");
    int returnVal = chooser.showSaveDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      File f = chooser.getSelectedFile();

      try {
        BufferedWriter bw = new BufferedWriter(new FileWriter(f.getPath()));

        for(int i=0; i < data.size(); i++) {
          String line = (String)data.elementAt(i);
          bw.write(line, 0, line.length());
          bw.newLine();
        }
        bw.close();
      }
      catch(Exception e) {
        JOptionPane.showMessageDialog(MigrateFrame.this,
             "Error writing file " + f.getPath() + ".",
             "Error", JOptionPane.WARNING_MESSAGE);
        return;
      }
    }
  }
}


}
