// MigrateOptionsDialog.java

package org.javalobby.cafebabe.plugin.migrateplug;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

// dialog for entering options for migration group task

public class MigrateOptionsDialog extends JDialog {

  private boolean isOk = true;

  private static final MigrationRulesModel rulesModel = new MigrationRulesModel();
  private static final JTable rulesTable = new JTable(rulesModel);
  private static final Vector data = rulesModel.getData();

  public MigrateOptionsDialog(JFrame parentFrame) {
    // Make sure we call the parent
    super(parentFrame, true);

    enableEvents(AWTEvent.FOCUS_EVENT_MASK);

    // Set the characteristics for this dialog instance
    this.setTitle("Select rules for migration process");
    this.setResizable(false);
    Rectangle r = parentFrame.getBounds();
    this.setBounds(r.x + r.width/2 - 600/2, r.y + r.height/2 - 200/2, 600, 200);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    JButton addButton     = new JButton("Add");
    JButton removeButton  = new JButton("Remove");
    JButton defaultButton = new JButton("Default");
    JButton okButton      = new JButton("Ok");
    JButton cancelButton  = new JButton("Cancel");

    addButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.addElement(":");
        rulesModel.fireTableDataChanged();
        int row = rulesTable.getRowCount()-1;
        rulesTable.setRowSelectionInterval(row, row);
      }
    });

    removeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.removeElementAt(rulesTable.getSelectedRow());
        rulesModel.fireTableDataChanged();
      }
    });

    defaultButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.removeAllElements();
        data.addElement("com/sun/java/swing:javax/swing");
        data.addElement("com/sun/java/swing/plaf/windows:com/sun/java/swing/plaf/windows");
        data.addElement("com/sun/java/swing/plaf/motif:com/sun/java/swing/plaf/motif");
        data.addElement("com/sun/java/swing/plaf/mac:com/sun/java/swing/plaf/mac");
        data.addElement("com/sun/java/accessibility:javax/accessibility");
        data.addElement("com/sun/java/accessibility/util:com/sun/java/accessibility/util");
        rulesModel.fireTableDataChanged();
      }
    });

    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        isOk = true;
        dispose();
      }
    });

    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        isOk = false;
        dispose();
      }
    });

    rulesTable.setRowSelectionInterval(0, 0);

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(new JScrollPane(rulesTable));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(addButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(removeButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(defaultButton);
    panel5.add(Box.createRigidArea(new Dimension(30, 0)));
    panel5.add(okButton);
    panel5.add(Box.createRigidArea(new Dimension(20, 0)));
    panel5.add(cancelButton);

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
    this.setContentPane(topPanel);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(panel2);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    topPanel.add(panel5);
    topPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    okButton.requestFocus();
  }

  public static Vector getRules() {
    return data;
  }

  public boolean getAnswer() { return isOk; }

}
