// ObfuscateFrame.java

package org.javalobby.cafebabe.plugin.obfplug;

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.border.*;

import org.javalobby.classfile.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.obfuscator.*;
import org.javalobby.task.event.*;
import org.javalobby.mdi.*;
import org.javalobby.cafebabe.*;
import org.javalobby.util.*;

// internal frame which help to process obfuscation task

public class ObfuscateFrame extends PluginFrame {

  private final JTextField fromField        = new JTextField();
  private final JTextField toField          = new JTextField();
  private final JTextField currFileField    = new JTextField();
  private final JTextField actionDescrField = new JTextField();
  private final JTextField actionField      = new JTextField();

  private final JTextField mainClassField  = new JTextField();
  private final JTextField classPathField  = new JTextField();

  private final JButton startButton   = new JButton("Start");
  private final JButton browseButton  = new JButton("Browse");
  private final JButton loadButton    = new JButton("Load");
  private final JButton saveButton    = new JButton("Save");
  private final JButton closeButton   = new JButton("Close");

  private NotificationListener infoListener = new NotificationListener() {
    public void notification(NotificationEvent event) {
      actionDescrField.setText(event.getDescription());
      actionField.setText(event.getNotification());
    }
  };

  private NotificationListener fileSelectionListener = new NotificationListener() {
    public void notification(NotificationEvent event) {
      currFileField.setText(event.getNotification());
      actionDescrField.setText(event.getDescription());
    }
  };

  private NotificationListener classSelectionListener = new NotificationListener() {
    public void notification(NotificationEvent event) {
    actionField.setText(event.getNotification() + " " + event.getDescription());
      actionDescrField.setText(event.getNotification());
    }
  };

  private final JCheckBox anonymousPackageModeCheck =
                 new JCheckBox("Anonymous package mode", true);
  private final JCheckBox obfuscatePackagesCheck =
                 new JCheckBox("Obfuscate packages", true);
  private final JCheckBox obfuscateClassesCheck =
                 new JCheckBox("Obfuscate classes", true);
  private final JCheckBox obfuscateFieldsCheck =
                 new JCheckBox("Obfuscate fields", true);
  private final JCheckBox obfuscateMethodsCheck =
                 new JCheckBox("Obfuscate methods", true);

  private JTabbedPane tabbedPane = new JTabbedPane();

  private File selectedFile;

  private final DeniedListModel deniedListModel = new DeniedListModel();
  private final JTable deniedList               = new JTable(deniedListModel);
  private final Vector data                     = deniedListModel.getData();

  private ProgramChooser chooser = ProgramChooser.getInstance();

  public ObfuscateFrame() {
    setTitle("Obfuscate Frame");
    setResizable(false);
    setMaximizable(false);

    setSelectedFile();

    JPanel taskPanel       = fillTaskPanel();
    JPanel optionsPanel    = fillOptionsPanel();

    tabbedPane = new JTabbedPane();
    this.setContentPane(tabbedPane);

    tabbedPane.addTab("Task", null, taskPanel, "Perform obfuscation task");
    tabbedPane.addTab("Configuration", null, optionsPanel,
                      "Allows to configure obfuscation process");

    tabbedPane.setSelectedIndex(0);

    startButton.requestFocus();
  }

  public void open() {
    // Set the characteristics for this dialog instance
    Rectangle r = getDesktopPane().getBounds();
    this.setBounds(r.x + r.width/2 - 650/2, r.y + r.height/2 - 490/2, 650, 360);

    super.open();
  }

  public void load(ProgramState programState) {
    setBounds(programState.getBoundsProperty("ObfuscateFrame", "20;20;500;400"));
  }

  public void save(ProgramState programState) {
    programState.putBoundsProperty("ObfuscateFrame", getBounds());
  }

  private void setSelectedFile() {
    File currentFile = chooser.getSelectedFile();

    if(currentFile == null) {
      try {
         currentFile = new File(new File(new File(".").getCanonicalPath()).getParent());
      }
      catch(Exception e) { e.printStackTrace(); }
    }
    else if(!currentFile.exists())
      currentFile = new File(new File(currentFile.getAbsolutePath()).getParent());

    String fileName = currentFile.toString();
    if(currentFile.isDirectory() ||
       fileName.endsWith("zip")  || fileName.endsWith("jar"))
      fromField.setText(fileName);
    else
      fromField.setText(new File(currentFile.getAbsolutePath()).getParent());
  }

  private JPanel fillTaskPanel() {
    JLabel label1 = new JLabel("From        :");
    JLabel label2 = new JLabel("To          :");
    JLabel label3 = new JLabel("File        :");
    JLabel label4 = new JLabel("Description :");
    JLabel label5 = new JLabel("Action      :");

    Font font  = new Font("Monospaced", Font.BOLD, 14);
    Font font2 = new Font("Monospaced", Font.BOLD, 12);

    label1.setFont(font);
    label2.setFont(font);
    label3.setFont(font);
    label4.setFont(font);
    label5.setFont(font);

    fromField.setFont(font);
    toField.setFont(font);
    classPathField.setFont(font2);
    currFileField.setFont(font2);
    actionDescrField.setFont(font2);
    actionField.setFont(font2);

    currFileField.setEnabled(false);
    actionDescrField.setEnabled(false);
    actionField.setEnabled(false);

    String parentDir = new File(fromField.getText()).getParent();
    if(parentDir == null) {
      toField.setText("Result");
    }
    else if(parentDir.endsWith(File.separator)) {
      toField.setText(parentDir + "Result");
    }
    else {
      toField.setText(parentDir + File.separatorChar + "Result");
    }

    classPathField.setText(System.getProperty("java.class.path"));

    actionField.setForeground(Color.green);

    startButton.addActionListener(new StartListener());
    browseButton.addActionListener(new BrowseListener());

    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(label1);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(fromField);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(browseButton);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(label2);
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));
    panel2.add(toField);
    panel2.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel3 = new JPanel();
    panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(label3);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));
    panel3.add(currFileField);
    panel3.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel4 = new JPanel();
    panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));
    panel4.add(label4);
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));
    panel4.add(actionDescrField);
    panel4.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel5 = new JPanel();
    panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
    panel5.add(Box.createRigidArea(new Dimension(10, 0)));
    panel5.add(label5);
    panel5.add(Box.createRigidArea(new Dimension(10, 0)));
    panel5.add(actionField);
    panel5.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel6 = new JPanel();
    panel6.setLayout(new BoxLayout(panel6, BoxLayout.X_AXIS));
    panel6.add(Box.createRigidArea(new Dimension(20, 0)));
    panel6.add(startButton);
    panel6.add(Box.createRigidArea(new Dimension(20, 0)));
    panel6.add(closeButton);

    JPanel taskPanel = new JPanel();
    taskPanel.setLayout(new BoxLayout(taskPanel, BoxLayout.Y_AXIS));
    taskPanel.add(Box.createRigidArea(new Dimension(0, 20)));
    taskPanel.add(panel1);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel2);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel3);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel4);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    taskPanel.add(panel5);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 65)));
    taskPanel.add(panel6);
    taskPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    return taskPanel;
  }

  private JPanel fillOptionsPanel() {
    JLabel label0 = new JLabel("Main Class :");
    JLabel label1 = new JLabel("CLASSPATH :");

    loadButton.addActionListener(new LoadListener());
    saveButton.addActionListener(new SaveListener());

    JPanel panel0 = new JPanel();
    panel0.setLayout(new BoxLayout(panel0, BoxLayout.X_AXIS));
    panel0.add(Box.createRigidArea(new Dimension(20, 0)));
    panel0.add(label0);
    panel0.add(Box.createRigidArea(new Dimension(10, 0)));
    panel0.add(mainClassField);
    panel0.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(label1);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));
    panel1.add(classPathField);
    panel1.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel2 = new JPanel();
    panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
    panel2.add(anonymousPackageModeCheck);
    panel2.add(Box.createRigidArea(new Dimension(0, 10)));
    panel2.add(obfuscatePackagesCheck);
    panel2.add(Box.createRigidArea(new Dimension(0, 10)));
    panel2.add(obfuscateClassesCheck);
    panel2.add(Box.createRigidArea(new Dimension(0, 10)));
    panel2.add(obfuscateFieldsCheck);
    panel2.add(Box.createRigidArea(new Dimension(0, 10)));
    panel2.add(obfuscateMethodsCheck);

    JButton addButton    = new JButton("Add");
    JButton removeButton = new JButton("Remove");
    addButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.addElement("");
        deniedListModel.fireTableDataChanged();
        int row = deniedList.getRowCount()-1;
        deniedList.setRowSelectionInterval(row, row);
      }
    });

    removeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        data.removeElementAt(deniedList.getSelectedRow());
        deniedListModel.fireTableDataChanged();
      }
    });

    JPanel panel3 = new JPanel();
    panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
    panel3.setSize(650, 300);
    panel3.setBorder(new TitledBorder(new EtchedBorder(), "Denied List"));
    panel3.add(Box.createRigidArea(new Dimension(0, 10)));
    panel3.add(new JScrollPane(deniedList));
    panel3.add(Box.createRigidArea(new Dimension(0, 10)));

    JPanel panel23 = new JPanel();
    panel23.setLayout(new BoxLayout(panel23, BoxLayout.X_AXIS));
    panel23.add(Box.createRigidArea(new Dimension(10, 0)));
    panel23.add(panel2);
//    panel23.add(Box.createGlue());
    panel23.add(Box.createRigidArea(new Dimension(10, 0)));
    panel23.add(panel3);
    panel23.add(Box.createRigidArea(new Dimension(10, 0)));

    JPanel panel4 = new JPanel();
    panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));
    panel4.add(loadButton);
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));
    panel4.add(saveButton);
    panel4.add(Box.createRigidArea(new Dimension(50, 0)));
    panel4.add(addButton);
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));
    panel4.add(removeButton);
    panel4.add(Box.createRigidArea(new Dimension(20, 0)));

    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.Y_AXIS));
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel0);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel1);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 25)));
    optionsPanel.add(panel23);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));
    optionsPanel.add(panel4);
    optionsPanel.add(Box.createRigidArea(new Dimension(0, 10)));

    return optionsPanel;
  }

  NamesTable names;
  LongOperation launcher1;

  static TaskDescription td = new TaskDescription();

class LoadListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    JFileChooser chooser =
      DescriptionChooser.getInstance(".obf", "Obfuscation files *.obf");
    int returnVal = chooser.showOpenDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      File f = chooser.getSelectedFile();

      try {
        td.load(f.getPath());
      }
      catch(Exception e) {
        JOptionPane.showMessageDialog(ObfuscateFrame.this,
             "Error reading file " + f.getPath() + ".",
             "Error", JOptionPane.WARNING_MESSAGE);
        return;
      }

      fromField.setText(td.getFrom());
      toField.setText(td.getTo());
      classPathField.setText(td.getClassPath());
      mainClassField.setText(td.getMainClass());

      boolean[] conditions = td.getConditions();

      anonymousPackageModeCheck.setSelected(td.getAnonymousMode());
      obfuscatePackagesCheck.setSelected(conditions[0]);
      obfuscateClassesCheck.setSelected(conditions[1]);
      obfuscateFieldsCheck.setSelected(conditions[2]);
      obfuscateMethodsCheck.setSelected(conditions[3]);

      Vector v =  td.getDeniedList();
      data.removeAllElements();
      for(int i=0; i < v.size(); i++)
        data.addElement(v.elementAt(i));

      deniedListModel.fireTableDataChanged();
    }
  }
}


class SaveListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    JFileChooser chooser =
      DescriptionChooser.getInstance(".obf", "Obfuscation files *.obf");
    int returnVal = chooser.showSaveDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      File f = chooser.getSelectedFile();

      td.setFrom(fromField.getText());
      td.setTo(toField.getText());
      td.setClassPath(classPathField.getText());
      td.setMainClass(mainClassField.getText());

      td.setAnonymousMode(anonymousPackageModeCheck.isSelected());

      td.setConditions(new boolean[] {
        obfuscatePackagesCheck.isSelected(),
        obfuscateClassesCheck.isSelected(),
        obfuscateFieldsCheck.isSelected(),
        obfuscateMethodsCheck.isSelected()
      });

      td.setDeniedList(data);

      String fileName = f.getPath();
      if(!fileName.endsWith(".obf")) fileName = fileName + ".obf";

      try {
        td.save(fileName);
      }
      catch(Exception e) {
        JOptionPane.showMessageDialog(ObfuscateFrame.this,
             "Error writing file " + fileName + ".",
             "Error", JOptionPane.WARNING_MESSAGE);
        return;
      }
    }
  }
}


class StartListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {

    String text = startButton.getText();
    if(text.equals("Start")) {
      boolean anonymousPackageMode = anonymousPackageModeCheck.isSelected();
      boolean obfuscatePackages    = obfuscatePackagesCheck.isSelected();
      boolean obfuscateClasses     = obfuscateClassesCheck.isSelected();
      boolean obfuscateFields      = obfuscateFieldsCheck.isSelected();
      boolean obfuscateMethods     = obfuscateMethodsCheck.isSelected();

      if(!anonymousPackageMode && !obfuscatePackages && !obfuscateClasses &&
         !obfuscateFields && !obfuscateMethods) {
        JOptionPane.showMessageDialog(ObfuscateFrame.this,
              "Nothing to do. Please select areas for obfuscation.",
               "Warning", JOptionPane.WARNING_MESSAGE);
        return;
      }

      String classPath = classPathField.getText();
      String mainClass = mainClassField.getText();

      if(classPath == null || classPath.length() == 0) {
        JOptionPane.showMessageDialog(ObfuscateFrame.this,
              "Please define CLASSPATH variable in Configuration panel",
               "Warning", JOptionPane.WARNING_MESSAGE);
        return;
      }

      if(mainClass == null || mainClass.length() == 0) {
        JOptionPane.showMessageDialog(ObfuscateFrame.this,
              "Please define main class in Configuration panel",
               "Warning", JOptionPane.WARNING_MESSAGE);
        return;
      }

      String fromDir = fromField.getText();

      names = new NamesTable(fromDir,
        classPath + File.pathSeparator + fromDir, mainClass, data);

      CollectInfoTask collectTask = new CollectInfoTask(names);
      final GroupTask groupTask = new GroupTask(collectTask, fromDir, true);
      actionDescrField.setText("Collect info from files...");
      groupTask.addNotificationListener(fileSelectionListener);
      names.addNotificationListener(classSelectionListener);

      Runnable runnable = new Runnable() {
        public void run() {
          launcher1 = new CollectInfoLongOperation(groupTask);

          setClosable(false);
          setCursor(new Cursor(Frame.WAIT_CURSOR));
          startButton.setText("Cancel");
          browseButton.setEnabled(false);
          fromField.setEnabled(false);
          toField.setEnabled(false);
          closeButton.setEnabled(false);
          tabbedPane.setEnabledAt(1, false);
        }
      };
      SwingUtilities.invokeLater(runnable);
    }
    else if(text.equals("Cancel")) {
      if(launcher1 != null) {
        launcher1.stop();
        launcher1 = null;
      }
    }
  }
}

class CollectInfoLongOperation extends LongOperation {

  private GroupTask groupTask;

  CollectInfoLongOperation(final GroupTask groupTask) {
    super(100);
    this.groupTask = groupTask;

    super.startLoading();
  }

  protected void execute() throws Exception {
    groupTask.processing();

    groupTask.removeNotificationListener(fileSelectionListener);

    boolean anonymousPackageMode = anonymousPackageModeCheck.isSelected();
    boolean obfuscatePackages    = obfuscatePackagesCheck.isSelected();
    boolean obfuscateClasses     = obfuscateClassesCheck.isSelected();
    boolean obfuscateFields      = obfuscateFieldsCheck.isSelected();
    boolean obfuscateMethods     = obfuscateMethodsCheck.isSelected();

    boolean[] conditions = new boolean[] {
      obfuscatePackages, obfuscateClasses, obfuscateFields, obfuscateMethods
    };

    actionDescrField.setText("Generating names...");
    currFileField.setText("");
    names.generateAllNames(anonymousPackageMode, conditions);

    ObfuscateTask obfuscateTask =  new ObfuscateTask(toField.getText(),
                  names, anonymousPackageMode, conditions);
    groupTask = new GroupTask(obfuscateTask, fromField.getText(), true);

    groupTask.addNotificationListener(fileSelectionListener);
    obfuscateTask.addNotificationListener(infoListener);
    names.addNotificationListener(classSelectionListener);

    groupTask.processing();

    groupTask.removeNotificationListener(fileSelectionListener);
    obfuscateTask.removeNotificationListener(classSelectionListener);
  }

  public synchronized void stopLoading() {
    super.stopLoading();

    int type = getResultType();
    if(type == LongOperation.OK_OPERATION) {
      JOptionPane.showMessageDialog(ObfuscateFrame.this,
            "Obfuscate process was succesfully finished." +
             "\n" + groupTask.getProcessedFiles() + " files processed." +
             "\n" + groupTask.getMovements() + " movements occured." +
             "\n" + groupTask.getReplacements() + " replacements occured.",
             "Success", JOptionPane.WARNING_MESSAGE);
    }
    else if(type == LongOperation.ERROR_OPERATION) {
      JOptionPane.showMessageDialog(ObfuscateFrame.this,
                  "Error during file processing \n" + getResultMessage(),
                  "Failure", JOptionPane.ERROR_MESSAGE);
    }

    setCursor(new Cursor(Frame.DEFAULT_CURSOR));
    setClosable(true);
    startButton.setText("Start");
    browseButton.setEnabled(true);
    fromField.setEnabled(true);
    toField.setEnabled(true);
    tabbedPane.setEnabledAt(1, true);
    closeButton.setEnabled(true);
    currFileField.setText("");
    actionDescrField.setText("");
    actionField.setText("");
  }

}

class BrowseListener implements ActionListener {
  public void actionPerformed(ActionEvent e) {
    int returnVal = chooser.showOpenDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      setSelectedFile();
    }
  }
}

}



