// SerFrame.java

package org.javalobby.cafebabe.plugin.serplug;

import java.io.*;
import java.beans.*;
import java.util.zip.*;
import java.awt.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.javalobby.mdi.*;
import org.javalobby.util.*;
import org.javalobby.cafebabe.*;

public class SerFrame extends ExtensionFrame implements SerPlugConstants {

  static {
    ProgramChooser chooser = ProgramChooser.getInstance();
    javax.swing.filechooser.FileFilter currFilter = chooser.getFileFilter();

    chooser.addChoosableFileFilter(new SerFileFilter());
    chooser.setFileFilter(currFilter);

    ProgramFileView fileView = (ProgramFileView)chooser.getFileView();

    fileView.putIcon("ser", IconProducer.getImageIcon(ICON_SER_FILE, SerFrame.class));
  }

  private SerTree serTree;
  private String serFileName;

  public SerFrame() {
    serTree = new SerTree();
    PlainActions actions = new PlainActions(this);

    setSize(700, 350);

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());
    this.getContentPane().add(topPanel);

    topPanel.add(actions.getToolBar(), BorderLayout.NORTH);

    JScrollPane scrollPane = new JScrollPane(serTree);
    topPanel.add(scrollPane, BorderLayout.CENTER);
  }

  public void load(ProgramState programState) {
    setBounds(programState.getBoundsProperty("SerFrame", "20;20;500;400"));
  }

  public void save(ProgramState programState) {
    programState.putBoundsProperty("SerFrame", getBounds());
  }

  public boolean canExit() {
    if(serTree.isAnyChange()) {
      int value = JOptionPane.showConfirmDialog(this,
                      "Do you want to save file " + serFileName + "?",
                       "Confirmation",
                       JOptionPane.YES_NO_CANCEL_OPTION);

      if(value == JOptionPane.CANCEL_OPTION) return false;

      if(value == JOptionPane.YES_OPTION)
        if(!saveAs(serFileName))
          return false;
    }

    return true;
  }

  public void setTutorialMode(boolean mode) {
    serTree.setTutorialMode(mode);
  }

  public void invoke(final Runnable runnable) {
    Thread thread = new Thread() {
      public void run() {
        Component glassPane = ((JFrame)SerFrame.this.getTopLevelAncestor()).getGlassPane();

//        glassPane.setVisible(true);
        setCursor(new Cursor(Frame.WAIT_CURSOR));
        runnable.run();
        setCursor(new Cursor(Frame.DEFAULT_CURSOR));
//        glassPane.setVisible(false);
      }
    };

    thread.setPriority(Thread.MIN_PRIORITY+2);
    thread.start();
  }

  public void collapse() {
    invoke(new Runnable() {
      public void run() {
        serTree.collapse();
      }
    });
  }

  public void expandFirstLevel() {
    invoke(new Runnable() {
      public void run() {
        serTree.expandFirstLevel();
      }
    });
  }

  public void expandAllLevels() {
    invoke(new Runnable() {
      public void run() {
        serTree.expandAllLevels();
      }
    });
  }

  public void search() {
    Component current = this;
    Component parent  = null;
    while(true) {
      parent = current.getParent();
      if(parent == null) break;
      if(parent instanceof JFrame) break;
      current = parent;
    }

    if(parent != null)
      serTree.search((JFrame)parent);
  }

  public void open(File f) {
    fireUpdatePerformed(this, getTitle(), false);
    setTitle(f.getAbsolutePath());
    fireUpdatePerformed(this, getTitle(), true);

    serTree.setFile(f);
  }

  public void close() {
    serTree.clear();
    serFileName = null;
  }

  public boolean saveAs(String serName) {
    OutputStream os = null;

    try {
      File f = new File(serName);

      serTree.saveAs(f);

      fireUpdatePerformed(this, getTitle(), false);
      setTitle(serName);
      fireUpdatePerformed(this, getTitle(), true);

      serFileName = serName;

      return true;
    }
    catch(Exception e) {
      e.printStackTrace();
    }

    return false;
  }

}
