// StripOptionsDialog.java

package org.javalobby.cafebabe.plugin.stripplug;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

// dialog for entering options for strip group task

public class StripOptionsDialog extends JDialog {

  private boolean isOk = true;

  private static JCheckBox unknownAttributesCheck=
                 new JCheckBox("Unknown class attributes", true);
  private static JCheckBox sourceFileCheck =
                 new JCheckBox("SourceFile attribute", true);
  private static JCheckBox optionalMethodInfoCheck =
                 new JCheckBox("Optional info inside method body", true);

  public StripOptionsDialog(JFrame parentFrame) {
    // Make sure we call the parent
    super(parentFrame, true);

    enableEvents(AWTEvent.FOCUS_EVENT_MASK);

    // Set the characteristics for this dialog instance
    this.setTitle("Select blocks to remove");
    this.setResizable(false);
    Rectangle r = parentFrame.getBounds();
    this.setBounds(r.x + r.width/2 - 300/2, r.y + r.height/2 - 200/2, 300, 200);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    // Options panel

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));

    panel1.add(optionalMethodInfoCheck);
    panel1.add(sourceFileCheck);
    panel1.add(unknownAttributesCheck);

    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.X_AXIS));
    optionsPanel.add(Box.createRigidArea(new Dimension(20, 0)));
    optionsPanel.add(panel1);
    optionsPanel.add(Box.createRigidArea(new Dimension(20, 0)));

    // Control panel

    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));

    JButton okButton     = new JButton("Ok");
    JButton cancelButton = new JButton("Cancel");

    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        isOk = true;
        dispose();
      }
    });

    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        isOk = false;
        dispose();
      }
    });

    controlPanel.add(Box.createHorizontalGlue());
    controlPanel.add(okButton);
    controlPanel.add(Box.createRigidArea(new Dimension(10, 0)));
    controlPanel.add(cancelButton);
    controlPanel.add(Box.createHorizontalGlue());

    Container contentPane = this.getContentPane();

    contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
    contentPane.add(Box.createRigidArea(new Dimension(0, 10)));
    contentPane.add(optionsPanel);
    contentPane.add(Box.createRigidArea(new Dimension(0, 20)));
    contentPane.add(controlPanel);
    contentPane.add(Box.createRigidArea(new Dimension(0, 10)));

    pack();

    okButton.requestFocus();
  }

  public boolean getAnswer() { return isOk; }

  public static boolean unknownAttributes() {
    return unknownAttributesCheck.isSelected();
  }

  public static boolean sourceFile() {
    return sourceFileCheck.isSelected();
  }

  public static boolean optionalMethodInfo() {
    return optionalMethodInfoCheck.isSelected();
  }
}
