// Entry.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents any entry inside class file:
 * const entry, interface entry, attribute entry etc.
 * Abstract form for presentation of some collection of entries
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public interface Entry {

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException;

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException;

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType();

  /**
   * Length of this entry in bytes
   *
   * @return  length of entry.
   */
  public long length();

}
