// FieldEntry.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

/**
 * Class that represents a "field" entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class FieldEntry extends MemberEntry {

  /**
   * The default constructor that creates field entry.
   */
  public FieldEntry() {}

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Field";
  }

  /**
   * Resolve this entry.
   *
   * @param   constPool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    Signature signature = new Signature(getSignature(constPool));

    return new String[] {
      accessFlags.toString(), signature.getTypeName(), getName(constPool)
    };
  }

  /**
   * Compares two Objects for equality. Two field entries
   * will be equals if they both have the same value and name.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof FieldEntry) {
      FieldEntry c = (FieldEntry)o;

      return ((nameIndex == c.nameIndex) && (valueIndex == c.valueIndex));
    }

    return false;
  }

}
