// MethodConst.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents a "method" const entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class MethodConst extends MemberConst {

  /**
   * The default constructor that creates method constant.
   */
  public MethodConst() {}

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Method Const";
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return CONSTANT_METHOD;
  }

  /**
   * Compares two Objects for equality. Two method consts
   * will be equals if they both have the same value and name.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof MethodConst) {
      MethodConst c = (MethodConst)o;
      return ((nameIndex == c.nameIndex) && (valueIndex == c.valueIndex));
    }

    return false;
  }

}
