// MethodEntry.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

/**
 * Class that represents a "method" entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class MethodEntry extends MemberEntry {

  /**
   * The default constructor that creates method entry.
   */
  public MethodEntry() {}

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Method";
  }

  /**
   * Get an attribute with the specified name in base of information
   * from constant pool.
   * @param name  the name of an attribute
   * @param     constPool constant pool that contains information for
   *            resolution process
   * @return    founded attribute or null if specified attribute doesn't exist.
   */
  public AttributeEntry getAttribute(String name, ConstPool constPool) {
    AttributeEntry attribute = null;

    for(short i=0; i < attributes.size(); i++) {
      AttributeEntry a = (AttributeEntry)attributes.elementAt(i);
      String attrName  = a.getName(constPool);

      if(attrName.equals(name)) {
        attribute = a;
        break;
      }
    }

    return attribute;
  }

  /**
   * Resolve this entry.
   *
   * @param   constPool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    MethodSignature signature = new MethodSignature(getSignature(constPool));

    Signature returnType   = signature.getReturnType();
    Signature[] parameters = signature.getParameters();

    StringBuffer sb = new StringBuffer();
    if(parameters.length > 0)
      sb.append(parameters[0].getTypeName());

    for(int i=1; i < parameters.length; i++) {
      sb.append(", " + parameters[i].getTypeName());
    }

    return new String[] {
       accessFlags.toString(),
       returnType.getTypeName() + " " + getName(constPool) + "(" +
       sb.toString() + ")"
    };
  }

  /**
   * Compares two Objects for equality. Two method entries
   * will be equals if they both have the same value and name.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof MethodEntry) {
      MethodEntry c = (MethodEntry)o;

      return ((nameIndex == c.nameIndex) && (valueIndex == c.valueIndex));
    }

    return false;
  }

}
