// MethodSignature.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

/**
 * Class that represents a signature of a method.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class MethodSignature {

  /* Signature that represents type of result that is returned
   * by a method */
  private Signature returnType;

  /* Array of signature that represents types of input parameters
   * for a method */
  private Signature[] parameters;

  /* A string representation of a signature for a method */
  private String key;

  /**
   * Constructs a method signature with the specified key.
   *
   * @param   key   the string representation of a method signature
   */
  public MethodSignature(String key) {
    this.key = key;

    Vector params = getParamList(key);

    parameters = new Signature[params.size()-1];

    for(int i=0; i < params.size()-1; i++) {
      parameters[i] = new Signature((String)params.elementAt(i));
    }

    returnType = new Signature((String)params.lastElement());
  }

  /**
   * Get the type of return value for this signature.
   *
   * @return  the type of return value for this signature
   */
  public Signature getReturnType() {
    return returnType;
  }

  /**
   * Get the array of types for input parameters.
   *
   * @return  the array of types for input parameters
   */
  public Signature[] getParameters() {
    return parameters;
  }

  /** Convert the method signature to a vector of strings.
   *
   * @return  the vector of strings that represent this method signature
   */
  public static Vector getParamList(String paramList) {
    Vector params = new Vector();

    int pos = 0;
    if(paramList.charAt(pos) == '(') // delete '('
      pos++;

    while(pos < paramList.length()-1) {
      String param = null;
      if(paramList.charAt(pos) == ')') {
        pos++; // delete ')'
      }

      param = getParam(paramList, pos);
      params.addElement(param);

      pos += param.length();
    }

    return params;
  }

  private static String getParam(String paramList, int pos) {
    int pos1 = pos;

    while(paramList.charAt(pos1) == '[') pos1++;

    char ch = paramList.charAt(pos1);
    if(ch == 'B' || ch == 'C' || ch == 'D' || ch == 'F' ||
       ch == 'I' || ch == 'J' || ch == 'S' || ch == 'Z') {
      pos1++;
    }
    else if(ch == 'L') {
      while(paramList.charAt(pos1) != ';') pos1++;
      pos1++;
    }
    else if(ch == 'V') {
      pos1++;
    }

    return paramList.substring(pos, pos1);
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("(");

    for(int i=0; i < parameters.length; i++) {
      sb.append(parameters[i].getKey());
    }

    sb.append(")");

    sb.append(returnType.getKey());

    return sb.toString();
  }

}
