// AttributeInterpretator.java


package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * Class that represents basic interpretator for attributes inside class file.
 * It take as a parameter plain attribute and help to interpret its contents
 * corresponding to nae of attribute (e.g. type of attribute).
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class AttributeInterpretator extends AttributeEntry {

  /** An attribute for which interpretation should be performed */
  protected AttributeEntry attribute;

  /**
   * Creates corresponding interpretator for specified attribute
   *
   * @param attribute  an attribute for which interpretation should
   * be performed.
   */
  public AttributeInterpretator(AttributeEntry attribute) {
    this.attribute = attribute;

    nameIndex = attribute.getNameIndex();
    buffer    = attribute.getBuffer();

    parse();
  }

  /**
   * This method performs parsing of buffer's contents and then building
   * corresponding structures that describe this attribute.
   */
  public void parse() {
    try {
      ByteArrayInputStream bais = new ByteArrayInputStream(buffer);
      DataInputStream dis = new DataInputStream(bais);
      read(dis);
      dis.close();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Set the byte array for this attribute. In this method there is a
   * synchronization of buffers' content both for original attribute and its
   * interpretator.
   *
   * @param buffer  the byte array
   */
  public void setBuffer(byte[] buffer) {
    this.buffer = buffer;

    attribute.setBuffer(buffer);
  }

}
