// ConstantValueAttribute.java

package org.javalobby.classfile.attribute;

import java.io.*;

import org.javalobby.classfile.*;

// this class explains how to interpret attribute "constant value" of field

/**
 * An interpretator for "Constant Value" attribute.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class ConstantValueAttribute extends AttributeInterpretator {

  /* an index that points to a string const with a content of constant value */
  private short index;

  /**
   * Creates interpretator for "Constant Value" attribute
   *
   * @param attribute  an original attribute
   */
  public ConstantValueAttribute(AttributeEntry attribute) {
    super(attribute);
  }

  /**
   * Read this attribute from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    index = in.readShort();
  }

  /**
   * Write this attributre to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(index);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Constant Value Attribute";
  }

  /**
   * Get an index that points to a content of constant value in constant pool
   *
   * @return  an index
   */
  public short getIndex() {
    return index;
  }

}

