// ExceptionsAttribute.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * An interpretator for "Exceptions" attribute.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class ExceptionsAttribute extends AttributeInterpretator {

  /* an array of exceptions entries */
  private ExceptionEntry[] exceptions;

  /**
   * Creates interpretator for "Exceptions" attribute
   *
   * @param attribute  an original attribute
   */
  public ExceptionsAttribute(AttributeEntry attribute) {
    super(attribute);
  }

  /**
   * Read this attribute from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    short size = in.readShort();

    exceptions = new ExceptionEntry[size];
    for(int i=0, j=1; i < size; i++) {
      (exceptions[i] = new ExceptionEntry()).read(in);
    }
  }

  /**
   * Write this attributre to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short size = (short)exceptions.length;

    out.writeShort(size);

    for(int i=0; i < size; i++) {
      exceptions[i].write(out);
    }
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Exceptions Attribute";
  }

  /**
   * Get an array of exception entries
   *
   * @return  an array of exception entries
   */
  public ExceptionEntry[] getExceptions() {
    return exceptions;
  }

}
