// InnerClassesAttribute.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * An interpretator for "Inner Classes" attribute.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class InnerClassesAttribute extends AttributeInterpretator {

  /* an array of inner class entries */
  private InnerClassEntry[] innerClasses;

  /**
   * Creates interpretator for "Inner Classes" attribute
   *
   * @param attribute  an original attribute
   */
  public InnerClassesAttribute(AttributeEntry attribute) {
    super(attribute);
  }

  /**
   * Read this attribute from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    short size = in.readShort();

    innerClasses = new InnerClassEntry[size];
    for(int i=0, j=1; i < size; i++) {
      (innerClasses[i] = new InnerClassEntry()).read(in);
    }
  }

  /**
   * Write this attributre to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short size = (short)innerClasses.length;

    out.writeShort(size);

    for(int i=0; i < size; i++) {
      innerClasses[i].write(out);
    }
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Inner Classes Attribute";
  }

  /**
   * Get an array of inner class entries
   *
   * @return  an array of inner class entries
   */
  public InnerClassEntry[] getInnerClasses() {
    return innerClasses;
  }

}

