// LineNumberEntry.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

// class that represent line number entry

/**
 * Class that represents a line number entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class LineNumberEntry implements Entry, Resolvable {

  /* An index that points to start PC index */
  private short startPc;

  /* An index that points to line number index */
  private short lineNumber;


  /**
   * The default constructor that creates line number entry.
   */
  public LineNumberEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    startPc    = in.readShort();
    lineNumber = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(startPc);
    out.writeShort(lineNumber);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Line Number";
  }

  /**
   * Length of this entry in bytes (start PC + line number).
   *
   * @return  length of entry
   */
  public long length() {
    return 2 + 2;
  }

  /**
   * Get start PC index
   *
   * @return  start PC index
   */
  public short getStartPc() {
    return startPc;
  }

  /**
   * Get line number index
   *
   * @return  line number index
   */
  public short getLineNumber() {
    return lineNumber;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    return new String[] { "(" + startPc + ", " + lineNumber + ")" };
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "(" + startPc + ", " + lineNumber + ")";
  }

}

