// LineNumberTableAttribute.java


package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * An interpretator for "Line Number Table" attribute.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class LineNumberTableAttribute extends AttributeInterpretator {

  /* an array of line number entries */
  private LineNumberEntry[] lineNumbers;

  /**
   * Creates interpretator for "Line Number Table" attribute
   *
   * @param attribute  an original attribute
   */
  public LineNumberTableAttribute(AttributeEntry attribute) {
    super(attribute);
  }

  /**
   * Read this attribute from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    short size = in.readShort();

    lineNumbers = new LineNumberEntry[size];
    for(int i=0, j=1; i < size; i++) {
      (lineNumbers[i] = new LineNumberEntry()).read(in);
    }
  }

  /**
   * Write this attributre to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short size = (short)lineNumbers.length;

    out.writeShort(size);

    for(int i=0; i < size; i++) {
      lineNumbers[i].write(out);
    }
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Line Number Table Attribute";
  }

  /**
   * Get an array of line number entries
   *
   * @return  an array of line number entries
   */
  public LineNumberEntry[] getLineNumbers() {
    return lineNumbers;
  }

}
