// LocalVariableTableAttribute.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * An interpretator for "Local Variable Table" attribute.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class LocalVariableTableAttribute extends AttributeInterpretator {

  /* an array of local variable entries */
  private LocalVariableEntry[] localVariables;


  /**
   * Creates interpretator for "Local Variable Table" attribute
   *
   * @param attribute  an original attribute
   */
  public LocalVariableTableAttribute(AttributeEntry attribute) {
    super(attribute);
  }

  /**
   * Read this attribute from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    short size = in.readShort();

    localVariables = new LocalVariableEntry[size];
    for(int i=0, j=1; i < size; i++) {
      (localVariables[i] = new LocalVariableEntry()).read(in);
    }
  }

  /**
   * Write this attributre to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short size = (short)localVariables.length;

    out.writeShort(size);

    for(int i=0; i < size; i++) {
      localVariables[i].write(out);
    }
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Local Variable Table Attribute";
  }

  /**
   * Get an array of local variable entries
   *
   * @return  an array of local variable entries
   */
  public LocalVariableEntry[] getLocalVariables() {
    return localVariables;
  }

}

