// RankTable.java

package org.javalobby.classfile.instruction;

import java.util.*;

/**
 * This class represents a table of ranks. In such manner we can classify
 * all instructions dividing them to predefined set of ranks.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class RankTable {

  /** an array of all ranks in the table */
  protected Rank[] ranks;

  /** an array with descriptions for all ranks */
  protected String[] descriptions;

  /**
   * A default constructor that empty rank table
   */
  public RankTable() {}

  /**
   * Get descriptions for all ranks.
   *
   * @return  array of descriptions for all ranks
   */
  public String[] descriptions() {
    return descriptions;
  }

  /**
   * Fing the rank for a given description.
   *
   * @return  the founded rank
   */
  public Rank getRank(String description) {
    Rank rank = null;

    for(int i=0; i < ranks.length; i++) {
      if(ranks[i].key.equals(description)) {
        rank = ranks[i];
        break;
      }
    }

    return rank;
  }

  /**
   * Fing the rank for a given opcode.
   *
   * @return  the founded rank
   */
  public Rank getRank(byte opcode) {
    Rank rank = null;

    for(int i=0; i < ranks.length; i++) {
      int[] opcodes = ranks[i].opcodes;
      for(int j=0; j < opcodes.length; j++) {
        if(opcodes[j] == (opcode & 0xff)) {
          rank = ranks[i];
          break;
        }
      }
    }

    return rank;
  }

}
