// MdiDesktopPane.java

package org.javalobby.mdi;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class MdiDesktopPane extends JDesktopPane
                            implements MdiListener, ActionListener {

  final String CASCADE_STRING   = "Cascade";
  final String TILE_STRING      = "Tile";
  final char CASCADE_CHAR       = 'C';
  final char TILE_CHAR          = 'T';
  final String CASCADE_D_STRING = "Cascade opened windows";
  final String TILE_D_STRING    = "Tile opened windows";

  private int numberOfControlItems = 2;

  private MdiMenuBar menuBar;
  private JMenu windowMenu;

  public MdiDesktopPane(MdiMenuBar menuBar) {
    super();

    this.menuBar = menuBar;

    this.setOpaque(true);

    windowMenu = menuBar.getWindowMenu();

    JMenuItem cascadeItem = new JMenuItem(CASCADE_STRING);
    cascadeItem.setToolTipText(CASCADE_D_STRING);
    cascadeItem.setMnemonic(CASCADE_CHAR);

    cascadeItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        cascadeFrames();
      }
    });

    JMenuItem tileItem = new JMenuItem(TILE_STRING);
    tileItem.setToolTipText(TILE_D_STRING);
    tileItem.setMnemonic(TILE_CHAR);

    tileItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        tileFrames();
      }
    });

    windowMenu.add(cascadeItem);
    windowMenu.add(tileItem);
  }

  public void updatePerformed(MdiEvent event) {
    String itemName = event.getItemName();

    if(event.isVisible()) {
      if(windowMenu.getMenuComponentCount() == numberOfControlItems)
        windowMenu.addSeparator();
      JMenuItem menuItem = menuBar.addWindowMenuItem(itemName);
      menuItem.addActionListener(this);
    }
    else {
      menuBar.removeWindowMenuItem(itemName);
      if(windowMenu.getMenuComponentCount() == numberOfControlItems+1)
        windowMenu.remove(numberOfControlItems);
    }
  }

  public void actionPerformed(ActionEvent event) {
    String itemText = event.getActionCommand();

    JInternalFrame[] framesList = getAllFrames();
    for(int i=0; i < framesList.length; i++) {
      JInternalFrame frame = (JInternalFrame)framesList[i];
      String title = frame.getTitle();
      if(title.equals(itemText)) {
        if(frame.isIcon())
          getDesktopManager().deiconifyFrame(frame);

        frame.moveToFront();
        try {
          frame.setSelected(true);
        }
        catch (java.beans.PropertyVetoException pve) {
          System.out.println("Could not select " + frame.getTitle());
        }
      }
    }
  }

/*  public void moveToFront(Component c) {
    if(c instanceof JInternalFrame) {
      JInternalFrame f = (JInternalFrame)c;
      ActionEvent event = new ActionEvent(this,
                              ActionEvent.ACTION_PERFORMED, f.getTitle());
      actionPerformed(event);
    }

    super.moveToFront(c);
  }
*/

  public void cascadeFrames() {
/*    int m = 0, n = 0;

    JInternalFrame[] framesList = getAllFrames();
    for(int i=0; i < framesList.length; i++) {
      JInternalFrame frame = (JInternalFrame)framesList[i];

      frame.setBounds(20*(m%10) + n*80, 20*(m%10), 650, 370);
      try {
        frame.setSelected(true);
      }
      catch (java.beans.PropertyVetoException pve) {
        System.out.println("Could not select " + frame.getTitle());
      }

      m++;
      if(m % 10 == 0) {
        if(n < 3)
          n++;
        else
          n = 0;
      }
    }
*/

    JInternalFrame[] list = this.getAllFrames();

    int h = this.getHeight() + 20;
    int w = this.getWidth() + 20;
    int n = list.length;
    for(int i=0; i < n; i++) {
      JInternalFrame frame = list[i];
      int x = i*20;
      int y = i*20;
      int width = (w-2*n*10);
      int height = (h-2*n*10);
      getDesktopManager().setBoundsForFrame(frame, x, y, width, height);
      frame.moveToFront();
      try {
        frame.setSelected(true);
      }
      catch(Exception e) {}
    }
  }

  public void tileFrames() {
    JInternalFrame[] list = this.getAllFrames();
    int h = this.getHeight();
    int w = this.getWidth();
    int n = list.length;

    if(n == 0 || h == 0)
      return;

    double d =  Math.sqrt(((h*w)/n)*(w/h));
    int width = (int)d ;
    int perLineW = (int) (w/width);
    int perLineH = (int) (n/perLineW);
    width = (int) (w/perLineW);
    int height = (int) (h/perLineH);
    int lastline =  n - (perLineW * (perLineH - 1));
    int x = 0, y = 0, curLine = 1;
    for(int i=0; i < n; i++) {
      JInternalFrame frame = list[i];
      if ((x + width) > w) {
        y = y + height;
        x = 0;
        curLine ++;
      }
      if (curLine == perLineH) {
        width =  (int)(w/lastline);
        height = h - y;
        getDesktopManager().setBoundsForFrame(frame, x, y, width, height);
        x = x + width;
      }
      else {
        getDesktopManager().setBoundsForFrame(frame, x, y, width, height);
        x = x + width;
      }
      frame.moveToFront();
      try {
        frame.setSelected(true);
      }
      catch(Exception e) {}
    }
  }

}
