// MdiFrame.java

package org.javalobby.mdi;

import java.util.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;

public class MdiFrame extends JInternalFrame {

  public MdiFrame(String title, boolean resizable,
                boolean closable, boolean maximizable, boolean iconifiable) {
    super(title, resizable, closable, maximizable, iconifiable);

    this.setOpaque(true);
    this.setVisible(true);

    setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);

    addVetoableChangeListener(new VetoableChangeListener() {
      public void vetoableChange(PropertyChangeEvent pce)
                                 throws PropertyVetoException {

//        System.out.println(pce.getPropertyName());
//        System.out.println(pce.getOldValue());
//        System.out.println(pce.getNewValue());

        if(pce.getPropertyName().equals(IS_CLOSED_PROPERTY)) {
          boolean changed = ((Boolean) pce.getNewValue()).booleanValue();
          if(changed) {
            if(canExit()) {

              /*JDesktopPane desktopPane = getDesktopPane();
              if(desktopPane != null) {
                desktopPane.remove(desktopPane.getIndexOf(MdiFrame.this));
                desktopPane.repaint();
                fireUpdatePerformed(this, getTitle(), false);
              }
              close();

              */

              fireUpdatePerformed(this, getTitle(), false);
            }
            else
              throw new PropertyVetoException("Cancelled", pce);
          }
        }
      }
    });
  }

  public void open() {
    JDesktopPane desktopPane = getDesktopPane();
    if(desktopPane != null) {
      addMdiListener((MdiListener)desktopPane);
      fireUpdatePerformed(this, getTitle(), true);
      desktopPane.moveToFront(this);
      try {
        setSelected(true);
      }
      catch (java.beans.PropertyVetoException pve) {
        System.out.println("Could not select " + getTitle());
      }
    }
  }

  public void close() {
    try {
      fireVetoableChange(JInternalFrame.IS_CLOSED_PROPERTY,
                         (Object)new Boolean(false), (Object)new Boolean(true));

      setClosed(true);
    }
    catch(PropertyVetoException e) {
      e.printStackTrace();
    }

  }

  public boolean canExit() {
    return true;
  }

  // support of listeners for event MdiEvent
  private Vector listeners = new Vector();

  public synchronized void addMdiListener(MdiListener l) {
    listeners.addElement(l);
  }
  public synchronized void removeMdiListener(MdiListener l) {
    listeners.removeElement(l);
  }

  // fire MdiEvent to registered listeners
  protected void fireUpdatePerformed(Object source, String itemName,
                                     boolean visible) {

    // 1. Creating event
    MdiEvent event = new MdiEvent(source, itemName, visible);

    // 2. Cloning of listeners list to eliminate race condition
    Vector ls = null;
    synchronized (this) {
      ls = (Vector)listeners.clone();
    }

    // 3. Notification of all registered listeners
    for(int i=0; i < ls.size(); i++) {
      MdiListener l = (MdiListener)ls.elementAt(i);
      l.updatePerformed(event);
    }
  }

}
