// MdiTest.java

package org.javalobby.mdi;

import java.beans.*;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

public final class MdiTest extends JFrame {

  private String GOODBYE_MESSAGE    = "Good bye!";

  private final Integer PROGRAM_LAYER = JLayeredPane.PALETTE_LAYER;
  private int windowNumber = 0;

  private final MdiMenuBar menuBar        = new MdiMenuBar();
  private final JToolBar toolBar          = new JToolBar();
  private MdiDesktopPane desktopPane      = new MdiDesktopPane(menuBar);

  public MdiTest() {
    super("MdiTest");

    this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

    desktopPane.putClientProperty("JDesktopPane.dragMode", "outline");

    init();
  }

  private void init() {
    JMenu windowMenu = menuBar.getWindowMenu();
    windowMenu.setText("Window");
    windowMenu.setMnemonic('W');

    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic('F');

    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic('H');

    menuBar.add(fileMenu);
    menuBar.add(windowMenu);
    menuBar.add(helpMenu);

    fileMenu.add(new AbstractAction("Create Window") {
      public void actionPerformed(ActionEvent e) {
        new CustomMdiFrame("Window " + (windowNumber++));
      }
    });

    fileMenu.add(new AbstractAction("Exit") {
      public void actionPerformed(ActionEvent e) {
        exit();
      }
    });

    helpMenu.add(new AbstractAction("About") {
      public void actionPerformed(ActionEvent e) {
        System.out.println("About");
      }
    });

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());

    this.setJMenuBar(menuBar);
    topPanel.add(toolBar, BorderLayout.NORTH);
    topPanel.add(desktopPane, BorderLayout.CENTER);

    this.setContentPane(topPanel);

    setBounds(10, 10, 750, 500);
    setVisible(true);
  }

  public void exit() {
    WindowEvent evt = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
    processWindowEvent(evt);
  }

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      super.processWindowEvent(e);
      System.out.println(GOODBYE_MESSAGE);
      System.exit(0);
    }
    else
      super.processWindowEvent(e);
  }

  InternalFrameListener internalFrameListener = new InternalFrameAdapter() {

    public void internalFrameActivated(InternalFrameEvent e) {
      System.out.println("Activated");
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
      System.out.println("Deactivated");
    }

    public void internalFrameClosing(InternalFrameEvent e) {
      System.out.println("Closing");
    }

    public void internalFrameOpened(InternalFrameEvent e) {
      System.out.println("Opened");
    }

    public void internalFrameClosed(InternalFrameEvent e) {
      System.out.println("Closed");
    }

    public void internalFrameIconified(InternalFrameEvent e) {
      System.out.println("Iconified");
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {
      System.out.println("Deiconified");
    }
  };

class CustomMdiFrame extends MdiFrame {

  CustomMdiFrame(String title) {
    super(title, true, true, true, true);
    this.setBounds(10, 10, 500, 400);
    this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    JPanel topPanel = new JPanel();
    topPanel.setLayout(new BorderLayout());
    this.getContentPane().add(topPanel);

    JButton okButton  = new JButton("Ok");

    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });

    JPanel panel1 = new JPanel();
    panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));

    panel1.add(Box.createRigidArea(new Dimension(0, 70)));
    panel1.add(okButton);
    panel1.add(Box.createRigidArea(new Dimension(0, 70)));

    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
    topPanel.add(Box.createRigidArea(new Dimension(250, 0)));
    topPanel.add(panel1);
    topPanel.add(Box.createRigidArea(new Dimension(250, 0)));

    addInternalFrameListener(internalFrameListener);

    desktopPane.add(this, PROGRAM_LAYER);
    open();

    okButton.requestFocus();
  }

}

  public static void main(String args[]) throws Exception {
    new MdiTest();
  }

}
