// SMTPClient.java

package org.javalobby.net;

import java.io.*;
import java.net.*;

import org.javalobby.net.*;

// simple implementation of SMTP mail client

public class SMTPClient implements Client {
  static final String EOL = "\r\n"; // network end of line

  static final short DEFAULT_PORT = 25;

  protected Socket socket;
  protected DataInputStream in;
  protected PrintStream out;

//  private String request;
//  private String responce;

  private String helohost;
  private String sender;
  private String receiver;
  private String subject;
  private String body;

  private int port = DEFAULT_PORT;

  private String host;

  public SMTPClient(String host) {
    this.host = host;
  }

  public void setRequest(Object request) throws IOException {
    out.println((String)request);
  }

  public Object getResponse() throws IOException {
    return in.readLine();
  }

  public boolean existsResponse() {
    return true;
  }

  public void connect() throws IOException {
    socket = new Socket(host, port);
    in  = new DataInputStream(socket.getInputStream());
    out = new PrintStream(socket.getOutputStream());
  }

  /**
   * Disconnection from the server.
   *
   * @exception  IOException  if an I/O error occurs.
   */
  protected void disconnect() throws IOException {
    try {
      if(in != null)
        in.close();
      if(out != null)
        out.close();
      if(socket != null)
        socket.close();
    }
    finally {
      in = null; out = null; socket = null;
    }
  }

  public void setHeloHost(String helohost) {
    this.helohost = helohost;
  }

  public void setSender(String sender) {
    this.sender = sender;
  }

  public void setReceiver(String receiver) {
    this.receiver = receiver;
  }

  public void setSubject(String subject) {
    this.subject = subject;
  }

  public void setBody(String body) {
    this.body = body;
  }

  public void interact() throws IOException {
    connect();

    stepProtocol("220", null);
    stepProtocol("250", "HELO " + helohost);
    stepProtocol("250", "MAIL FROM:" + sender);
    stepProtocol("250", "RCPT TO:" + receiver);
    stepProtocol("354", "DATA");
    stepProtocol(null , "SUBJECT:" + subject);

    // send body
    BufferedReader reader = new BufferedReader(new StringReader(body));
    String line;
    while ((line = reader.readLine()) != null)
      out.println(line);

    out.println();

    reader.close();

    stepProtocol("250", ".");

    stepProtocol("221", "QUIT");
    disconnect();
  }

  void stepProtocol(String expected, String say) throws IOException {
    if(say != null)
      setRequest(say);

    if(expected != null)
      missLine(expected);
  }

  void missLine(String expected) throws IOException {
    String response = (String)getResponse();

    if(!response.startsWith(expected))
      throw new IOException(response);

    while(response.startsWith(expected + "-")) {
      response = (String)getResponse();
    }
  }

  public static void main(String args[]) {
    String mailHost = "turnpike.kana.com";

    String heloHost = "turnpike.kana.com";
    String sender   = "ashvets@kana.com";
    String receiver = "ashvets@kana.com";
    String subject  = "SMTP test";
    String body     = "Test";

    SMTPClient client = new SMTPClient(mailHost);

    client.setHeloHost(heloHost);
    client.setSender(sender);
    client.setReceiver(receiver);
    client.setSubject(subject);
    client.setBody(body);

    System.out.println("Waiting...");

    try {
      client.interact();
      System.out.println("e-mail was succesfully sent");
    }
    catch(IOException e) {
      System.out.println("e-mail wasn't succesfully sent: " +
                         e.getMessage());
    }
  }

}


