// BlockDataLongEntry.java

package org.javalobby.serfile;

import java.io.*;

/**
 *
 * Abstract class for representing short block data
 *
 * blockdatalong: TC_BLOCKDATALONG (int)<size> (byte)[size]
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public class BlockDataLongEntry extends BlockDataEntry {

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException  {
    int size = in.readInt();

    buffer = new byte[size];
    in.readFully(buffer);
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());

    int size = buffer.length;

    out.writeInt(size);
    out.write(buffer, 0, size);
  }

  /**
   * Length of this entry in bytes
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 4 + buffer.length;
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Block Data Long";
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return TC_BLOCKDATALONG;
  }

}
