// BooleanEntry.java

package org.javalobby.serfile;

import java.io.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an entry for holding boolean value
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public final class BooleanEntry implements Entry {

  /** boolean value */
  private boolean value;

  /**
   * The default constructor that creates "boolean" entry.
   */
  public BooleanEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    value = in.readBoolean();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeBoolean(value);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "boolean";
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 1;
  }

  /**
   * Get the boolean value for this entry.
   *
   * @return  the boolean value for this entry
   */
  public boolean getValue() {
    return value;
  }

  /**
   * Compares two boolean entries. They
   * will be equals if they both have the same values.
   *
   * @param   e  the boolean entry tor comparison.
   * @return  true if this boolean entry has the same value; false otherwise.
   */
  public boolean equals(BooleanEntry e) {
    return (value == e.value);
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return String.valueOf(value);
  }

}
