// CharacterEntry.java

package org.javalobby.serfile;

import java.io.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an entry for holding short value
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public final class CharacterEntry implements Entry {

  /** char value */
  private char value;

  /**
   * The default constructor that creates "char" entry.
   */
  public CharacterEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    value = in.readChar();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeChar(value);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "char";
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 2;
  }

  /**
   * Get the char value for this entry.
   *
   * @return  the char value for this entry
   */
  public char getValue() {
    return value;
  }

  /**
   * Compares two char entries. They
   * will be equals if they both have the same values.
   *
   * @param   e  the char entry tor comparison.
   * @return  true if this char entry has the same value; false otherwise.
   */
  public boolean equals(CharacterEntry e) {
    return (value == e.value);
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return String.valueOf(value);
  }

}
