// HandlesTable.java

package org.javalobby.serfile;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * The instance of this class holds all allocated objects and their handles.
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */

public class HandlesTable {

  /** current handle */
  private int currHandle;

  /** a hashtable that keeps all handled objects */
  private Hashtable hashTable = new Hashtable();

  /**
   * Default constructor that creates handles table.
   */
  public HandlesTable() {}

  /**
   * Cleat handles table
   */
  public void clear() {
    currHandle = SerConstants.baseWireHandle;

    hashTable.clear();
  }

  /**
   * Put new handled entry into hashtable.
   * Current handler will be incremented after that.
   *
   * @param entry handled entry to be added
   */
  public int put(HandleSerEntry entry) {
    int handle = currHandle++;

    hashTable.put(new Integer(handle), entry);

    return handle;
  }

  /**
   * Get handled entry from hashtable.
   *
   * @param handle  a handle to an entry that is looked for
   * @return handle  handled entry
   */
  public HandleSerEntry get(int handle) {
    return (HandleSerEntry)hashTable.get(new Integer(handle));
  }

  /**
   * Get enumeration for all elements in handled table.
   *
   * @return  enumeration for all elements in handled table
   */
  public Enumeration elements() {
    return hashTable.elements();
  }

}
