// IntegerEntry.java

package org.javalobby.serfile;

import java.io.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an entry for holding short value
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public final class IntegerEntry implements Entry {

  /** int value */
  private int value;

  /**
   * The default constructor that creates "int" entry.
   */
  public IntegerEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    value = in.readInt();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeInt(value);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "int";
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 4;
  }

  /**
   * Get the integer value for this entry.
   *
   * @return  the integer value for this entry
   */
  public int getValue() {
    return value;
  }

  /**
   * Compares two int entries. They
   * will be equals if they both have the same values.
   *
   * @param   e  the int entry tor comparison.
   * @return  true if this int entry has the same value; false otherwise.
   */
  public boolean equals(IntegerEntry e) {
    return (value == e.value);
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return String.valueOf(value);
  }

}
