// LongEntry.java

package org.javalobby.serfile;

import java.io.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an entry for holding long value
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public final class LongEntry implements Entry {

  /** short value */
  private long value;

  /**
   * The default constructor that creates "long" entry.
   */
  public LongEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    value = in.readLong();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeLong(value);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "long";
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 8;
  }

  /**
   * Get the long value for this entry.
   *
   * @return  the long value for this entry
   */
  public long getValue() {
    return value;
  }

  /**
   * Compares two long entries. They
   * will be equals if they both have the same values.
   *
   * @param   e  the long entry tor comparison.
   * @return  true if this long entry has the same value; false otherwise.
   */
  public boolean equals(LongEntry e) {
    return (value == e.value);
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return String.valueOf(value);
  }

}
