// LongStringSerEntry.java

package org.javalobby.serfile;

import java.io.*;

/**
 * Class that represents a serialized long string entry.
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public class LongStringSerEntry extends HandleSerEntry {

  /** byte array value */
  private byte[] value;

  /**
   * The default constructor that creates serialized long string entry
   *
   * @param handlesTable  table of handles
   */
  public LongStringSerEntry(HandlesTable handlesTable) {
    super(handlesTable);
  }

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException  {
    long size = in.readLong();

    value = new byte[(int)size];
    in.readFully(value);

    handle = handlesTable.put(this);
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());
    out.writeLong(value.length);
    out.write(value);
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 8 + value.length;
  }

  /**
   * Get the byte array that contains this entry.
   *
   * @return  the byte array that contains this entry
   */
  public byte[] getValue() {
    return value;
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Long String";
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return TC_LONGSTRING;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return getType() + "(" + handle + ", \"" + new String(value) + "\")";
  }

}
