// PlainEntryCollection.java

package org.javalobby.serfile;

import java.io.*;

import org.javalobby.classfile.*;

/**
 * Class-container that can hold entries. Size of collection is unknown at
 * the time of reading.
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public class PlainCollection extends EntryCollection {

  /** Table that contains all handles to allocated elements of serialized file */
  protected HandlesTable handlesTable;

  /**
   * Creates a collection that can hold only elements of specified type.
   *
   * @param clazz  the type of allowed in this collection class
   * @param handlesTable  table of handles
   */
  public PlainCollection(Class clazz, String type, HandlesTable handlesTable) {
    super(clazz, type);

    this.handlesTable = handlesTable;
  }

  /**
   * Write this collection to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short cnt = (short)entries.size();

    for(int i=0; i < cnt; i++) {
      ((Entry)entries.elementAt(i)).write(out);
    }
  }

  /**
   * Length of collection in bytes
   *
   * @return  length of collection in bytes
   */
  public long length() {
    int len = 0;
    for(int i=0; i < entries.size(); i++) {
      len += (long)((Entry)entries.elementAt(i)).length();
    }

    return len;
  }

}
