// ReferenceSerEntry.java

package org.javalobby.serfile;

import java.io.*;

/**
 * Class that represents a serialized reference entry.
 *
 * @version 1.0 05/25/2000
 * @author Alexander Shvets
 */
public class ReferenceSerEntry extends HandleSerEntry {

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException  {
    handle = in.readInt();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());
    out.writeInt(handle);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Reference";
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 4;
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return TC_REFERENCE;
  }

}
