// Comparator.java

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;

public class Comparator {

  public static Vector compare(ClassFile classFile1, ClassFile classFile2) {
    Vector diff = new Vector();

    ConstPool constPool1 = classFile1.getConstPool();
    ConstPool constPool2 = classFile2.getConstPool();

    for(short i=1; i < constPool1.size(); i++) {
      PoolEntry pei = (PoolEntry)constPool1.elementAt(i);

      if(pei == null) {
        continue;
      }

      if(!contains(constPool1, constPool2, pei)) {
        diff.addElement(pei);
      }
    }

    return diff;
  }

  public static boolean contains(ConstPool constPool1, ConstPool constPool2, PoolEntry pe) {
    for(short i=1; i < constPool2.size(); i++) {
      PoolEntry pei = (PoolEntry)constPool2.elementAt(i);

      if(pei == null) {
        continue;
      }

      if(pe instanceof StringConst && pei instanceof StringConst) {
        short nameIndex1 = ((StringConst)pe).getNameIndex();
        short nameIndex2 = ((StringConst)pei).getNameIndex();

        String value1 = ((Utf8Const)constPool1.getEntry(nameIndex1)).getValue();
        String value2 = ((Utf8Const)constPool2.getEntry(nameIndex2)).getValue();

        if(value1.equals(value2)) {
          return true;
        }
      }
      else if(pe instanceof ClassConst && pei instanceof ClassConst) {
        short nameIndex1 = ((ClassConst)pe).getNameIndex();
        short nameIndex2 = ((ClassConst)pei).getNameIndex();

        String className1 = ((Utf8Const)constPool1.getEntry(nameIndex1)).getValue();
        String className2 = ((Utf8Const)constPool2.getEntry(nameIndex2)).getValue();

        if(className1.equals(className2)) {
          return true;
        }
      }
      else if(pe instanceof NameAndTypeConst && pei instanceof NameAndTypeConst) {
        NameAndTypeConst natConst1 = (NameAndTypeConst)pe;
        NameAndTypeConst natConst2 = (NameAndTypeConst)pei;

        short nameIndex1 = natConst1.getNameIndex();
        short nameIndex2 = natConst2.getNameIndex();

        short valueIndex1 = natConst1.getValueIndex();
        short valueIndex2 = natConst2.getValueIndex();

        String className1 = ((Utf8Const)constPool1.getEntry(nameIndex1)).getValue();
        String className2 = ((Utf8Const)constPool2.getEntry(nameIndex2)).getValue();

        String signature1 = ((Utf8Const)constPool1.getEntry(valueIndex1)).getValue();
        String signature2 = ((Utf8Const)constPool2.getEntry(valueIndex2)).getValue();

        if(className1.equals(className2) && signature1.equals(signature2)) {
          return true;
        }
      }
      else if(pe instanceof MemberConst && pei instanceof MemberConst) {
        MemberConst memberConst1 = (MemberConst)pe;
        MemberConst memberConst2 = (MemberConst)pei;

        short nameIndex1 = memberConst1.getNameIndex();
        short nameIndex2 = memberConst2.getNameIndex();

        short valueIndex1 = memberConst1.getValueIndex();
        short valueIndex2 = memberConst2.getValueIndex();

        ClassConst classConst1 = (ClassConst)constPool1.getEntry(nameIndex1);
        ClassConst classConst2 = (ClassConst)constPool2.getEntry(nameIndex2);

        NameAndTypeConst natConst1 = (NameAndTypeConst)constPool1.getEntry(valueIndex1);
        NameAndTypeConst natConst2 = (NameAndTypeConst)constPool2.getEntry(valueIndex2);

        String className1 = ((Utf8Const)constPool1.getEntry(classConst1.getNameIndex())).getValue();
        String className2 = ((Utf8Const)constPool2.getEntry(classConst2.getNameIndex())).getValue();

        String memberName1 = ((Utf8Const)constPool1.getEntry(natConst1.getNameIndex())).getValue();
        String memberName2 = ((Utf8Const)constPool2.getEntry(natConst2.getNameIndex())).getValue();

        String signature1 = ((Utf8Const)constPool1.getEntry(natConst1.getValueIndex())).getValue();
        String signature2 = ((Utf8Const)constPool2.getEntry(natConst2.getValueIndex())).getValue();

        if(className1.equals(className2) && memberName1.equals(memberName2) &&
           signature1.equals(signature1)) {
          return true;
        }
      }
      else {
        if(pe.equals(pei)) {
          return true;
        }
      }
    }

    return false;
  }

  public static Vector findUnusedEntries(ClassFile classFile) {
    Vector unusedEntries = new Vector();

    ConstPool constPool = classFile.getConstPool();

    for(short i=1; i < constPool.size(); i++) {
      PoolEntry constant = (PoolEntry)constPool.elementAt(i);
      if(constant == null) {
        continue;
      }

      if(!isEntryUsed(constant, classFile)) {
        unusedEntries.addElement(constant);
      }
    }

    return unusedEntries;
  }

  private static boolean isEntryUsed(PoolEntry poolEntry, ClassFile classFile) {
    ConstPool constPool = classFile.getConstPool();

    ClassConst thisClassConst = (ClassConst)constPool.getEntry(classFile.getThisClass());
    Utf8Const thisUtf8Const   = (Utf8Const)constPool.getEntry(thisClassConst.getNameIndex());

    if(thisClassConst.equals(poolEntry) || thisUtf8Const.equals(poolEntry)) {
      return true;
    }

    ClassConst superClassConst = (ClassConst)constPool.getEntry(classFile.getSuperClass());
    Utf8Const superUtf8Const   = (Utf8Const)constPool.getEntry(superClassConst.getNameIndex());

    if(superClassConst.equals(poolEntry) || superUtf8Const.equals(poolEntry)) {
      return true;
    }

    EntryCollection interfaces = classFile.interfaces();

    for(short i=0; i < interfaces.size(); i++) {
      InterfaceEntry entry = (InterfaceEntry)interfaces.elementAt(i);

      short nameIndex = entry.getNameIndex();

      ClassConst classConst = (ClassConst)constPool.getEntry(nameIndex);
      Utf8Const utf8Const   = (Utf8Const)constPool.getEntry(classConst.getNameIndex());

      if(poolEntry.equals(classConst) || poolEntry.equals(utf8Const)) {
        return true;
      }
    }

    EntryCollection fields = classFile.fields();

    if(isEntryUsedWithMember(poolEntry, fields, constPool)) {
      return true;
    }

    EntryCollection methods = classFile.methods();

    if(isEntryUsedWithMember(poolEntry, methods, constPool)) {
      return true;
    }

    EntryCollection attributes = classFile.attributes();

    for(short i=0; i < attributes.size(); i++) {
      AttributeEntry attributeEntry = (AttributeEntry)attributes.elementAt(i);
      if(isEntryUsedWithAttribute(poolEntry, attributeEntry, constPool)) {
        return true;
      }
    }

    return false;
  }

  private static boolean isEntryUsedWithMember(PoolEntry poolEntry, EntryCollection members,
                                               ConstPool constPool) {
    for(short i=0; i < members.size(); i++) {
      MemberEntry entry = (MemberEntry)members.elementAt(i);

      Utf8Const utf8Const1 = (Utf8Const)constPool.getEntry(entry.getNameIndex());
      Utf8Const utf8Const2 = (Utf8Const)constPool.getEntry(entry.getValueIndex());

      if(poolEntry.equals(utf8Const1) || poolEntry.equals(utf8Const2)) {
        return true;
      }

      EntryCollection attributes = entry.getAttributes();

      for(short j=0; j < attributes.size(); j++) {
        AttributeEntry attributeEntry = (AttributeEntry)attributes.elementAt(j);
        if(isEntryUsedWithAttribute(poolEntry, attributeEntry, constPool)) {
          return true;
        }
      }
    }

    return false;
  }

  private static RankTable rankTable = ParameterRanking.getInstance();

  private static boolean isEntryUsedWithAttribute(PoolEntry poolEntry, AttributeEntry attribute,
                                                  ConstPool constPool) {
    Utf8Const utf = (Utf8Const)constPool.getEntry(attribute.getNameIndex());

    if(utf.equals(poolEntry)) {
      return true;
    }

    String attrName = attribute.getName(constPool);

    if(attrName.equals(AttributeEntry.SOURCE_FILE)) {
      SourceFileAttribute a = new SourceFileAttribute(attribute);

      Utf8Const utf8Const = (Utf8Const)constPool.getEntry(a.getIndex());

      if(utf8Const.equals(poolEntry)) {
        return true;
      }
    }
    else if(attrName.equals(AttributeEntry.CONSTANT_VALUE)) {
      ConstantValueAttribute a = new ConstantValueAttribute(attribute);

      StringConst stringConst = (StringConst)constPool.getEntry(a.getIndex());
      Utf8Const utf8Const     = (Utf8Const)constPool.getEntry(stringConst.getNameIndex());

      if(stringConst.equals(poolEntry) || utf8Const.equals(poolEntry)) {
        return true;
      }
    }
    else if(attrName.equals(AttributeEntry.INNER_CLASSES)) {
      InnerClassesAttribute a = new InnerClassesAttribute(attribute);

      InnerClassEntry[] innerClasses = a.getInnerClasses();

      for(int j=0; j < innerClasses.length; j++) {
        InnerClassEntry innerClass = innerClasses[j];

        short index1 = innerClass.getInnerClassIndex();
        short index2 = innerClass.getOuterClassIndex();
        short index3 = innerClass.getInnerNameIndex();

        ClassConst classConst1 = (ClassConst)constPool.getEntry(index1);
        Utf8Const utf8Const1   = (Utf8Const)constPool.getEntry(classConst1.getNameIndex());

        if(index2 != 0) {
          ClassConst classConst2 = (ClassConst)constPool.getEntry(index2);
          Utf8Const utf8Const2   = (Utf8Const)constPool.getEntry(classConst2.getNameIndex());

          if(index3 != 0) {
            Utf8Const utf8Const3 = (Utf8Const)constPool.getEntry(index3);

            if(classConst1.equals(poolEntry) || utf8Const1.equals(poolEntry) ||
               classConst2.equals(poolEntry) || utf8Const2.equals(poolEntry) ||
               utf8Const3.equals(poolEntry)) {
              return true;
            }
          }
          else { // index3 == 0
            if(classConst1.equals(poolEntry) || utf8Const1.equals(poolEntry) ||
               classConst2.equals(poolEntry) || utf8Const2.equals(poolEntry)) {
              return true;
            }
          }
        }
        else { // index2 == 0
          if(index3 != 0) {
            Utf8Const utf8Const3 = (Utf8Const)constPool.getEntry(index3);

            if(classConst1.equals(poolEntry) || utf8Const1.equals(poolEntry) ||
               utf8Const3.equals(poolEntry)) {
              return true;
            }
          }
          else {
            if(classConst1.equals(poolEntry) || utf8Const1.equals(poolEntry)) {
              return true;
            }
          }
        }
      }
    }
    else if(attrName.equals(AttributeEntry.EXCEPTIONS)) {
      ExceptionsAttribute a = new ExceptionsAttribute(attribute);

      ExceptionEntry[] exceptions = a.getExceptions();

      for(int j=0; j < exceptions.length; j++) {
        ExceptionEntry exception = exceptions[j];

        ClassConst classConst = (ClassConst)constPool.getEntry(exception.getNameIndex());
        Utf8Const utf8Const   = (Utf8Const)constPool.getEntry(classConst.getNameIndex());

        if(classConst.equals(poolEntry) || utf8Const.equals(poolEntry)) {
          return true;
        }
      }
    }
    else if(attrName.equals(AttributeEntry.CODE)) {
      CodeAttribute a = new CodeAttribute(attribute);

      MethodBody methodBody = a.getMethodBody();
      Instruction[] instructions = methodBody.getInstructions();

      for(int i=0, pos=0; i < instructions.length; i++) {
        Instruction instruction = instructions[i];

        byte tag = instruction.getTag();
        OpcodeDescription description = OpcodeTable.getDescription(tag);

        if(description.length > 1) {
          byte[] parameter = instruction.getParameter();

          String group = rankTable.getRank(tag).key;

          if(group.equals(ParameterRanking.FIELD_RANK)  ||
             group.equals(ParameterRanking.METHOD_RANK) ||
             group.equals(ParameterRanking.IMETHOD_RANK)) {
            short index = (short)Converter.getShort(parameter, 0);
            MemberConst memberConst = (MemberConst)constPool.getEntry(index);

            ClassConst classConst     = (ClassConst)constPool.getEntry(memberConst.getNameIndex());
            NameAndTypeConst natConst = (NameAndTypeConst)constPool.getEntry(memberConst.getValueIndex());

            Utf8Const entry1 = (Utf8Const)constPool.getEntry(classConst.getNameIndex());
            Utf8Const entry2 = (Utf8Const)constPool.getEntry(natConst.getNameIndex());
            Utf8Const entry3 = (Utf8Const)constPool.getEntry(natConst.getValueIndex());

            if(memberConst.equals(poolEntry) ||
               classConst.equals(poolEntry) || natConst.equals(poolEntry) ||
               entry1.equals(poolEntry) || entry2.equals(poolEntry) || entry3.equals(poolEntry)) {
              return true;
            }
          }
          else if(group.equals(ParameterRanking.LDC_RANK)) {
            short index = (short)Converter.getByte(parameter, 0);
            PoolEntry pe = (PoolEntry)constPool.getEntry(index);

            if(pe instanceof IntegerConst) {
              IntegerConst integerConst = (IntegerConst)pe;

              if(integerConst.equals(poolEntry)) {
                return true;
              }
            }
            else if(pe instanceof FloatConst) {
              FloatConst floatConst = (FloatConst)pe;

              if(floatConst.equals(poolEntry)) {
                return true;
              }
            }
            else if(pe instanceof StringConst) {
              StringConst stringConst = (StringConst)pe;
              Utf8Const entry = (Utf8Const)constPool.getEntry(stringConst.getNameIndex());

              if(stringConst.equals(poolEntry) || entry.equals(poolEntry)) {
                return true;
              }
            }
          }
          else if(group.equals(ParameterRanking.LDC2_RANK)) {
            short index = (short)Converter.getShort(parameter, 0);
            PoolEntry pe = (PoolEntry)constPool.getEntry(index);

            if(pe instanceof LongConst) {
              LongConst longConst = (LongConst)pe;

              if(longConst.equals(poolEntry)) {
                return true;
              }
            }
            else if(pe instanceof DoubleConst) {
              DoubleConst doubleConst = (DoubleConst)pe;

              if(doubleConst.equals(poolEntry)) {
                return true;
              }
            }
            else if(pe instanceof StringConst) {
              StringConst stringConst = (StringConst)pe;
              Utf8Const entry = (Utf8Const)constPool.getEntry(stringConst.getNameIndex());

              if(stringConst.equals(poolEntry) || entry.equals(poolEntry)) {
                return true;
              }
            }
          }
          else if(group.equals(ParameterRanking.PRIMITIVE_RANK)) {

          }
          else if(group.equals(ParameterRanking.MULTI_RANK)) {

          }
        }
      }

      ExceptionHandlerEntry[] exceptionHandlers = a.getExceptionHandlers();

      for(int j=0; j < exceptionHandlers.length; j++) {
        ExceptionHandlerEntry exceptionHandler = exceptionHandlers[j];

        ClassConst classConst = (ClassConst)constPool.getEntry(exceptionHandler.getCatchType());
        Utf8Const utf8Const   = (Utf8Const)constPool.getEntry(classConst.getNameIndex());

        if(classConst.equals(poolEntry) || utf8Const.equals(poolEntry)) {
          return true;
        }
      }

      AttributeEntry[] attributes = a.getAttributes();

      for(short i=0; i < attributes.length; i++) {
        if(isEntryUsedWithAttribute(poolEntry, attributes[i], constPool)) {
          return true;
        }
      }
    }
    else if(attrName.equals(AttributeEntry.LINE_NUMBER_TABLE)) {
      // do nothing
    }
    else if(attrName.equals(AttributeEntry.LOCAL_VARIABLE_TABLE)) {
      LocalVariableTableAttribute a = new LocalVariableTableAttribute(attribute);

      LocalVariableEntry[] localVariables = a.getLocalVariables();

      for(int j=0; j < localVariables.length; j++) {
        LocalVariableEntry localVariable = localVariables[j];

        short index1 = localVariable.getNameIndex();
        short index2 = localVariable.getDescriptionIndex();

        Utf8Const utf8Const1 = (Utf8Const)constPool.getEntry(index1);
        Utf8Const utf8Const2 = (Utf8Const)constPool.getEntry(index2);

        if(utf8Const1.equals(poolEntry) || utf8Const2.equals(poolEntry)) {
          return true;
        }
      }
    }
    else {
      // unknown attribute, trying to use simplest scheme
      byte[] buffer = attribute.getBuffer();
      if(buffer.length == 2) {
        short index = (short)((buffer[0] & 0xff) << 8 | (buffer[1] & 0xff));
        if(index < constPool.size()) {
          PoolEntry pe = constPool.getEntry(index);

          if(pe instanceof Utf8Const) {
            Utf8Const utf8Const = (Utf8Const)pe;

            if(utf8Const.equals(poolEntry)) {
              return true;
            }
          }
        }
      }
    }

    return false;
  }

  public static void main(String[] args) throws Exception {
    ClassFile classFile1 = new ClassFile();
    ClassFile classFile2 = new ClassFile();

    classFile1.read(args[0]);
    classFile2.read(args[1]);

/*    Vector diff1 = Comparator.compare(classFile1, classFile2);

    System.out.println("Class " + args[0] + " has unique constant pool entries vs. " + args[1]);
    for(int i=0; i < diff1.size(); i++) {
      Entry entry = (Entry)diff1.elementAt(i);
      System.out.println(entry);
    }

    Vector diff2 = Comparator.compare(classFile2, classFile1);

    System.out.println("Class " + args[1] + " has unique constant pool entries vs. " + args[0]);
    for(int i=0; i < diff2.size(); i++) {
      System.out.println(diff2.elementAt(i));
    }
*/
    Vector unused1 = Comparator.findUnusedEntries(classFile1);

    System.out.println("!!! Unused constant pool entries for class " + args[0]);
    for(int i=0; i < unused1.size(); i++) {
      PoolEntry poolEntry = (PoolEntry)unused1.elementAt(i);

      String[] str =  poolEntry.resolve(classFile1.getConstPool());

      System.out.println(poolEntry + " : " + Converter.merge(str, " "));
    }

    Vector unused2 = Comparator.findUnusedEntries(classFile2);

    System.out.println("Unused constant pool entries for class " + args[1]);
    for(int i=0; i < unused2.size(); i++) {
      System.out.println(unused2.elementAt(i));
    }
  }


}
