// Dietolog.java

package org.javalobby.task;

import java.io.*;
import java.util.*;

import org.javalobby.task.compactor.*;
import org.javalobby.task.event.*;
import org.javalobby.task.util.*;

public class Dietolog implements NotificationListener {

  public Dietolog(String descrFileName) {
    CompactTaskDescription td = new CompactTaskDescription();

    try {
      td.load(descrFileName);
    }
    catch(Exception e) {
      e.printStackTrace();
      return;
    }

    String from            = td.getFrom();
    String to              = td.getTo();
    String classPath       = td.getClassPath() + File.pathSeparator + from;
    String mainClassName   = td.getMainClass();
    String archiveName     = td.getArchiveName();
    String classpathBounds = td.getClasspathBounds();
    String packagesBounds  = td.getPackagesBounds();

    // Create helping classes

    Archiver archiver = new Archiver();
    Copier copier     = new Copier();

    DependenciesFinder finder = new CompleteFinder(classPath, from, to, archiver, copier);
//    DependenciesFinder finder = new MethodFinder(classPath, to, archiver, copier);

    archiver.addNotificationListener(this);
    copier.addNotificationListener(this);
    finder.addNotificationListener(this);

    // Search dependencies

    finder.setClasspathBounds(classpathBounds);
    finder.setPackagesBounds(packagesBounds);

    // show classpath entries
    finder.showClasspath();

    finder.analyze(mainClassName);

    Vector classesList = finder.getFoundedClasses();
    Vector dllList     = finder.getDllList();

    // zip founded classes
    if(archiveName != null) {
      System.out.println("\nZipping...\n");
      archiver.zip(classesList, to, to + "/" + archiveName);
    }

    // print founded classes
    System.out.println("\nFounded classes:\n");
    for(int i=0; i < classesList.size(); i++) {
      String name = (String)classesList.elementAt(i);
      System.out.println((i+1) + " " + name);
    }

/*    Vector methodsList = finder.getFoundedMethods();
    int ii = methodsList.size();
    boolean flag = true;
    while(flag) {
      flag = false;
      for(int k = 0; k < ii - 1; k++) {
        String el1 = (String)methodsList.elementAt(k);
        String el2 = (String)methodsList.elementAt(k + 1);
        if(el1.compareTo(el2) > 0) {
          String pair = (String)methodsList.elementAt(k + 1);
          methodsList.removeElementAt(k + 1);
          methodsList.insertElementAt(pair, k);
          flag = true;
        }
      }
    }

    // print founded methods
    System.out.println("\nFounded methods:\n");
    for(int i=0; i < methodsList.size(); i++) {
      String name = (String)methodsList.elementAt(i);
      System.out.println((i+1) + " " + name);
    }
*/

    // print founded dll libraries
    System.out.println("\nDll libraries:\n");
    for(int i=0; i < dllList.size(); i++) {
      String name = (String)dllList.elementAt(i);
      System.out.println(name + ".dll");
    }
  }

  public void notification(NotificationEvent event) {
    System.out.println("   " + event.getNotification() +
                       "... " + event.getDescription());
  }

  public static void main(String[] argv) {

    String descrFileName = null;

    try {
      descrFileName = argv[0];
    }
    catch(Exception e) {
      System.out.println("\nUsage: java org.javalobby.task.Dietolog task_descr.cmp");
      System.out.println("         where task_descr.cmp is a file that contains");
      System.out.println("         description of executed task");
      System.out.println("Author : " + Constants.AUTHOR);
      System.out.println("e-mail : " + Constants.EMAIL);
      return;
    }

    new Dietolog(descrFileName);
  }
}
