// Obfuscator.java

package org.javalobby.task;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.event.*;
import org.javalobby.task.obfuscator.*;

// An example for performing single Migrate-task

public class Obfuscator implements NotificationListener {

  private NamesTable names;

  private String from, to, classPath, mainClass;
  private boolean anonymousPackageMode;
  private boolean[] conditions;
  private Vector deniedList;

  public Obfuscator(String from, String to, String classPath,
                            String mainClass, boolean anonymousPackageMode,
                            boolean[] conditions, Vector deniedList) {
    this.from        = from;
    this.to          = to;
    this.classPath   = classPath;
    this.mainClass   = mainClass;
    this.anonymousPackageMode = anonymousPackageMode;
    this.conditions  = conditions;
    this.deniedList  = deniedList;
  }

  public void notification(NotificationEvent event) {
    System.out.println("  " + event.getNotification() +
                       " " + event.getDescription());
  }

  public GroupTask collectInfo() throws Exception {
    names = new NamesTable(from, classPath, mainClass, deniedList);

    CollectInfoTask collectTask = new CollectInfoTask(names);
    GroupTask groupTask = new GroupTask(collectTask, from, true);

    groupTask.addNotificationListener(this);
    names.addNotificationListener(this);

    groupTask.processing();

    groupTask.removeNotificationListener(this);

    return groupTask;
  }

  public GroupTask obfuscate() throws Exception {

    ObfuscateTask obfuscateTask =
        new ObfuscateTask(to, names, anonymousPackageMode, conditions);
    GroupTask groupTask = new GroupTask(obfuscateTask, from, true);

    groupTask.addNotificationListener(this);
    obfuscateTask.addNotificationListener(this);

    groupTask.processing();

    groupTask.removeNotificationListener(this);
    obfuscateTask.removeNotificationListener(this);

    return groupTask;
  }

  public static void main(String[] argv) {
    TaskDescription td = new TaskDescription();

    try {
      td.load(argv[0]);
    }
    catch(Exception e) {
      System.out.println("\nUsage: java org.javalobby.task.Obfuscator task_descr.obf");
      System.out.println("         where task_descr.obf is a file that contains");
      System.out.println("         description of executed task");
      System.out.println("Author : " + Constants.AUTHOR);
      System.out.println("e-mail : " + Constants.EMAIL);
      return;
    }

    String from   = td.getFrom();
    String to     = td.getTo();
    String classPath = td.getClassPath() + File.pathSeparator + from;
    String mainClass = td.getMainClass();

    boolean anonymousPackageMode = td.getAnonymousMode();

    boolean[] conditions = td.getConditions();

    Vector deniedList = td.getDeniedList();

    Obfuscator obfuscator =
        new Obfuscator(from, to, classPath, mainClass,
                               anonymousPackageMode, conditions, deniedList);

    try {
      System.err.println("Collect info group command started...\n");
      GroupTask groupTask = obfuscator.collectInfo();
      System.err.println("Collect info group command finished.\n");
      System.err.println(groupTask.getProcessedFiles() + " files scanned.");

      System.err.println("Generating names...\n");
      obfuscator.names.generateAllNames(anonymousPackageMode, conditions);

//      obfuscator.names.print();
      System.err.println("Generation finished.\n");

      System.err.println("Obfuscate group command started...\n");
      groupTask = obfuscator.obfuscate();
      System.err.println("Obfuscate group command finished.\n");

      System.err.println(groupTask.getProcessedFiles() + " files processed.");
      System.err.println(groupTask.getMovements() + " movements occured.");
      System.err.println(groupTask.getReplacements() + " replacements occured.");
      System.exit(0);
    }
    catch(Exception e) {
      e.printStackTrace();
      System.err.println(e);
    }
  }
}
