// TemplateGroupTask.java

package org.javalobby.task;

import java.io.*;

import org.javalobby.classfile.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.event.*;
import org.javalobby.task.stripper.*;
import org.javalobby.task.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.classfile.attribute.*;


class SampleTask extends Task {

  private ConstPool constPool;

  private String zipFileName;

  private PrintWriter logWriter;

  public SampleTask(PrintWriter logWriter) {
    super(null);

    this.logWriter = logWriter;
  }

  public void body(ClassFile classFile) {
    logWriter.println("Class: " + classFile.getClassName().replace('/', '.'));

    constPool = classFile.getConstPool();
    EntryCollection fields = classFile.fields();
    EntryCollection methods = classFile.methods();

    // analyze classfile
  }

  public void action() throws IOException {
    logWriter.println();
  }

  public void closeZip() {
    super.closeZip();
    logWriter.println("Archive's end.\n");
    zipFileName = null;
  }

}


public class TemplateGroupTask extends GroupTask
                               implements NotificationListener, GroupListener {

  private PrintWriter logWriter;

  public TemplateGroupTask(Task task, String from, boolean recursive, PrintWriter logWriter) {
    super(task, from, recursive);

    this.logWriter = logWriter;

    this.addNotificationListener(this);
    this.addGroupListener(this);
    task.addNotificationListener(this);
  }

  public void newFileSelected(GroupEvent event) {
    String fileName = event.getFileName();

    if(fileName.endsWith(".class")) {
      logWriter.println("File : " + fileName);
    }
    else if(fileName.endsWith(".jar") || fileName.endsWith(".zip")) {
      logWriter.println("Archive : " + fileName + "\n");
    }
  }

  public void notification(NotificationEvent event) {
    System.out.println("  " + event.getNotification());
  }

  public static void main(String[] argv) {
    String dir = null;
    String log = null;
    boolean recursively = true;
    try {
      dir = new File(argv[0]).getPath();
      log = new File(argv[1]).getPath();
    }
    catch(Exception e) {
      System.out.println("\nUsage: java TemplateGroupTask dir log [recursively]");
      System.out.println("Where dir is a directory or archive;");
      System.out.println("      recursively is boolean\n");
      System.out.println("Author : " + Constants.AUTHOR);
      System.out.println("e-mail : " + Constants.EMAIL);
      return;
    }

    try {
      recursively = Boolean.valueOf(argv[2]).booleanValue();
    }
    catch(Exception e) {}

    try {
      PrintWriter logWriter = new PrintWriter(new FileWriter(log));

      SampleTask task = new SampleTask(logWriter);

      GroupTask groupTask = new TemplateGroupTask(task, dir, recursively, logWriter);

      System.out.println("Seek group command started...\n");

      groupTask.processing();

      System.out.println("Seek group command finished...\n");

      logWriter.close();

      System.out.println(groupTask.getProcessedFiles() + " files processed.");
      System.out.println(groupTask.getReplacements() + " replacements occured.");
    }
    catch(Exception e) {
      System.out.println(e);
    }
  }

}
