// Task.java

package org.javalobby.task.basic;

import java.util.*;
import java.util.zip.*;
import java.io.*;

import org.javalobby.classfile.*;
import org.javalobby.task.event.*;

// Partial realization of Solvable interface.
// This class acts as bean-generator of events ReplaceEvent

public class Task implements Solvable {

  private NotificationSupport support = new NotificationSupport(this);

  private int replacements;
  private boolean isMoved = false;

  protected ClassFile classFile;

  protected String fromName;

  protected String to;

  public Task(String to) {
    this.to = to;
  }

  public void solve(String name) throws IOException {
    solve(name, (String)null);
  }

  public void solve(String name, String methodName) throws IOException {
    replacements = 0;
    classFile = new ClassFile();
    fromName = new File(name).getCanonicalPath();
    classFile.read(fromName);
    solve0(methodName);
  }

  public void closeZip() {
    ZipSupport.closeZip();
  }

  public void solve(ZipFile zipFile, ZipEntry zipEntry)
                    throws IOException {
    replacements = 0;
    classFile = new ClassFile();

    fromName = zipFile.getName() + "#" + zipEntry.getName();

    classFile.read(zipFile.getInputStream(zipEntry));
    solve0((String)null);
  }

  private void solve0(String methodName) throws IOException {
    if(methodName == null)
      solve(classFile);
    else
      solve(classFile, methodName);

    if(replacements > 0)
      classFile.setChanged(true);

    action();
  }

  public void action() throws IOException {
    if(to == null) return;

    String toName = getDestinationFileName();

    mkDirs(toName);

    if(toName.indexOf("#") != -1) {
//      System.out.println("classFile.length() " + classFile.length());
//      System.out.println("classFile.getClassName() " + classFile.getClassName());

      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      classFile.write(baos);
      ZipSupport.write(baos.toByteArray(), toName);
    }
    else
      classFile.write(toName);
  }

  public void copy(String dirName, String fileName) throws IOException {
    if(to == null) return;

    fromName = fileName;

    String toName = to + File.separator + fileName.substring(dirName.length()+1);

    mkDirs(toName);

    String name = toName;
    if(fromName.equals(toName)) {
      name = dirName + File.separator +
             new Long(new Date().getTime()).toString();
    }

    BufferedInputStream in = new BufferedInputStream(
                                 new FileInputStream(fromName));
    BufferedOutputStream out = new BufferedOutputStream(
                                   new FileOutputStream(name));

    for(int n; ((n = in.read()) != -1);) out.write(n);

    in.close(); out.close();

    if(fromName.equals(toName)) {
      new File(fromName).delete();
      new File(name).renameTo(new File(toName));
    }
  }

  public void copy(ZipFile zipFile, ZipEntry zipEntry) throws IOException {
    if(to == null) return;

    fromName = zipFile.getName() + "#" + zipEntry.getName();
    String toName = to + File.separator +
                    new File(zipFile.getName()).getName() + "#" +
                    zipEntry.getName();
    mkDirs(toName);

    ZipSupport.write(zipFile.getInputStream(zipEntry), toName);
  }

  private void mkDirs(String toName) {
    isMoved = !fromName.equals(toName);

    if(replacements > 0 || isMoved) {
      String name = null;
      int pos = toName.lastIndexOf("#");
      if(pos != -1)
        name = toName.substring(0, pos);
      else
        name = toName;

      pos = name.lastIndexOf(File.separatorChar);

      File toDir = new File(name.substring(0, pos+1));
      if(!toDir.exists())
        toDir.mkdirs();
    }
  }

  public void solve(ClassFile classFile) {
    solve(classFile, (String)null);
  }

  public void solve(ClassFile classFile, String methodName) {
    if(methodName == null)
      body(classFile);
    else
      body(classFile, methodName);

    if(replacements > 0)
      classFile.setChanged(true);
  }

  public String getDestinationFileName() {
    String destination = null;

    int pos = fromName.indexOf("#");
    if(pos != -1) {
      int pos2 = fromName.lastIndexOf(File.separator);
      String zipName = null;
      if(pos2 == -1)
        zipName = fromName.substring(0, pos);
      else
        zipName = fromName.substring(pos2+1, pos);

      return to + File.separator + zipName + "#" + fromName.substring(pos+1);
    }
    else
      return to + File.separator +
             classFile.getClassName().replace('/', File.separatorChar) +
             ".class";
  }

  public void body(ClassFile classFile) {}

  public void body(ClassFile classFile, String methodName) {}


  public int getReplacements() {
    return replacements;
  }

  public boolean isMoved() {
    return isMoved;
  }

  public synchronized void addNotificationListener(NotificationListener l) {
    support.addNotificationListener(l);
  }
  public synchronized void removeNotificationListener(NotificationListener l) {
    support.removeNotificationListener(l);
  }

  // fire NotificationEvent to registered listeners
  protected void fireNotification(String notification, String description) {
    support.fireNotification(notification, description);
    replacements++;
  }

}
