// ZipSupport.java

package org.javalobby.task.basic;

import java.io.*;
import java.util.zip.*;

import org.javalobby.classfile.*;

public class ZipSupport {

  private static ZipOutputStream zipOutputStream;

  private ZipSupport() {}

  public static void closeZip() {
    try {
      if(zipOutputStream != null)
        zipOutputStream.close();
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    zipOutputStream = null;
  }

  public static void write(InputStream is, String toName) {
    try {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      byte[] from = new byte[4096];

      int n = -1;
      while((n = is.read(from)) > 0) {
        baos.write(from, 0, n);
      }
      write(baos.toByteArray(), toName);

      is.close();
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }

  public static void write(byte[] from, String to) {
    try {
//      System.out.println("to " + to);
      int pos = to.indexOf("#");
      String zipName   = to.substring(0, pos);
      String entryName = to.substring(pos+1).replace(File.separatorChar, '/');
//      System.out.println("zipName " + zipName);
//      System.out.println("entryName " + entryName);

      if(zipOutputStream == null)
        openZip(zipName);

      putEntryToZip(from, entryName);
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }

  private static void openZip(String zipName) {
    FileOutputStream fos = null;
    try {
      fos = new FileOutputStream(zipName);
    }
    catch(IOException e) {
      System.out.println("Error while opening file " + zipName + ".");
      return;
    }

    CheckedOutputStream csum = new CheckedOutputStream(fos, new CRC32());
    zipOutputStream = new ZipOutputStream(new BufferedOutputStream(fos));
//    zipOutputStream.setMethod(ZipOutputStream.STORED); // !
  }


  private static void putEntryToZip(byte[] from, String entryName)
                      throws IOException {
//    ByteArrayOutputStream baos = new ByteArrayOutputStream(classFile.length());
//    classFile.write(baos);

    CheckedInputStream in =
            new CheckedInputStream(
                  new BufferedInputStream(
                      new ByteArrayInputStream(from/*baos.toByteArray()*/)),
                  new CRC32()
                );
//    int sz = classFile.length();
    int sz = from.length;
    byte buf[] = new byte[sz];
    in.read(buf, 0, sz);

    in.close();
    ZipEntry zipEntry = new ZipEntry(entryName);
    zipEntry.setSize(sz);
    zipEntry.setCrc(in.getChecksum().getValue());

    zipOutputStream.putNextEntry(zipEntry);
    zipOutputStream.write(buf, 0, sz);
    zipOutputStream.closeEntry();
  }

}