// Archiver.java

package org.javalobby.task.compactor;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.task.event.*;

// A class for representing operations with zip archive
public class Archiver {

  private NotificationSupport support = new NotificationSupport(this);

  public Archiver() {}

  // try to unzip fileList' files  from archiveName
  // for accelerating this process size of list fileList is
  // decremented each time when file is inside an archive

  public int unzip(Vector fileList, String fromZip, String toDir) {
    return unzip(fileList, 0, fromZip, toDir);
  }

  public int unzip(Vector fileList, int index, String fromZip, String toDir) {

    int cnt = 0;

    ZipFile zipFile = null;
    try {
      zipFile = new ZipFile(fromZip);
    }
    catch(FileNotFoundException e1) {
      System.out.println("File " + fromZip + " does not found.");
      return cnt;
    }
    catch(IOException e2) {
      System.out.println(e2);
      return cnt;
    }

    for(int i=index; i < fileList.size(); i++) {
      String item     = fileList.elementAt(i).toString().replace('.', '/');
      String fromFile = item + ".class";
      String toFile   = toDir + "/" + item + ".class";

      try {
        if(unzip(zipFile, fromFile, toFile)) {
          cnt++;
          fireNotification(fromFile, "extracting file");
        }
//        else
//          System.out.println("Cannot unzip " + fromFile + " from " + fromZip);
      }
      catch(IOException e3) {
        System.out.println(e3);
        continue;
      }
    }

    return cnt;
  }

  public boolean unzip(ZipFile zipFile, String fromFile, String toFile)
                    throws IOException {
    ZipEntry zipEntry = zipFile.getEntry(fromFile);

    if((zipEntry == null) || zipEntry.isDirectory()) return false;

    BufferedInputStream in = new BufferedInputStream(
                                 zipFile.getInputStream(zipEntry));
    makeDirs(toFile);

    BufferedOutputStream out = new BufferedOutputStream(
                                   new FileOutputStream(toFile));

    for(int n; ((n = in.read()) != -1);) out.write(n);

    in.close(); out.close();

    return true;
  }


  // create an archive archiveName and add to it files from fileList
  // fileList is decremented each time when next file is added to archive

  public void zip(Vector fileList, String fromDir, String toZip) {
    zip(fileList, 0, fromDir, toZip);
  }

  public void zip(Vector fileList, int index,
                  String fromDir, String toZip) {

    FileOutputStream fos = null;
    try {
      fos = new FileOutputStream(toZip);
    }
    catch(IOException e) {
      System.out.println("Error while opening file " + toZip + ".");
      return;
    }

    CheckedOutputStream csum = new CheckedOutputStream(fos, new CRC32());
    ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(fos));
    out.setMethod(ZipOutputStream.STORED); // !

    for(int i=index; i < fileList.size(); i++) {
      String item     = fileList.elementAt(i).toString().replace('.', '/');
      String fromFile = fromDir + "/" + item + ".class";
      String toFile   = item + ".class";

      try {
        zip(out, fromFile, toFile);
        fireNotification(fromFile, "compressing file");
      }
      catch(IOException e2) {
        System.out.println(e2);
        continue;
      }
    }

    try { out.close(); } catch(IOException e) {}
  }

  public void zip(ZipOutputStream out, String fromFile, String toFile)
                  throws IOException {

    File f = new File(fromFile);
    CheckedInputStream in =
            new CheckedInputStream(
                  new BufferedInputStream(new FileInputStream(f)),
                  new CRC32()
                );
    int sz = (int)f.length();

    byte buf[] = new byte[sz];
    in.read(buf, 0, sz);
    in.close();

    ZipEntry zipEntry = new ZipEntry(toFile);
    zipEntry.setSize(sz);
    zipEntry.setCrc(in.getChecksum().getValue());

    out.putNextEntry(zipEntry);
    out.write(buf, 0, sz);
    out.closeEntry();
  }

  public static void makeDirs(String fileName) {

    int pos = fileName.lastIndexOf("/");
    if(pos != -1) {
      String dirName =
               fileName.substring(0, pos).replace('/', File.separatorChar);
      File dir = new File(dirName);
      if(!dir.exists()) dir.mkdirs();
    }
  }

  public synchronized void addNotificationListener(NotificationListener l) {
    support.addNotificationListener(l);
  }
  public synchronized void removeNotificationListener(NotificationListener l) {
    support.removeNotificationListener(l);
  }

  // fire NotificationEvent to registered listeners
  protected void fireNotification(String notification, String description) {
    support.fireNotification(notification, description);
  }

  public static void main(String[] args) {
    Vector classesList = new Vector();

    // add qualified classes names
    classesList.addElement("AlbumPlayer");

    Archiver archiver = new Archiver();
    archiver.zip(classesList, 0, ".", "test2.zip");
  }

}

