// CompactTask2.java

package org.javalobby.task.compactor;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.util.*;

public class CompactTask2 extends Task {

  private ClassAnalyzer analyzer;
  private OneToManyTable classToParentsList = new OneToManyTable();

  private ConstPool constPool;

  private Vector classesList, methodsList, discoveredMethodsList, dllList;
  private PackagesBounds bounds;

  public CompactTask2(String classPath, Vector classesList,
                      Vector methodsList, Vector discoveredMethodsList,
                      Vector dllList, PackagesBounds bounds) {
    super(null);
    this.classesList           = classesList;
    this.methodsList           = methodsList;
    this.discoveredMethodsList = discoveredMethodsList;
    this.dllList               = dllList;
    this.bounds                = bounds;

    analyzer = new BytecodeClassAnalyzer(classPath, classToParentsList);
  }

  public void body(ClassFile classFile, String methodName) {
    constPool = classFile.getConstPool();

    EntryCollection methods = classFile.methods();
    for(short i=0; i < methods.size(); i++) {
      MethodEntry method = (MethodEntry)methods.elementAt(i);
      short nameIndex  = method.getNameIndex();
      short valueIndex = method.getValueIndex();
      Utf8Const entry1 = (Utf8Const)constPool.getEntry(nameIndex);
      Utf8Const entry2 = (Utf8Const)constPool.getEntry(valueIndex);

      String name = entry1.getValue() + entry2.getValue();
      if(name.equals(methodName)) {
        try {
          EntryCollection attributes = method.getAttributes();

          for(short j=0; j < attributes.size(); j++) {
            AttributeEntry attribute = (AttributeEntry)attributes.elementAt(j);

            String attrName = attribute.getName(constPool);
            if(attrName.equals("Code")) {
              CodeAttribute codeAttribute = new CodeAttribute(attribute);
              analyzeMethodBody(codeAttribute.getMethodBody().getInstructions());
            }
            else if(attrName.equals("Exceptions")) {
              ExceptionsAttribute exceptionsAttribute = new ExceptionsAttribute(attribute);
              ExceptionEntry[] exceptions = exceptionsAttribute.getExceptions();

              for(int k=1; k < exceptions.length; k++) {
                ExceptionEntry exception = exceptions[k];
                tryAddClass(exception.resolve(constPool)[0]);
              }
            }
          }
        }
        catch(Exception e) { e.printStackTrace(); }
      }
    }
  }

  private void analyzeMethodBody(Instruction[] instructions) {
    for(int i=0; i < instructions.length; i++) {
      String[] str = instructions[i].resolve(constPool);

      if(str.length > 1) {
        StringTokenizer st = new StringTokenizer(str[1]);

        String type = st.nextToken();
        if(type.equals("CLASS")) {
          String className = st.nextToken();
          tryAddClass(className);
        }
        else if(type.equals("METHOD")) {
          analyzeMethod(st, instructions, i);
        }
        else if(type.equals("FIELD")) {
          String className = st.nextToken();
          st.nextToken(); // fieldName
          String signatureName = st.nextToken();

          tryAddClass(className);
          tryAddClass(signatureName);
        }
      }
    }
  }

  private void analyzeMethod(StringTokenizer st, Instruction[] instructions, int i) {
    String className     = st.nextToken().replace('/', '.');
    String methodName    = st.nextToken();
    String signatureName = st.nextToken();

    String fullName = className + "." + methodName + signatureName;

    if(!methodsList.contains(fullName) &&
       !discoveredMethodsList.contains(fullName)) {

      discoveredMethodsList.addElement(fullName);

      tryAddClass(className);

//      Vector params = Util.getParamList(signatureName);
      MethodSignature methodSignature = new MethodSignature(signatureName);
      Signature[] parameters = methodSignature.getParameters();

      for(int j=0; j < parameters.length; j++)
        tryAddClass(parameters[j].getKey());

      if(methodName.equals("forName") || methodName.startsWith("class$")) {
        String s = getLDCValue(instructions, i);
        if(s != null) {
          try {
            Class.forName(s);
            tryAddClass(s);
          }
          catch(Exception ex) {}
        }
      }
    }
    else if(methodName.equals("loadLibrary")) {
      String s = getLDCValue(instructions, i);
      if(s != null && !dllList.contains(s))
        dllList.addElement(s);
    }
  }

  private String getLDCValue(Instruction[] instructions, int i) throws NumberFormatException {
    for(int cnt = i-1; cnt >= 0; cnt--) {
      String[] str = instructions[cnt].resolve(constPool);
      if(str.length > 1) {
        StringTokenizer st = new StringTokenizer(str[1]);

        if(st.nextToken().startsWith("ld")) {
          return st.nextToken();
        }
      }
    }

    return null;
  }
  public void tryAddClass(String className) {
    className = Signature.cleanName(className);
    if(className == null) return;
    className = className.replace('/', '.');

    if((bounds == null) || (!bounds.contains(className))) {
      if(!classesList.contains(className)) {
        classesList.addElement(className);
        discoveredMethodsList.addElement(className + ".<clinit>()V");
        discoveredMethodsList.addElement(className + ".finalize()V");
//        discoveredMethodsList.addElement(className + ".<init>()V");
        System.out.println("!className " + className);
      }
    }

    analyzeInheritance(className);
  }

  public void tryAddMethod(String fullName) {
    if(fullName == null) return;

    if(!methodsList.contains(fullName) &&
       !discoveredMethodsList.contains(fullName))
      discoveredMethodsList.addElement(fullName);
  }

  Vector analyzedClasses = new Vector();

  private void analyzeInheritance(String className) {
    if(analyzedClasses.contains(className))
      return;

    analyzedClasses.addElement(className);

    try {
      analyzer.analyze(className);
    }
    catch(Throwable e) {
      System.err.println("!!!" + e);
      return;
    }

    Enumeration e = classToParentsList.many(className.replace('.', '/'));
    while(e.hasMoreElements()) {
      ManyElement element = (ManyElement)e.nextElement();
      String parent = (String)element.getValue();
      tryAddClass(parent);
      analyzeInheritance(parent);
    }
  }

}

