// CompactTaskDescription.java

package org.javalobby.task.compactor;

import java.io.*;
import java.util.*;

public class CompactTaskDescription {

  private String from, to;

  private String classPath = System.getProperty("java.class.path");
  private String mainClass, archiveName;
  private String classpathBounds, packagesBounds;

  public CompactTaskDescription() {}

  public void load(String fileName) throws IOException {
    BufferedReader reader = new BufferedReader(new FileReader(fileName));
    String line = null;
    while(true) {
      line = reader.readLine();
      if(line == null) break;
      line = line.trim();
      if(line.length() == 0) continue;

      if(line.startsWith("from="))
        from = getRightPart(line);
      else if(line.startsWith("to="))
        to = getRightPart(line);
      else if(line.startsWith("classpath="))
        classPath = getRightPart(line);
      else if(line.startsWith("mainclass="))
        mainClass = getRightPart(line);
      else if(line.startsWith("archive="))
        archiveName = getRightPart(line);
      else if(line.startsWith("CLASSPATH_BOUNDS="))
        classpathBounds = getRightPart(line);
      else if(line.startsWith("PACKAGES_BOUNDS="))
        packagesBounds = getRightPart(line);
    }

    reader.close();
  }

  private String getRightPart(String line) {
    int pos = line.indexOf("=");
    line = line.substring(pos+1).trim();
    if(line.length() == 0)
      line = null;

    return line;
  }

  public void save(String fileName) throws IOException {
    PrintWriter writer = new PrintWriter(new BufferedWriter(
                             new FileWriter(fileName)));
    writer.println("from= "           + from);
    writer.println("to= "             + to);
    writer.println("classpath= "      + classPath);
    writer.println("mainclass= "      + mainClass);
    writer.print("archive= ");
    if(archiveName != null)
      writer.println(archiveName);
    else
      writer.println();
    writer.println("CLASSPATH_BOUNDS= " + classpathBounds);
    writer.println("PACKAGES_BOUNDS= "  + packagesBounds);

    writer.close();
  }

  public String getFrom() { return from; }

  public String getTo() { return to; }

  public String getClassPath() { return classPath; }

  public String getMainClass() { return mainClass; }

  public String getArchiveName() { return archiveName; }

  public String getClasspathBounds() { return classpathBounds; }

  public String getPackagesBounds() { return packagesBounds; }

  public void setFrom(String from) {
    this.from = from;
  }

  public void setTo(String to) {
    this.to = to;
  }

  public void setClassPath(String classPath) {
    this.classPath = classPath;
  }

  public void setMainClass(String mainClass) {
    this.mainClass = mainClass;
  }

  public void setArchiveName(String archiveName) {
    this.archiveName = archiveName;
  }

  public void setClasspathBounds(String classpathBounds) {
    this.classpathBounds = classpathBounds;
  }


  public void setPackagesBounds(String packagesBounds) {
    this.packagesBounds = packagesBounds;
  }


  public static void main(String[] argv) throws IOException {
    CompactTaskDescription td = new CompactTaskDescription();

    td.load(argv[0]);

    System.out.println("From            : " + td.getFrom());
    System.out.println("To              : " + td.getTo());
    System.out.println("ClassPath       : " + td.getClassPath());
    System.out.println("mainClass       : " + td.getMainClass());
    System.out.println("Archive         : " + td.getArchiveName());
    System.out.println("CLASSPATH_BOUNDS: " + td.getArchiveName());
    System.out.println("PACKAGES_BOUNDS : " + td.getArchiveName());

    td.save(argv[0] + "1");
  }

}