// CompleteFinder.java

package org.javalobby.task.compactor;

import java.io.*;
import java.util.*;

import org.javalobby.task.compactor.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.event.*;
import org.javalobby.task.util.*;

public class CompleteFinder extends DependenciesFinder {

  private Vector discoveredClassesList = new Vector();

  private Task task;
  private GroupTask groupTask;

  protected String from, to;
  private Archiver archiver;
  private Copier copier;

  public CompleteFinder(String classPath, String from, String to,
                        Archiver archiver, Copier copier) {
    super(classPath);

    this.from     = from;
    this.to       = to;
    this.archiver = archiver;
    this.copier   = copier;
  }

  public void analyze(String className) {
    classesList.addElement(className);

    task = new CompactTask(null, classesList, discoveredClassesList,
                           dllList, packagesBounds);
    groupTask = new GroupTask(task, to, true);

    groupTask.addNotificationListener(new NotificationListener() {
      public void notification(NotificationEvent event) {
        fireNotification(event.getNotification(), event.getDescription());
      }
    });

    int pos = 0;
    while(true) {
      collectFiles(pos);
      discoverFiles(pos);

      if(discoveredClassesList.size() == 0)
        break;

      pos = classesList.size();

      copyFiles();

      System.out.println("Number of founded classes = " + classesList.size());
    }
  }

  private void collectFiles(int pos) {
    Enumeration e = classPathVariable.elements();

    int index = pos;
    while(e.hasMoreElements() && (index < classesList.size())) {

      String name = (String)e.nextElement();
      name = name.replace('\\', '/');

      if(classpathBounds.contains(name)) continue;

      File fileEntry = new File(name);
      if(!fileEntry.isDirectory()) {
        fireNotification(name, "Seek in archive");
        index += archiver.unzip(classesList, pos, name, to);
      }
      else {
        fireNotification(name, "Seek in directory");
        index += copier.copy(classesList, pos, name, to);
      }
    }
  }

  public void discoverFiles(int analyzedPoint) {

    if((classesList.size() == 0) || (classesList == null))
      return;

    if(analyzedPoint >= classesList.size())
      return;

    String[] list = new String[classesList.size()-analyzedPoint];
    for(int i=analyzedPoint, j=0; i < classesList.size(); i++, j++) {
      list[j] = classesList.elementAt(i).toString().replace('.', File.separatorChar) +
                ".class";
    }

    groupTask.processing(list);
  }


  private void copyFiles() {
    for(int i=0; i < discoveredClassesList.size(); i++)
      classesList.addElement(discoveredClassesList.elementAt(i));

    discoveredClassesList.removeAllElements();
  }

}
