// Copier.java

package org.javalobby.task.compactor;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.task.event.*;

// a class performs copying if class file is not in archive

public class Copier {

  private NotificationSupport support = new NotificationSupport(this);

  public Copier() {}

  public int copy(Vector fileList, String fromDir, String toDir) {
    return copy(fileList, 0, fromDir, toDir);
  }

  public int copy(Vector fileList, int index, String fromDir, String toDir) {

    int cnt = 0;

    fromDir = fromDir.replace('\\', '/');
    toDir = toDir.replace('\\', '/');

    for(int i=index; i < fileList.size(); i++) {
      String fileName = fileList.elementAt(i).toString().replace('.', '/');
      String fromFile = fromDir + "/" + fileName + ".class";
      String toFile   = toDir   + "/" + fileName + ".class";
      try {
        copy(fromFile, toFile);
        cnt++;
        fireNotification(fromFile, "copying file");
      }
      catch(FileNotFoundException e1) {
//        System.out.println("\nFile " + fromFile + " does not found.");
        continue;
      }
      catch(IOException e2) {
        System.out.println("\n"+e2);
        continue;
      }
    }

    return cnt;
  }

  public void copy(String fromFile, String toFile)
                   throws FileNotFoundException, IOException {

    BufferedInputStream in = new BufferedInputStream(
                                 new FileInputStream(fromFile));

    makeDirs(toFile);

    BufferedOutputStream out = new BufferedOutputStream(
                                   new FileOutputStream(toFile));

    for(int n; ((n = in.read()) != -1);) out.write(n);

    in.close(); out.close();
  }

  public static void makeDirs(String fileName) {

    int pos = fileName.lastIndexOf("/");
    if(pos != -1) {
      String dirName =
               fileName.substring(0, pos).replace('/', File.separatorChar);
      File dir = new File(dirName);
      if(!dir.exists()) dir.mkdirs();
    }
  }

  public synchronized void addNotificationListener(NotificationListener l) {
    support.addNotificationListener(l);
  }
  public synchronized void removeNotificationListener(NotificationListener l) {
    support.removeNotificationListener(l);
  }

  // fire NotificationEvent to registered listeners
  protected void fireNotification(String notification, String description) {
    support.fireNotification(notification, description);
  }

  public static void main(String[] args) {

    Vector classesList = new Vector();

    // add qualified classes names
    classesList.addElement("AlbumPlayer");

    Copier copier = new Copier();
    copier.copy(classesList, ".", "d:/Work/Dietolog/classes/Result");
  }

}
