// GroupSupport.java

package org.javalobby.task.event;

import java.util.*;

public class GroupSupport {

  private Vector listeners;

  private Object source;

  /**
   * @sourceBean  The bean to be given as the source for any events.
   */

  public GroupSupport(Object source) {
    this.source = source;
  }

  /**
   * Add a NotificationListener to the listener list.
   *
   * @param listener  The NotificationListener to be added
   */

  public synchronized void addGroupListener(GroupListener listener) {
    if(listeners == null)
      listeners = new java.util.Vector();

    listeners.addElement(listener);
  }

  /**
   * Remove a NotificationListener from the listener list.
   *
   * @param listener  The NotificationListener to be removed
   */

  public synchronized void removeGroupListener(GroupListener listener) {
    if(listeners == null)
      return;

    listeners.removeElement(listener);
  }

  /**
   * Report a bound property update to any registered listeners.
   * No event is fired if old and new are equal and non-null.
   *
   */
  public void fireNewFileSelected(String fileName) {
    Vector targets = null;
    synchronized (this) {
      if(listeners == null)
        return;

      // 1. Cloning of listeners list to eliminate race condition
      targets = (Vector)listeners.clone();
    }

    // 2. Creating event
    GroupEvent evt = new GroupEvent(source, fileName);

    // 3. Notification of all registered listeners
    for (int i = 0; i < targets.size(); i++) {
      GroupListener target = (GroupListener)targets.elementAt(i);
      target.newFileSelected(evt);
    }
  }

}
