// NotificationSupport.java

package org.javalobby.task.event;

import java.util.*;

public class NotificationSupport {

  private Vector listeners;

  private Object source;

  /**
   * @sourceBean  The bean to be given as the source for any events.
   */

  public NotificationSupport(Object source) {
    this.source = source;
  }

  /**
   * Add a NotificationListener to the listener list.
   *
   * @param listener  The NotificationListener to be added
   */

  public synchronized void addNotificationListener(
                      NotificationListener listener) {
    if(listeners == null)
      listeners = new java.util.Vector();

    listeners.addElement(listener);
  }

  /**
   * Remove a NotificationListener from the listener list.
   *
   * @param listener  The NotificationListener to be removed
   */

  public synchronized void removeNotificationListener(
                      NotificationListener listener) {
    if(listeners == null)
      return;

    listeners.removeElement(listener);
  }

  /**
   * Report a bound property update to any registered listeners.
   * No event is fired if old and new are equal and non-null.
   *
   */
  public void fireNotification(String notification, String description) {
    Vector targets = null;
    synchronized (this) {
      if(listeners == null)
        return;

      // 1. Cloning of listeners list to eliminate race condition
      targets = (Vector)listeners.clone();
    }

    // 2. Creating event
    NotificationEvent evt = new NotificationEvent(source, notification, description);

    // 3. Notification of all registered listeners
    for (int i = 0; i < targets.size(); i++) {
      NotificationListener target = (NotificationListener)targets.elementAt(i);
      target.notification(evt);
    }
  }

}
