// MigrateTask.java

package org.javalobby.task.migrator;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.task.*;
import org.javalobby.task.util.*;
import org.javalobby.task.obfuscator.*;


// Migration is a sort of obfuscation, when:
// 1. you have to change only names of packages, not names of classes, fields
//    or methods
// 2. name of package is changed on base of migration table

public class MigrateTask extends ObfuscateTask {

  private Vector migrateTable = new Vector();

  public MigrateTask(String to, Vector migrateTable) {
    super(to, null, true, new boolean[] {true, false, false, false});

    this.migrateTable = migrateTable;

    sort();
  }

  public MigrateTask(String tableFileName, String to) {
    super(to, null, true, new boolean[] {true, false, false, false});

    try {
      BufferedReader br = new BufferedReader(new FileReader(tableFileName));

      String line = null;
      while((line = br.readLine()) != null) {
        if(line.length() > 0) {
          line = line.trim().replace('.', '/');
          migrateTable.addElement(line);
        }
      }
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    sort();
  }

/*  public String getDestinationFileName() {
    String destination = toDirName;
    String name = super.getDestinationFileName();
    System.out.println("destination " + destination);
    System.out.println("name " + name);
    int pos = name.indexOf("#");
    if(pos != -1)
      destination = destination + File.separator + name.substring(0, pos) + "#";
    else
      destination = destination + File.separator;

    return destination +
           classFile.getClassName().replace('/', File.separatorChar) + ".class";
  }
*/

  protected String findNewValue(String className, String value, int type) {
    if(type == CLASS_NAME)
      if(value != null)
        return findNewValue(value);

    return null;
  }

  private String findNewValue(String value) {
    int pos1 = value.lastIndexOf('/');

    String packagePart = null;
    String classPart   = null;
    if(pos1 != -1) {
      packagePart = value.substring(0, pos1);
      classPart   = value.substring(pos1+1);
    }
    else {
      packagePart = "";
      classPart   = value;
    }

    for(int i=migrateTable.size()-1; i >= 0; i--) {
      String line = (String)migrateTable.elementAt(i);
      int pos2 = line.indexOf(':');
      String fromPackage = line.substring(0, pos2);
      String toPackage   = line.substring(pos2+1);
      if(packagePart.equals(fromPackage))
        if(toPackage.length() == 0)
          return classPart;
        else
          return toPackage + '/' + classPart;
    }

    for(int i=migrateTable.size()-1; i >= 0; i--) {
      String line = (String)migrateTable.elementAt(i);
      int pos2 = line.indexOf(':');
      String fromPackage = line.substring(0, pos2);
      String toPackage   = line.substring(pos2+1);
      if(!packagePart.startsWith(toPackage) &&
         packagePart.startsWith(fromPackage)) {
        toPackage = toPackage + packagePart.substring(fromPackage.length());
        if(toPackage.length() == 0)
          return classPart;
        else
          return toPackage + '/' + classPart;
      }
    }

    return value;
  }

  public void sort() {
    int i = migrateTable.size();
    boolean flag = true;
    while(flag) {
      flag = false;
      for(int k = 0; k < i - 1; k++) {
        Object o1 = migrateTable.elementAt(k);
        Object o2 = migrateTable.elementAt(k + 1);
        String s1 = o1.toString().substring(0, o1.toString().indexOf(':'));
        String s2 = o2.toString().substring(0, o2.toString().indexOf(':'));
        if(s1.compareTo(s2) > 0) {
          Object tmp = o2;
          migrateTable.removeElementAt(k + 1);
          migrateTable.insertElementAt(o2, k);
          flag = true;
        }
      }
    }
  }

  public static void main(String[] argv) {
    String tableFileName = "migration.table";

    MigrateTask migrateTask = new MigrateTask("result", tableFileName);

    try {
      migrateTask.solve("p1/Test.class");
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

}
