// NamesTable.java

package org.javalobby.task.obfuscator;

import java.io.*;
import java.util.*;

import org.javalobby.task.event.*;
import org.javalobby.task.util.*;

public class NamesTable {

  private String ANONYMOUS_PACKAGE = "<anonymous package>";

  private NotificationSupport support = new NotificationSupport(this);

  private ClassAnalyzer analyzer;

  private NameGenerator packageNameGen      = new NameGenerator();
  private NameGenerator classNameGen        = new NameGenerator();
  private NameGenerator nameGen             = new NameGenerator();
  private NameGenerator sharedGen           = new NameGenerator();

  private PairsTable packagesList           = new PairsTable();
  private PairsTable classesList            = new PairsTable();
  private Vector otherClassesList           = new Vector();
  private NameRepository namesList          = new NameRepository();
  private NameRepository innerNamesList     = new NameRepository();

  private PairsTable dependencies           = new PairsTable();
  private OneToManyTable classToParentsList = new OneToManyTable();

  private String mainClass;
  private Vector deniedList;

  public NamesTable(String fromDir, String classPath, String mainClass,
                    Vector deniedList) {
  /*  if(System.getProperty("java.version").startsWith("1.2"))
      analyzer = new LoaderClassAnalyzer(fromDir, classToParentsList, namesList);
    else

      analyzer = new BytecodeClassAnalyzer(classPath, classToParentsList, namesList);
*/

    this.mainClass  = mainClass;
    this.deniedList = deniedList;

    analyzer = new BytecodeClassAnalyzer(classPath, classToParentsList, namesList);

    fillReservedNames();
  }

  private Vector reservedNames      = new Vector();
  private Vector reservedBeginning  = new Vector();

  private void fillReservedNames() {
    reservedNames.addElement("main");
    reservedNames.addElement("<init>");
    reservedNames.addElement("<clinit>");
    reservedNames.addElement("readExternal");
    reservedNames.addElement("writeExternal");
    reservedNames.addElement("readObject");
    reservedNames.addElement("writeObject");

    reservedBeginning.addElement("class$");

/*    reservedBeginning.addElement("this$");
    reservedBeginning.addElement("constructor$");
    reservedBeginning.addElement("access$");
    reservedBeginning.addElement("block$");
*/
  }

  public boolean isReservedName(String name) {
    if(reservedNames.contains(name))
      return true;

    for(int i=0; i < reservedBeginning.size(); i++) {
      String n = (String)reservedBeginning.elementAt(i);
      if(name.startsWith(n))
        return true;
    }

    return false;
  }

  public void addClassName(String className) {
    if(className != null) {
      if(isSystemClass(className)) return;

      classesList.put(className);
      addPackageName(className);
    }
  }

  private void addPackageName(String className) {
    int pos = className.lastIndexOf('/');
    String packageName = null;
    if(pos == -1) {
      if(!packagesList.contains(ANONYMOUS_PACKAGE))
        packagesList.put(ANONYMOUS_PACKAGE);
      return;
    }

    String line = className;
    while(true) {
      pos = line.lastIndexOf('/');
      if(pos == -1)
        break;

      packageName = line.substring(0, pos);

      if(!packagesList.contains(packageName) && packageName != null)
        packagesList.put(packageName);

      line = packageName;
    }
  }

  public void addDependency(String fromName, String toName, String name) {
    if(!fromName.equals(toName)) {
      dependencies.put(fromName + ":" + toName, name);
      if(!otherClassesList.contains(fromName))
        otherClassesList.addElement(fromName);
    }
  }

  public void addName(String className, String name, boolean intacted) {
    NameElement n = (NameElement)namesList.put(className, name);
    n.name.intacted = intacted;
  }

  public void addInnerName(String className, String name, boolean intacted) {
    NameElement n = (NameElement)innerNamesList.put(className, name);
    n.name.intacted = intacted;
  }

  public String getObfuscatedClassName(String className) {
    if(className != null) {
      Pair pair = classesList.get(className);
      if(pair != null)
        return (String)pair.getSecond();
    }

    return null;
  }

  public String getObfuscatedName(String className, String name) {
    NameElement n = (NameElement)namesList.get(className, name);
    if(n != null)
      return (String)n.name.pair.getSecond();

    return null;
  }

  public String getObfuscatedInnerName(String className, String name) {
    NameElement n = (NameElement)innerNamesList.get(className, name);
    if(n != null)
      return (String)n.name.pair.getSecond();

    return null;
  }

  public PairsTable getClassesList() { return classesList; }

  public void generateAllNames(boolean anonymousPackageMode,
                               boolean[] conditions) {
    generatePackagesNames(anonymousPackageMode, conditions[0]);

    if(conditions[1])
      generateClassesNames();
    else
      replicateClassesNames();

    if(anonymousPackageMode)
      resolveCollision();

    if(conditions[2] || conditions[3]) {
      findParentClasses();
      createCommonNames();
      generateNamesForOutClasses();
      generateNamesForInClasses(conditions[2], conditions[3]);
    }
  }

  private void generatePackagesNames(boolean anonymousPackageMode,
                                     boolean isPackage) {
    packageNameGen.clear();
    packagesList.sort();  // !

    Enumeration e = packagesList.elements();

    if(anonymousPackageMode) {
      while(e.hasMoreElements()) {
        Pair pair = (Pair)e.nextElement();
        pair.setSecond("");
      }
    }
    else {
      while(e.hasMoreElements()) {
        Pair pair = (Pair)e.nextElement();

        String oldName = (String)pair.getFirst();
        if(oldName.length() == 0 || oldName.equals(ANONYMOUS_PACKAGE))
          pair.setSecond("");
        else if(isPackage) {
          String newName = packageNameGen.getNewName();
          pair.setSecond(convertParentPackages(oldName, newName, false));
        }
      }
    }
  }

  private String convertParentPackages(String oldName, String newName,
                                       boolean isClass) {
    if(!isClass) {
      int pos = oldName.lastIndexOf('/');
      if(pos != -1) {
        String part = oldName.substring(0, pos);
        String newParent = (String)packagesList.get(part, false).getSecond();
        if(newParent != null) {
          if(newParent.length() > 0)
            return newParent + '/' + newName;
          else
            return newName;
        }
        else
          return part + '/' + newName;
      }

      return newName;
    }
    else { // for class
      String part = null;
      int pos = oldName.lastIndexOf('/');
      if(pos == -1)
        part = ANONYMOUS_PACKAGE;
      else
        part = oldName.substring(0, pos);

      String newParent = (String)packagesList.get(part, false).getSecond();
      if(newParent != null) {
        if(newParent.length() > 0)
          return newParent + '/' + newName;
        else
          return newName;
      }
      else
        return part + '/' + newName;
    }
  }

  private void generateClassesNames() {
    classNameGen.clear();
    classesList.sort();  // !

    // generator for inner classes names
    NameGenerator ic_gen = new NameGenerator();

    Enumeration e = classesList.elements();
    while(e.hasMoreElements()) {
      Pair pair = (Pair)e.nextElement();

      String oldName = (String)pair.getFirst();

      if(deniedList.contains(oldName)) {
//        pair.setSecond(convertParentPackages(oldName, oldName, true));
        pair.setSecond(oldName);
      }
      else if(oldName.equals(mainClass)) {
        pair.setSecond(convertParentPackages(oldName, oldName, true));
      }
      else {
        String s = classNameGen.getNewName();
        String newName = Character.toUpperCase(s.charAt(0)) + s.substring(1);
        int pos = oldName.lastIndexOf('$');
        if(pos == -1) { // no inner class
          pair.setSecond(convertParentPackages(oldName, newName, true));
        }
        else { // inner class
          String ss = ic_gen.getNewName();
          String in = Character.toUpperCase(ss.charAt(0)) + ss.substring(1);
          pair.setSecond(convertParentClass(oldName, pos, in));
        }
      }
    }
  }

  private void replicateClassesNames() {
    Enumeration e = classesList.elements();
    while(e.hasMoreElements()) {
      Pair pair = (Pair)e.nextElement();
      String oldName = (String)pair.getFirst();
      String newName = null;

      int pos = oldName.lastIndexOf('/');
      if(pos == -1)
        newName = oldName;
      else
        newName = oldName.substring(pos+1);

      pair.setSecond(convertParentPackages(oldName, newName, true));
    }
  }

  private void resolveCollision() {
    Enumeration e1 = classesList.elements();
    while(e1.hasMoreElements()) {
      Pair pair1     = (Pair)e1.nextElement();
      String first1  = (String)pair1.getFirst();
      String second1 = (String)pair1.getSecond();

      boolean isCollision = false;
      Enumeration e2 = classesList.elements();
      while(e2.hasMoreElements()) {
        Pair pair2     = (Pair)e2.nextElement();
        String first2  = (String)pair2.getFirst();
        String second2 = (String)pair2.getSecond();
        if(second2.equals(second1) && pair1 != pair2) {
          if(!isCollision)
            pair1.setSecond(pair1.getFirst());

          isCollision = true;
          pair2.setSecond(pair2.getFirst());
        }
      }
    }

//    System.out.println("???");
//    printClasses();
  }

  private String convertParentClass(String oldName, int pos, String newName) {
    String part2 = oldName.substring(pos);
    String part1 = oldName.substring(0, pos);

    StringBuffer buff = new StringBuffer();

    while(true) {
      String newParent = (String)classesList.get(part1, false).getSecond();
      if(newParent != null) {
        if(buff.length() == 0)
          return newParent + '$' + newName;
        else
          return newParent + buff.toString() + '$' + newName;
      }
      else {
        pos = part1.lastIndexOf('$');
        if(pos == -1)
          break;
        part2 = part1.substring(pos);
        part1 = part1.substring(0, pos);
        buff.insert(0, part2);
      }
    }

    return newName;
  }

  Vector analyzedClasses = new Vector();

  private void findParentClasses() {
    Enumeration e = classesList.elements();
    while(e.hasMoreElements()) {
      Pair pair = (Pair)e.nextElement();
      analyzeInheritance((String)pair.getFirst());
    }

    e = otherClassesList.elements();
    while(e.hasMoreElements())
      analyzeInheritance((String)e.nextElement());

    analyzedClasses.removeAllElements();
    analyzedClasses = null;
  }

  //  Find parent classes for current class + names (fields and methods)
  //  that are inside of these parents.

  public void analyzeInheritance(String className) {
    if(analyzedClasses.contains(className)) return;

    analyzedClasses.addElement(className);

    fireNotification(className, "analyzing parent class");

    try {
      analyzer.analyze(className);
    }
    catch(Throwable e) {
      System.err.println("!!!" + e);
//      e.printStackTrace();
      return;
    }

    Enumeration e = classToParentsList.many(className);
    while(e.hasMoreElements()) {
      ManyElement element = (ManyElement)e.nextElement();
      String parent = (String)element.getValue();
      analyzeInheritance(parent);
    }
  }

  private void createCommonNames() {
    Enumeration e1 = classesList.elements();
    while(e1.hasMoreElements()) {
      Pair pair = (Pair)e1.nextElement();
      String className = (String)pair.getFirst();
      fireNotification(className, "creating common names");

      Enumeration e2 = namesList.many(className);
      while(e2.hasMoreElements()) {
        NameElement ni = (NameElement)e2.nextElement();
        putNameToDependentClasses(className, ni);
      }
    }
  }

  private void putNameToDependentClasses(String className, NameElement n) {
    String name = (String)n.getValue();

    Vector dependList = new Vector();

    // put name to parent classes (inheritance)

    Enumeration e = classToParentsList.many(className);
    while(e.hasMoreElements()) {
      ManyElement element = (ManyElement)e.nextElement();
      String parent = (String)element.getValue();
      NameElement ni = (NameElement)namesList.get(parent, name);
      spreadValues(ni, n, dependList);
    }

    // put name to dependent classes (utilization)

    e = dependencies.elements();
    while(e.hasMoreElements()) {
      Pair p = (Pair)e.nextElement();
      String name2    = (String)p.getSecond();
      if(!name2.equals(name)) continue;

      String pairName = (String)p.getFirst();
      int pos = pairName.indexOf(':');
      String fromName = pairName.substring(0, pos);
      String toName   = pairName.substring(pos+1);

      if(className.equals(fromName)) {
        NameElement ni = (NameElement)namesList.get(toName, name);
        spreadValues(ni, n, dependList);
      }
      else if(className.equals(toName)) {
        NameElement ni = (NameElement)namesList.get(fromName, name);
        spreadValues(ni, n, dependList);
      }
    }

    Enumeration e1 = namesList.ones();
    while(e1.hasMoreElements()) {
      String cn = (String)e1.nextElement();
      Enumeration e2 = namesList.many(cn);
      while(e2.hasMoreElements()) {
        NameElement ni = (NameElement)e2.nextElement();
        if(n.name.pair != ni.name.pair && dependList.contains(ni.name.pair)) {
          spreadValues(ni, n, dependList);
        }
      }
    }
  }

  private void spreadValues(NameElement n1, NameElement n2, Vector dependList) {
    if(n1 != null && n2.name.pair != n1.name.pair) {
      if(!dependList.contains(n1.name.pair))
        dependList.addElement(n1.name.pair);

      n1.name.pair = n2.name.pair;

      if(n1.name.shared)      n2.name.shared = true;
      else if(n2.name.shared) n1.name.shared = true;

      if(n1.name.intacted)      n2.name.intacted = true;
      else if(n2.name.intacted) n1.name.intacted = true;
    }
  }

  // determine names for classes that are out of obfuscation area
  private void generateNamesForOutClasses() {
    Enumeration e1 = namesList.ones();
    while(e1.hasMoreElements()) {
      String className = (String)e1.nextElement();

      if(!classesList.contains(className)) {
        Enumeration e2 = namesList.many(className);
        while(e2.hasMoreElements()) {
          NameElement ni = (NameElement)e2.nextElement();
          String oldName = (String)ni.name.pair.getFirst();
          ni.name.pair.setSecond(oldName);
        }
      }
    }
  }

  // determine names for classes that are inside of obfuscation area
  private void generateNamesForInClasses(boolean isFields, boolean isMethods) {
    sharedGen.setSeed(1000);

    Enumeration e1 = classesList.elements();
    while(e1.hasMoreElements()) {
      Pair pair1 = (Pair)e1.nextElement();
      String oldClassName = (String)pair1.getFirst();
      String newClassName = (String)pair1.getSecond();

      nameGen.clear();

      // Exclude names that are:
      // a). used in other classes and are shared with names of this class
      // b). reserved
      // c). method/field name is shared with string const
      Enumeration e2 = namesList.many(oldClassName);
      while(e2.hasMoreElements()) {
        NameElement ni = (NameElement)e2.nextElement();
        String oldName = (String)ni.name.pair.getFirst();
        String newName = (String)ni.name.pair.getSecond();

        if(isReservedName(oldName) || ni.name.intacted)  {
          ni.name.pair.setSecond(oldName);
          nameGen.excludeName(oldName);
          continue;
        }

        if(newName != null)
          nameGen.excludeName(newName);
      }

      e2 = innerNamesList.many(oldClassName);
      while(e2.hasMoreElements()) {
        NameElement ni = (NameElement)e2.nextElement();
        String oldName = (String)ni.name.pair.getFirst();
        String newName = (String)ni.name.pair.getSecond();
        if(ni.name.intacted) {
          ni.name.pair.setSecond(oldName);
          nameGen.excludeName(oldName);
          continue;
        }
        else if(oldName.length() == 0) {
          ni.name.pair.setSecond(oldName);
        }
      }

      e2 = namesList.many(oldClassName);
      while(e2.hasMoreElements()) {

        NameElement ni = (NameElement)e2.nextElement();
        String oldName = (String)ni.name.pair.getFirst();
        String newName = (String)ni.name.pair.getSecond();

        if(isReservedName(oldName))
          continue;

        if(newName == null) {
          if(ni.name.shared && (isFields ^ isMethods))
            ni.name.pair.setSecond(oldName);
          else if(ni.name.shared) {
//            String s = ni.name.pair.toString();
 //           newName = s.substring(s.lastIndexOf("@")+1);
            newName = sharedGen.getNewName();
            ni.name.pair.setSecond(newName);
          }
          else {
            newName = nameGen.getNewName();
            ni.name.pair.setSecond(newName);
          }
        }
      }
    }
  }

  private static boolean isSystemClass(String className) {
    if(className.startsWith("java/") || className.startsWith("javax/"))
      return true;

    return false;
  }

  public void printPackages() {
    System.out.println("\nPackages:");
    Enumeration e = packagesList.elements();
    while(e.hasMoreElements()) {
      Pair pair = (Pair)e.nextElement();

      String newName = (String)pair.getSecond();
      if(newName == null)
        System.out.println(pair);
      else if(newName.length() > 0)
        System.out.println(pair);
      else
        System.out.println(pair.getFirst() + " " + "<anonymous package>");
    }
  }

  public void printClasses() {
    System.out.println("\nClasses:");
    Enumeration e = classesList.elements();
    while(e.hasMoreElements()) {
      System.out.println(e.nextElement());
    }
  }

  public void printNames() {
    System.out.println("\nClass->Names:");
    namesList.print();
  }

  public void printInnerNames() {
    System.out.println("\nClass->InnerNames:");
    innerNamesList.print();
  }

  public void printParents() {
    System.out.println("\nClass->Parents:");
    classToParentsList.print();
  }

  public void printDependencies() {
    System.out.println("\nDependencies between classes:");
    Enumeration e = dependencies.elements();
    while(e.hasMoreElements()) {
      Pair pair = (Pair)e.nextElement();
      System.out.println(pair);
    }
  }

  public void printOtherClasses() {
    System.out.println("\nOther Classes:");
    Enumeration e = otherClassesList.elements();
    while(e.hasMoreElements()) {
      System.out.println(e.nextElement());
    }
  }

  public void print() {
    printPackages();
    printClasses();
    printOtherClasses();
    printParents();
    printNames();
    printInnerNames();
    printDependencies();
  }

  public synchronized void addNotificationListener(NotificationListener l) {
    support.addNotificationListener(l);
  }
  public synchronized void removeNotificationListener(NotificationListener l) {
    support.removeNotificationListener(l);
  }

  // fire NotificationEvent to registered listeners
  protected void fireNotification(String notification, String description) {
    support.fireNotification(notification, description);
  }

}


class NameElement implements ManyElement {
  class Name {
    Pair pair;
    boolean shared;
    boolean intacted;

    public String toString() {
      return pair.toString() + " " + shared + " " + intacted;
    }
  }

  Name name = new Name();

  public NameElement(Object first, boolean shared) {
    name.pair   = new Pair(first);
    name.shared = shared;
  }

  public Object getValue() { return name.pair.getFirst(); }

  public String toString() { return name.toString(); }

}

class Names extends DefaultManyContainer {

  public ManyElement put(ManyElement element) {
    int i = find(element);

    if(i == -1) {
      list.addElement(element);
      return element;
    }
    else {
      NameElement n = (NameElement)list.elementAt(i);
      n.name.shared = true;
      return n;
    }
  }

}

class NameRepository extends OneToManyTable {

  public ManyElement createManyElement(Object value) {
    return new NameElement(value, false); // not intacted
  }

  public ManyContainer createManyContainer() {
    return new Names();
  }

}

