// TaskDescription.java

package org.javalobby.task.obfuscator;

import java.io.*;
import java.util.*;

public class TaskDescription {

  private String from, to;

  private String classPath = System.getProperty("java.class.path");
  private String mainClass;

  private boolean anonymousMode = true;
  private boolean packageYes    = true;
  private boolean classYes      = true;
  private boolean fieldYes      = true;
  private boolean methodYes     = true;

  private Vector deniedList = new Vector();

  public TaskDescription() {}

  public void load(String fileName) throws IOException {
    BufferedReader reader = new BufferedReader(new FileReader(fileName));
    String line = null;
    while(true) {
      line = reader.readLine();
      if(line == null) break;
      line = line.trim();
      if(line.length() == 0) continue;

      if(line.startsWith("from:"))
        from = getRightPart(line);
      else if(line.startsWith("to:"))
        to = getRightPart(line);
      else if(line.startsWith("classpath:"))
        classPath = getRightPart(line);
      else if(line.startsWith("mainclass:"))
        mainClass = getRightPart(line);
      else if(line.startsWith("anonymous mode:"))
        anonymousMode = new Boolean(getRightPart(line)).booleanValue();
      else if(line.startsWith("packages:"))
        packageYes = new Boolean(getRightPart(line)).booleanValue();
      else if(line.startsWith("classes:"))
        classYes = new Boolean(getRightPart(line)).booleanValue();
      else if(line.startsWith("fields:"))
        fieldYes = new Boolean(getRightPart(line)).booleanValue();
      else if(line.startsWith("methods:"))
        methodYes = new Boolean(getRightPart(line)).booleanValue();
      else if(line.startsWith("denied:"))
        readDenied(reader);
    }

    reader.close();
  }

  private String getRightPart(String line) {
    int pos = line.indexOf(":");
    return line.substring(pos+1).trim();
  }

  private void readDenied(BufferedReader reader) throws IOException {
    String line = null;
    while(true) {
      line = reader.readLine();
      if(line == null) break;
      line = line.trim();
      if(line.length() == 0) continue;
      deniedList.addElement(line.replace('.', '/'));
    }
  }

  public void save(String fileName) throws IOException {
    PrintWriter writer = new PrintWriter(new BufferedWriter(
                             new FileWriter(fileName)));
    writer.println("from: "           + from);
    writer.println("to: "             + to);
    writer.println("classpath: "      + classPath);
    writer.println("mainclass: "      + mainClass);
    writer.println("anonymous mode: " + anonymousMode);
    writer.println("packages: "       + packageYes);
    writer.println("classes: "        + classYes);
    writer.println("fields: "         + fieldYes);
    writer.println("methods: "        + methodYes);

    writer.println("denied:");
    for(int i=0; i < deniedList.size(); i++) {
      writer.println(deniedList.elementAt(i));
    }

    writer.close();
  }

  public String getFrom() {
    return from;
  }

  public String getTo() {
    return to;
  }

  public String getClassPath() {
    return classPath;
  }

  public String getMainClass() { return
    mainClass;
  }

  public boolean getAnonymousMode() {
    return anonymousMode;
  }

  public boolean getPackagesYes() {
    return packageYes;
  }

  public boolean getClassesYes() {
    return classYes;
  }

  public boolean getFieldsYes() {
    return fieldYes;
  }

  public boolean getMethodsYes() {
    return methodYes;
  }

  public boolean[] getConditions() {
    return new boolean[] { packageYes, classYes, fieldYes, methodYes};
  }

  public Vector getDeniedList() {
    return deniedList;
  }

  public void setFrom(String from) {
    this.from = from;
  }

  public void setTo(String to) {
    this.to = to;
  }

  public void setClassPath(String classPath) {
    this.classPath = classPath;
  }

  public void setMainClass(String mainClass) {
    this.mainClass = mainClass;
  }

  public void setAnonymousMode(boolean anonymousMode) {
    this.anonymousMode = anonymousMode;
  }

  public void setConditions(boolean[] conditions) {
    packageYes = conditions[0];
    classYes   = conditions[1];
    fieldYes   = conditions[2];
    methodYes  = conditions[3];
  }

  public void setDeniedList(Vector deniedList) {
    this.deniedList =  deniedList;
  }


  public static void main(String[] argv) throws IOException {
    TaskDescription td = new TaskDescription();

    td.load(argv[0]);

    System.out.println("From      : " + td.getFrom());
    System.out.println("To        : " + td.getTo());
    System.out.println("ClassPath : " + td.getClassPath());
    System.out.println("mainClass : " + td.getMainClass());
    System.out.println("Anonymous : " + td.getAnonymousMode());
    System.out.println("Packages  : " + td.getPackagesYes());
    System.out.println("Classes   : " + td.getClassesYes());
    System.out.println("Fields    : " + td.getFieldsYes());
    System.out.println("Methods   : " + td.getMethodsYes());

    System.out.println("Denied list:");
    Vector deniedList = td.getDeniedList();
    for(int i=0; i < deniedList.size(); i++) {
      System.out.println(deniedList.elementAt(i));
    }

    td.save(argv[0] + "1");
  }

}