// StripTask.java

package org.javalobby.task.stripper;

import java.io.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.task.basic.*;

// task for removing additional information from class file

public class StripTask extends Task {

  private boolean rmUnknown, rmSrcFile, rmOptInfo;

  public StripTask(String to, boolean[] conditions) {
    super(to);
    rmUnknown = conditions[0];
    rmSrcFile = conditions[1];
    rmOptInfo = conditions[2];
  }

  public void body(ClassFile classFile) {
    if(rmUnknown || rmSrcFile)
      removeOptionalClassAttributes(classFile);

    if(rmOptInfo)
      removeOptionalMethodInfo(classFile);
  }

  private void removeOptionalClassAttributes(ClassFile classFile) {
    ConstPool constPool        = classFile.getConstPool();
    EntryCollection attributes = classFile.attributes();

    EntryCollection attributesClone = (EntryCollection)attributes.clone();
    short cnt = (short)attributesClone.size();

    for(short i=0; i < cnt; i++) {
      AttributeEntry attribute = (AttributeEntry)attributesClone.elementAt(i);

      String attrName = attribute.getName(constPool);
      if(rmSrcFile && attrName.equals(AttributeEntry.SOURCE_FILE)) {
        attributes.remove(attribute);
        constPool.modifyUTF(attribute.getNameIndex(), "");

        SourceFileAttribute sa = new SourceFileAttribute(attribute);
        constPool.modifyUTF(sa.getIndex(), "");
        fireNotification("", "Removing class attribute \"source file\"");
      }
      else if(rmUnknown && !attrName.equals(AttributeEntry.INNER_CLASSES) &&
              !attrName.equals(AttributeEntry.SOURCE_FILE)) {
        attributes.remove(attribute);
        constPool.modifyUTF(attribute.getNameIndex(), "");
        fireNotification("", "Removing class attribute \"" +
                         attribute.getName(constPool) + "\"");
      }
    }
  }

  private void removeOptionalMethodInfo(ClassFile classFile) {
    ConstPool constPool     = classFile.getConstPool();
    EntryCollection methods = classFile.methods();

    short cnt = (short)methods.size();

    for(short i=0; i < methods.size(); i++) {
      MethodEntry methodEntry = (MethodEntry)methods.elementAt(i);

      AttributeEntry attribute =
                     methodEntry.getAttribute(AttributeEntry.CODE, constPool);

      if(attribute != null) {
        CodeAttribute codeAttribute = new CodeAttribute(attribute);

        if(codeAttribute.clearCodeAttributes()) {
          String[] str = methodEntry.resolve(constPool);
          fireNotification(Converter.merge(str, " "),
                           "Removing attributes for code section");
        }
      }
    }
  }

}
