// LoaderClassAnalyzer.java

package org.javalobby.task.util;

import java.lang.reflect.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.classfile.*;
import org.javalobby.util.*;

public class LoaderClassAnalyzer implements ClassAnalyzer {

  private String currentClassName;
  private OneToManyTable classToParentsTable;
  private OneToManyTable classToNamesTable;

  private FileClassLoader loader;

  public LoaderClassAnalyzer(String dirName,
                             OneToManyTable classToParentsTable,
                             OneToManyTable classToNamesTable) {
    loader = new FileClassLoader(dirName);

    this.classToParentsTable = classToParentsTable;
    this.classToNamesTable   = classToNamesTable;
  }

  public OneToManyTable getParents() {
    return classToParentsTable;
  }

  public OneToManyTable getNames() {
    return classToNamesTable;
  }

  public void analyze(String className) {
    currentClassName = className.replace('.', '/');

    seek(currentClassName);
  }

  private void seek(String className) {
    try {
      Class clazz = Class.forName(className.replace('/', '.'));
      getSuperclass(clazz);
      getInterfaces(clazz);
      getNames(clazz);
    }
    catch(Exception e) {
      seek0(className);
    }
  }

  private void seek0(String className) {
    Class clazz = null;
    try {
      clazz = loader.loadClass(className.replace('/', '.'), true);
      getSuperclass(clazz);
      getInterfaces(clazz);
      getNames(clazz);
    }
    catch(Throwable e2) {
      System.err.println("Error reading class " + className);
    }
  }

  private void getSuperclass(Class clazz) {
    Class parentClass = clazz.getSuperclass();

    if(parentClass != null) {
      String parentClassName = parentClass.getName().replace('.', '/');
      classToParentsTable.put(currentClassName, parentClassName);
      seek(parentClassName);
    }
  }

  private void getInterfaces(Class clazz) {
    Class[] interfaces = clazz.getInterfaces();

    for(int i=0; i < interfaces.length; i++) {
      String interfaceName = interfaces[i].getName().replace('.', '/');
      classToParentsTable.put(currentClassName, interfaceName);
      getInterfaces(interfaces[i]);
    }
  }

  public void getNames(Class clazz) {
    String className = clazz.getName().replace('.', '/');

    Method methods[] = clazz.getDeclaredMethods();

    for(int i=0; i < methods.length; i++)
      classToNamesTable.put(className, methods[i].getName());

    Field fields[] = clazz.getDeclaredFields();

    for(int i=0; i < fields.length; i++)
      classToNamesTable.put(className, fields[i].getName());
  }

  public static void main(String[] args) {

    String dirName = "d:\\Work\\CafeBabe\\lib\\ppp";

    OneToManyTable classToParentsTable = new OneToManyTable();
    OneToManyTable classToNamesTable = new OneToManyTable();

    ClassAnalyzer analyzer = new LoaderClassAnalyzer(dirName,
                                 classToParentsTable, classToNamesTable);

    Vector v = new Vector();
    v.addElement("Slider");
/*    v.addElement("calculator.Expression");
    v.addElement("calculator.BinaryOperation");
    v.addElement("calculator.Calculable");
    v.addElement("calculator.CalculatorException");
    v.addElement("calculator.Constants");
    v.addElement("calculator.Divide");
    */

    Enumeration e = v.elements();
    while(e.hasMoreElements()) {
      String className = (String)e.nextElement();
      try {
        analyzer.analyze(className);
      }
      catch(Exception ex) {
        ex.printStackTrace();
        return;
      }

      System.out.println("Class->Parents:");
      classToParentsTable.print();

      System.out.println("Class->Names:");
      classToNamesTable.print();
    }
  }

}
