// JSplash - simple splash screen
// Copyright 2000, Tim O'Neil yadda yadda yadda
//
// This will draw the image pointed at by strLogoPath
// passed through the contrsuctor into a beveled box,
// which will hang around for 4 and 1/2 seconds, or
// will be dismissed by a mouse click.

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

class JSplash extends JFrame {
    Toolkit tk = Toolkit.getDefaultToolkit();
    JWindow win = new JWindow();
    JLabel lbl;
    private boolean bDone = false;

    public JSplash(String strLogoPath) {
        // We could just as well make this a url I suppose
       lbl = new JLabel(new ImageIcon(strLogoPath));

       // We'll be nice and give the user a chance to dismiss the
       // logo with a mouse click, so we need a mouse listener.
       win.addMouseListener(new MouseAdapter() {
          public void mousePressed(MouseEvent e) {
             bDone = true;
             win.dispose();
          }
        });

        // Start a timer for 5.5 seconds, if you wanted to
        // get fancy I guess you could pass this in the
        // constructor too
        Timer t = new Timer(5500, new TimerActionListener());
        // start the timer...
        t.start();
        // ...then start the action
        waitSplash();
    }

    class TimerActionListener implements ActionListener {
       public void actionPerformed(ActionEvent evt) {
          Timer timer = (Timer)evt.getSource();
          bDone = true;
          win.dispose();
       }
    }

    // The splash dialog has to be sync'd or the main process
    // will just race on by and start drawing itself before
    // the splash is done.
    public synchronized void waitSplash(){
       lbl.setBorder(BorderFactory.createRaisedBevelBorder());
       win.getContentPane().add(lbl, BorderLayout.CENTER);
       // See below for this code:
       centerWindow();
       // Actually draw the thing, and wait
       win.show();
          while(!bDone) {
             try {
                wait(5500);
           }
           catch(InterruptedException ie) {
              bDone = true;
           }
       }
       notifyAll(); // Do we really need this?
       return;
    }
    // Our own little center a window method
    private void centerWindow() {
       Dimension scrnSize = tk.getScreenSize();
       Dimension labelSize = lbl.getPreferredSize();
       int labelWidth = labelSize.width,
       labelHeight = labelSize.height;

       win.setLocation(scrnSize.width/2 - (labelWidth/2),
                    scrnSize.height/2 - (labelHeight/2));
       win.pack();
    }

  public static void main(String[] args) throws Exception {
    new JSplash("Icons/face1.jpg");
  }
}



