// ProgramState.java

package org.javalobby.util;

import java.io.*;
import java.util.*;
import java.awt.*;

// class that works with properties file for application

public class ProgramState extends Properties {

  private boolean existsFile = false;

  private String propsFileName;

  public ProgramState(String propsFileName) {
    this.propsFileName = propsFileName;
  }

  public boolean existsPropertiesFile() {
    return existsFile;
  }

  public void load() {
    File file = new File(propsFileName);

    if(file.exists()) {
      existsFile = true;
      try {
        load(new FileInputStream(file));
      }
      catch(IOException e) {
        e.printStackTrace();
      }
    }
    else
      existsFile = false;
  }

  public void save() {
    File file = new File(propsFileName);

    if(!file.canWrite()) {
      System.out.println("File " + propsFileName + " cannot be used for write operation.");
      return;
    }

    try {
      save(new FileOutputStream(file), null);
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }

  public void putProperty(String name, String defValue) {
    put(name, defValue);
  }

  public int getIntProperty(String name, int defValue) {
    return Integer.parseInt(getProperty(name, String.valueOf(defValue)));
  }

  public void putIntProperty(String name, int value) {
    put(name, String.valueOf(value));
  }

  public boolean getBooleanProperty(String name, boolean defValue) {
    String dv = String.valueOf(defValue);
    return Boolean.valueOf(getProperty(name, dv)).booleanValue();
  }

  public void putBooleanProperty(String name, boolean value) {
    put(name, new Boolean(value).toString());
  }

  public Rectangle getBoundsProperty(String name, String defValue) {
    String bs = getProperty(name, String.valueOf(defValue));
    StringTokenizer st = new StringTokenizer(bs, "; ");

    Rectangle r = new Rectangle(0, 0, 0, 0);

    if(st.hasMoreTokens()) {
      r.x = Integer.parseInt(st.nextToken());
    }

    if(st.hasMoreTokens()) {
      r.y = Integer.parseInt(st.nextToken());
    }

    if(st.hasMoreTokens()) {
      r.width  = Integer.parseInt(st.nextToken());
    }

    if(st.hasMoreTokens()) {
      r.height = Integer.parseInt(st.nextToken());
    }

    return r;
  }

  public void putBoundsProperty(String name, Rectangle r) {
    put(name, "" + r.x + ";" + r.y + ";" + r.width + ";" + r.height);
  }

}
